@@ -17,11 +17,18 @@
 #include "H5Exception.h"
 #include "H5IdComponent.h"
 #include "H5PropList.h"
+#include "H5FaccProp.h"
+#include "H5FcreatProp.h"
+#include "H5OcreatProp.h"
+#include "H5DcreatProp.h"
+#include "H5DxferProp.h"
+#include "H5LcreatProp.h"
 #include "H5LaccProp.h"
 #include "H5Location.h"
 #include "H5Object.h"
-#include "H5Alltypes.h"
+#include "H5DataSpace.h"
 #include "H5AbstractDs.h"
+#include "H5Alltypes.h"
 
 namespace H5 {
 
@@ -60,7 +67,7 @@ H5T_class_t AbstractDs::getTypeClass() const
     // which object invokes getTypeClass
     hid_t datatype_id;
     try {
-      datatype_id = p_get_type();  // returned value is already validated
+        datatype_id = p_get_type();  // returned value is already validated
     }
     catch (DataSetIException& E) {
         throw DataTypeIException("DataSet::getTypeClass", E.getDetailMsg());
@@ -78,7 +85,7 @@ H5T_class_t AbstractDs::getTypeClass() const
     {
         if (fromClass() == "DataSet")
             throw DataTypeIException("DataSet::getTypeClass", "H5Tclose failed");
-    else if (fromClass() == "Attribute")
+        else if (fromClass() == "Attribute")
             throw DataTypeIException("Attribute::getTypeClass", "H5Tclose failed");
     }
 
@@ -86,9 +93,9 @@ H5T_class_t AbstractDs::getTypeClass() const
     if (type_class == H5T_NO_CLASS)
     {
         if (fromClass() == "DataSet")
-        throw DataTypeIException("DataSet::getTypeClass", "H5Tget_class returns H5T_NO_CLASS");
-    else if (fromClass() == "Attribute")
-        throw DataTypeIException("Attribute::getTypeClass", "H5Tget_class returns H5T_NO_CLASS");
+            throw DataTypeIException("DataSet::getTypeClass", "H5Tget_class returns H5T_NO_CLASS");
+        else if (fromClass() == "Attribute")
+            throw DataTypeIException("Attribute::getTypeClass", "H5Tget_class returns H5T_NO_CLASS");
     }
     return(type_class);
 }
