@@ -18,7 +18,7 @@
 #endif
 #include <string>
 
-#include "H5private.h"          // for HDfree
+#include "H5private.h"             // for HDfree
 #include "H5Include.h"
 #include "H5Exception.h"
 #include "H5IdComponent.h"
@@ -27,6 +27,7 @@
 #include "H5FcreatProp.h"
 #include "H5OcreatProp.h"
 #include "H5DcreatProp.h"
+#include "H5LcreatProp.h"
 #include "H5LaccProp.h"
 #include "H5Location.h"
 #include "H5Object.h"
@@ -36,12 +37,10 @@
 #include "H5Attribute.h"
 
 namespace H5 {
-#ifndef H5_NO_STD
-    using std::cerr;
-    using std::endl;
-#endif  // H5_NO_STD
+using std::cerr;
+using std::endl;
 
-//class H5_DLLCPP H5Object;  // forward declaration for UserData4Aiterate
+class H5_DLLCPP H5Object;  // forward declaration for UserData4Aiterate
 
 //--------------------------------------------------------------------------
 // Function:    Attribute default constructor
@@ -52,7 +51,7 @@ Attribute::Attribute() : AbstractDs(), H5Location(), id(H5I_INVALID_HID) {}
 
 //--------------------------------------------------------------------------
 // Function:    Attribute copy constructor
-///\brief       Copy constructor: makes a copy of the original Attribute object.
+///\brief       Copy constructor: same HDF5 object as \a original
 ///\param       original  - IN: Original Attribute object to copy
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
@@ -84,8 +83,8 @@ Attribute::Attribute(const hid_t existing_id) : AbstractDs(), H5Location(), id(e
 //--------------------------------------------------------------------------
 void Attribute::write(const DataType& mem_type, const void *buf) const
 {
-    herr_t ret_value = H5Awrite(id, mem_type.getId(), buf);
-    if(ret_value < 0)
+   herr_t ret_value = H5Awrite(id, mem_type.getId(), buf);
+   if (ret_value < 0)
     {
         throw AttributeIException("Attribute::write", "H5Awrite failed");
     }
@@ -140,8 +139,8 @@ void Attribute::write(const DataType& mem_type, const H5std_string& strg) const
 //--------------------------------------------------------------------------
 void Attribute::read(const DataType& mem_type, void *buf) const
 {
-    herr_t ret_value = H5Aread(id, mem_type.getId(), buf);
-    if(ret_value < 0)
+   herr_t ret_value = H5Aread(id, mem_type.getId(), buf);
+   if (ret_value < 0)
     {
         throw AttributeIException("Attribute::read", "H5Aread failed");
     }
@@ -200,7 +199,7 @@ size_t Attribute::getInMemDataSize() const
 
     // Get the data type of this attribute
     hid_t mem_type_id = H5Aget_type(id);
-    if(mem_type_id < 0)
+    if (mem_type_id < 0)
     {
         throw AttributeIException(func, "H5Aget_type failed");
     }
@@ -261,39 +260,22 @@ size_t Attribute::getInMemDataSize() const
 //--------------------------------------------------------------------------
 DataSpace Attribute::getSpace() const
 {
-    // Calls C function H5Aget_space to get the id of the dataspace
-    hid_t dataspace_id = H5Aget_space(id);
+   // Calls C function H5Aget_space to get the id of the dataspace
+   hid_t dataspace_id = H5Aget_space(id);
 
-    // If the dataspace id is valid, create and return the DataSpace object
-    if(dataspace_id > 0)
+   // If the dataspace id is valid, create and return the DataSpace object
+   if (dataspace_id > 0)
     {
         DataSpace dataspace;
         f_DataSpace_setId(&dataspace, dataspace_id);
         return(dataspace);
     }
-    else
+   else
     {
         throw AttributeIException("Attribute::getSpace", "H5Aget_space failed");
     }
 }
 
-//--------------------------------------------------------------------------
-// Function:    Attribute::getFileName
-///\brief       Gets the name of the file, in which this attribute belongs.
-///\return      File name
-///\exception   H5::IdComponentException
-// Programmer   Binh-Minh Ribler - Jul, 2004
-//--------------------------------------------------------------------------
-H5std_string Attribute::getFileName() const
-{
-    try {
-        return(p_get_file_name());
-    }
-    catch (IdComponentException& E) {
-        throw FileIException("Attribute::getFileName", E.getDetailMsg());
-    }
-}
-
 //--------------------------------------------------------------------------
 // Function:    Attribute::getName
 ///\brief       Gets the name of this attribute, returning its length.
@@ -463,7 +445,7 @@ ssize_t Attribute::getName(H5std_string& attr_name, size_t len) const
 //--------------------------------------------------------------------------
 //ssize_t Attribute::getName(size_t len, H5std_string& attr_name) const
 //{
-//  return (getName(attr_name, len));
+//    return (getName(attr_name, len));
 //}
 
 //--------------------------------------------------------------------------
@@ -477,33 +459,8 @@ ssize_t Attribute::getName(H5std_string& attr_name, size_t len) const
 //--------------------------------------------------------------------------
 hsize_t Attribute::getStorageSize() const
 {
-    hsize_t storage_size = H5Aget_storage_size(id);
-    return (storage_size);
-}
-
-//--------------------------------------------------------------------------
-// Function:    Attribute::flush
-///\brief       Flushes all buffers associated with a file specified by
-///             this attribute, to disk.
-///\param       scope - IN: Specifies the scope of the flushing action,
-///             which can be either of these values:
-///             \li \c H5F_SCOPE_GLOBAL - Flushes the entire virtual file
-///             \li \c H5F_SCOPE_LOCAL - Flushes only the specified file
-///\exception   H5::AttributeIException
-///\par Description
-///             This attribute is used to identify the file to be flushed.
-// Programmer   Binh-Minh Ribler - 2013
-// Modification
-//      Mar 2013 - BMR
-//              Duplicated from H5Location
-//--------------------------------------------------------------------------
-void Attribute::flush(H5F_scope_t scope) const
-{
-    herr_t ret_value = H5Fflush(getId(), scope);
-    if(ret_value < 0)
-    {
-        throw AttributeIException("Attribute::flush", "H5Fflush failed");
-    }
+   hsize_t storage_size = H5Aget_storage_size(id);
+   return (storage_size);
 }
 
 //--------------------------------------------------------------------------
@@ -516,10 +473,15 @@ void Attribute::flush(H5F_scope_t scope) const
 //              addition, member IdComponent::id is moved into subclasses, and
 //              IdComponent::getId now becomes pure virtual function.
 // Programmer   Binh-Minh Ribler - May, 2008
+// Modification
+//      Aug 2016 - BMR
+//              Note that Attribute is now inheriting from H5Location, because
+//              an attribute id can be used to specify a location in HDF5
+//              library.
 //--------------------------------------------------------------------------
 hid_t Attribute::getId() const
 {
-    return(id);
+   return(id);
 }
 
 //--------------------------------------------------------------------------
@@ -534,7 +496,7 @@ hid_t Attribute::getId() const
 hid_t Attribute::p_get_type() const
 {
     hid_t type_id = H5Aget_type(id);
-    if(type_id > 0)
+    if (type_id > 0)
         return(type_id);
     else
     {
@@ -544,10 +506,10 @@ hid_t Attribute::p_get_type() const
 
 //--------------------------------------------------------------------------
 // Function:    Attribute::p_read_fixed_len (private)
-// brief        Reads a fixed length \a H5std_string from an attribute.
-// param        mem_type  - IN: Attribute datatype (in memory)
-// param        strg      - IN: Buffer for read string
-// exception    H5::AttributeIException
+// brief       Reads a fixed length \a H5std_string from an attribute.
+// param       mem_type  - IN: Attribute datatype (in memory)
+// param       strg      - IN: Buffer for read string
+// exception   H5::AttributeIException
 // Programmer   Binh-Minh Ribler - Jul, 2009
 // Modification
 //      Jul 2009
@@ -566,9 +528,9 @@ void Attribute::p_read_fixed_len(const DataType& mem_type, H5std_string& strg) c
     {
         char *strg_C = new char[attr_size+1];
         herr_t ret_value = H5Aread(id, mem_type.getId(), strg_C);
-        if(ret_value < 0)
+        if (ret_value < 0)
         {
-            delete []strg_C;    // de-allocate for fixed-len string
+            delete []strg_C;        // de-allocate for fixed-len string
             throw AttributeIException("Attribute::read", "H5Aread failed");
         }
         // Get string from the C char* and release resource allocated locally
@@ -580,13 +542,13 @@ void Attribute::p_read_fixed_len(const DataType& mem_type, H5std_string& strg) c
 
 //--------------------------------------------------------------------------
 // Function:    Attribute::p_read_variable_len (private)
-// brief        Reads a variable length \a H5std_string from an attribute.
-// param        mem_type  - IN: Attribute datatype (in memory)
-// param        strg      - IN: Buffer for read string
-// exception    H5::AttributeIException
+// brief       Reads a variable length \a H5std_string from an attribute.
+// param       mem_type  - IN: Attribute datatype (in memory)
+// param       strg      - IN: Buffer for read string
+// exception   H5::AttributeIException
 // Programmer   Binh-Minh Ribler - Jul, 2009
 // Modification
-//      Jul 2009
+//        Jul 2009
 //              Separated the variable length case from the original
 //              Attribute::read. -BMR
 //--------------------------------------------------------------------------
@@ -598,7 +560,7 @@ void Attribute::p_read_variable_len(const DataType& mem_type, H5std_string& strg
     // Read attribute, no allocation for variable-len string; C library will
     herr_t ret_value = H5Aread(id, mem_type.getId(), &strg_C);
 
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw AttributeIException("Attribute::read", "H5Aread failed");
     }
@@ -647,7 +609,7 @@ void Attribute::close()
     if (p_valid_id(id))
     {
         herr_t ret_value = H5Aclose(id);
-        if(ret_value < 0)
+        if (ret_value < 0)
         {
             throw AttributeIException("Attribute::close", "H5Aclose failed");
         }
@@ -675,4 +637,5 @@ Attribute::~Attribute()
         cerr << "Attribute::~Attribute - " << close_error.getDetailMsg() << endl;
     }
 }
+
 } // end namespace
