@@ -17,12 +17,15 @@
 #include "H5Exception.h"
 #include "H5IdComponent.h"
 #include "H5PropList.h"
+#include "H5OcreatProp.h"
+#include "H5DcreatProp.h"
+#include "H5DxferProp.h"
+#include "H5LcreatProp.h"
 #include "H5LaccProp.h"
 #include "H5Location.h"
 #include "H5Object.h"
 #include "H5Alltypes.h"
 #include "H5AbstractDs.h"
-#include "H5DxferProp.h"
 #include "H5DataSpace.h"
 #include "H5DataSet.h"
 
@@ -37,7 +40,7 @@ CompType::CompType() : DataType() {}
 
 //--------------------------------------------------------------------------
 // Function:    CompType copy constructor
-///\brief       Copy constructor: makes copy of the original CompType object
+///\brief       Copy constructor: same HDF5 object as \a original
 ///\param       original - IN: Original CompType instance
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
@@ -77,12 +80,71 @@ CompType::CompType(const DataSet& dataset) : DataType()
     id = H5Dget_type(dataset.getId());
 
     // If the datatype id is invalid, throw exception
-    if(id < 0)
+    if (id < 0)
     {
         throw DataSetIException("CompType constructor", "H5Dget_type failed");
     }
 }
 
+//--------------------------------------------------------------------------
+// Function:    CompType overloaded constructor
+///\brief       Creates an CompType instance by opening an HDF5 compound
+///             given its name, provided as a C character string.
+///\param       loc        - IN: Location of the type
+///\param       dtype_name - IN: Compound type name
+///\exception   H5::DataTypeIException
+// Programmer   Binh-Minh Ribler - Dec 2016
+// Description
+//              In 1.10.1, this constructor was introduced and may replace the
+//              existing function CommonFG::openCompType(const char*) to
+//              improve usability.
+//              -BMR, Dec 2016
+//--------------------------------------------------------------------------
+CompType::CompType(const H5Location& loc, const char *dtype_name) : DataType()
+{
+    id = p_opentype(loc, dtype_name);
+}
+
+//--------------------------------------------------------------------------
+// Function:    CompType overloaded constructor
+///\brief       Creates an CompType instance by opening an HDF5 compound
+///             datatype given its name, provided as an \c H5std_string.
+///\param       loc        - IN: Location of the type
+///\param       dtype_name - IN: Compound type name
+///\exception   H5::DataTypeIException
+// Programmer   Binh-Minh Ribler - Dec 2016
+// Description
+//              In 1.10.1, this constructor was introduced and may replace the
+//              existing function CommonFG::openCompType(const H5Location&)
+//              to improve usability.
+//              -BMR, Dec 2016
+//--------------------------------------------------------------------------
+CompType::CompType(const H5Location& loc, const H5std_string& dtype_name) : DataType()
+{
+    id = p_opentype(loc, dtype_name.c_str());
+}
+
+//--------------------------------------------------------------------------
+// Function:    CompType::decode
+///\brief       Returns a CompType object via DataType* by decoding the
+///             binary object description of this datatype.
+///\exception   H5::DataTypeIException
+// Programmer   Binh-Minh Ribler - Aug 2017
+//--------------------------------------------------------------------------
+DataType* CompType::decode() const
+{
+    hid_t encoded_cmptype_id = H5I_INVALID_HID;
+    try {
+        encoded_cmptype_id = p_decode();
+    }
+    catch (DataTypeIException &err) {
+        throw;
+    }
+    CompType *encoded_cmptype = new CompType;
+    encoded_cmptype->p_setId(encoded_cmptype_id);
+    return(encoded_cmptype);
+}
+
 //--------------------------------------------------------------------------
 // Function:    CompType::getNmembers
 ///\brief       Returns the number of members in this compound datatype.
@@ -93,10 +155,10 @@ CompType::CompType(const DataSet& dataset) : DataType()
 int CompType::getNmembers() const
 {
     int num_members = H5Tget_nmembers(id);
-    if(num_members < 0)
+    if (num_members < 0)
     {
         throw DataTypeIException("CompType::getNmembers",
-                "H5Tget_nmembers returns negative number of members");
+            "H5Tget_nmembers returns negative number of members");
     }
     return(num_members);
 }
@@ -112,10 +174,10 @@ int CompType::getNmembers() const
 H5std_string CompType::getMemberName(unsigned member_num) const
 {
     char* member_name_C = H5Tget_member_name(id, member_num);
-    if(member_name_C == NULL)  // NULL means failure
+    if (member_name_C == NULL)  // NULL means failure
     {
         throw DataTypeIException("CompType::getMemberName",
-                "H5Tget_member_name returns NULL for member name");
+             "H5Tget_member_name returns NULL for member name");
     }
     H5std_string member_name = H5std_string(member_name_C); // convert C string to string
     H5free_memory(member_name_C); // free the C string
@@ -137,10 +199,10 @@ H5std_string CompType::getMemberName(unsigned member_num) const
 int CompType::getMemberIndex(const char* name) const
 {
     int member_index = H5Tget_member_index(id, name);
-    if(member_index < 0)
+    if (member_index < 0)
     {
         throw DataTypeIException("CompType::getMemberIndex",
-                "H5Tget_member_index returns negative value");
+             "H5Tget_member_index returns negative value");
     }
     return(member_index);
 }
@@ -155,7 +217,6 @@ int CompType::getMemberIndex(const H5std_string& name) const
 ///             respect to the beginning of the compound data type datum.
 ///\param       member_num - IN: Zero-based index of the member
 ///\return      Byte offset
-///\exception   H5::DataTypeIException
 // Programmer   Binh-Minh Ribler - 2000
 // Description
 ///             Members are stored in no particular order with numbers 0
@@ -184,10 +245,10 @@ size_t CompType::getMemberOffset(unsigned member_num) const
 H5T_class_t CompType::getMemberClass(unsigned member_num) const
 {
     H5T_class_t member_class = H5Tget_member_class(id, member_num);
-    if(member_class == H5T_NO_CLASS)
+    if (member_class == H5T_NO_CLASS)
     {
         throw DataTypeIException("CompType::getMemberClass",
-                "H5Tget_member_class returns H5T_NO_CLASS");
+             "H5Tget_member_class returns H5T_NO_CLASS");
     }
     return(member_class);
 }
@@ -200,7 +261,7 @@ hid_t CompType::p_get_member_type(unsigned member_num) const
 {
     // get the id of the specified member first
     hid_t member_type_id = H5Tget_member_type(id, member_num);
-    if(member_type_id > 0)
+    if (member_type_id > 0)
         return(member_type_id);
     else
     {
@@ -243,7 +304,7 @@ DataType CompType::getMemberDataType(unsigned member_num) const
 ArrayType CompType::getMemberArrayType(unsigned member_num) const
 {
     try {
-        ArrayType arraytype(p_get_member_type(member_num));
+        ArrayType arraytype;
         f_DataType_setId(&arraytype, p_get_member_type(member_num));
         return(arraytype);
     }
@@ -264,7 +325,7 @@ ArrayType CompType::getMemberArrayType(unsigned member_num) const
 CompType CompType::getMemberCompType(unsigned member_num) const
 {
     try {
-      CompType comptype(p_get_member_type(member_num));
+        CompType comptype;
         f_DataType_setId(&comptype, p_get_member_type(member_num));
         return(comptype);
     }
@@ -285,7 +346,7 @@ CompType CompType::getMemberCompType(unsigned member_num) const
 EnumType CompType::getMemberEnumType(unsigned member_num) const
 {
     try {
-      EnumType enumtype(p_get_member_type(member_num));
+        EnumType enumtype;
         f_DataType_setId(&enumtype, p_get_member_type(member_num));
         return(enumtype);
     }
@@ -306,7 +367,7 @@ EnumType CompType::getMemberEnumType(unsigned member_num) const
 IntType CompType::getMemberIntType(unsigned member_num) const
 {
     try {
-      IntType inttype(p_get_member_type(member_num));
+        IntType inttype;
         f_DataType_setId(&inttype, p_get_member_type(member_num));
         return(inttype);
     }
@@ -327,7 +388,7 @@ IntType CompType::getMemberIntType(unsigned member_num) const
 FloatType CompType::getMemberFloatType(unsigned member_num) const
 {
     try {
-      FloatType floatype(p_get_member_type(member_num));
+        FloatType floatype;
         f_DataType_setId(&floatype, p_get_member_type(member_num));
         return(floatype);
     }
@@ -348,7 +409,7 @@ FloatType CompType::getMemberFloatType(unsigned member_num) const
 StrType CompType::getMemberStrType(unsigned member_num) const
 {
     try {
-      StrType strtype(p_get_member_type(member_num));
+        StrType strtype;
         f_DataType_setId(&strtype, p_get_member_type(member_num));
         return(strtype);
     }
@@ -369,7 +430,7 @@ StrType CompType::getMemberStrType(unsigned member_num) const
 VarLenType CompType::getMemberVarLenType(unsigned member_num) const
 {
     try {
-      VarLenType varlentype(p_get_member_type(member_num));
+        VarLenType varlentype;
         f_DataType_setId(&varlentype, p_get_member_type(member_num));
         return(varlentype);
     }
@@ -379,7 +440,7 @@ VarLenType CompType::getMemberVarLenType(unsigned member_num) const
 }
 
 /* old style of getMemberType - using overloads; new style above
-    returns the appropriate datatypes but has different named functions.
+   returns the appropriate datatypes but has different named functions.
    In the old style, a datatype must be passed into the function.
    May, 2004: These should be reconsidered to provide more convenience.
 // Returns the datatype of the specified member in this compound datatype.
@@ -430,7 +491,7 @@ void CompType::insertMember(const H5std_string& name, size_t offset, const DataT
 
     // Call C routine H5Tinsert to add the new member
     herr_t ret_value = H5Tinsert(id, name_C, offset, new_member_id);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataTypeIException("CompType::insertMember", "H5Tinsert failed");
     }
@@ -447,7 +508,7 @@ void CompType::pack() const
 {
     // Calls C routine H5Tpack to remove padding
     herr_t ret_value = H5Tpack(id);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataTypeIException("CompType::pack", "H5Tpack failed");
     }
