@@ -18,17 +18,20 @@
 #endif
 #include <string>
 
-#include "H5private.h"          // for HDfree and HDmemset
+#include "H5private.h"             // for HDfree
 #include "H5Include.h"
 #include "H5Exception.h"
 #include "H5IdComponent.h"
 #include "H5PropList.h"
-#include "H5LaccProp.h"
-#include "H5Location.h"
-#include "H5Object.h"
+#include "H5FaccProp.h"
+#include "H5FcreatProp.h"
 #include "H5OcreatProp.h"
 #include "H5DxferProp.h"
 #include "H5DcreatProp.h"
+#include "H5LcreatProp.h"
+#include "H5LaccProp.h"
+#include "H5Location.h"
+#include "H5Object.h"
 #include "H5DataType.h"
 #include "H5DataSpace.h"
 #include "H5AbstractDs.h"
@@ -36,10 +39,8 @@
 #include "H5DataSet.h"
 
 namespace H5 {
-#ifndef H5_NO_STD
-    using std::cerr;
-    using std::endl;
-#endif  // H5_NO_STD
+using std::cerr;
+using std::endl;
 
 //--------------------------------------------------------------------------
 // Function:    DataSet default constructor
@@ -67,7 +68,7 @@ DataSet::DataSet(const hid_t existing_id) : H5Object(), AbstractDs(), id(existin
 
 //--------------------------------------------------------------------------
 // Function:    DataSet copy constructor
-///\brief       Copy constructor: makes a copy of the original DataSet object.
+///\brief       Copy constructor: same HDF5 object as \a original
 ///\param       original - IN: DataSet instance to copy
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
@@ -84,18 +85,16 @@ DataSet::DataSet(const DataSet& original) : H5Object(), AbstractDs(), id(origina
 ///                       object that the dataset is located within.
 ///\param       ref - IN: Reference pointer
 ///\param       ref_type - IN: Reference type - default to H5R_OBJECT
+///\param       plist - IN: Property list - default to PropList::DEFAULT
 ///\exception   H5::DataSetIException
 ///\par Description
 ///             \c loc can be DataSet, Group, H5File, or named DataType, that
 ///             is a datatype that has been named by DataType::commit.
 // Programmer   Binh-Minh Ribler - Oct, 2006
-// Modification
-//      Jul, 2008
-//              Added for application convenience.
 //--------------------------------------------------------------------------
-DataSet::DataSet(const H5Location& loc, const void* ref, H5R_type_t ref_type) : AbstractDs(), H5Object(), id(H5I_INVALID_HID)
+DataSet::DataSet(const H5Location& loc, const void* ref, H5R_type_t ref_type, const PropList& plist) : H5Object(), AbstractDs(), id(H5I_INVALID_HID)
 {
-    id = H5Location::p_dereference(loc.getId(), ref, ref_type, "constructor - by dereferenced");
+    id = H5Location::p_dereference(loc.getId(), ref, ref_type, plist, "constructor - by dereferenced");
 }
 
 //--------------------------------------------------------------------------
@@ -105,15 +104,13 @@ DataSet::DataSet(const H5Location& loc, const void* ref, H5R_type_t ref_type) :
 ///\param       attr - IN: Specifying location where the referenced object is in
 ///\param       ref - IN: Reference pointer
 ///\param       ref_type - IN: Reference type - default to H5R_OBJECT
+///\param       plist - IN: Property list - default to PropList::DEFAULT
 ///\exception   H5::ReferenceException
 // Programmer   Binh-Minh Ribler - Oct, 2006
-// Modification
-//      Jul, 2008
-//              Added for application convenience.
 //--------------------------------------------------------------------------
-DataSet::DataSet(const Attribute& attr, const void* ref, H5R_type_t ref_type) : AbstractDs(), H5Object(), id(H5I_INVALID_HID)
+DataSet::DataSet(const Attribute& attr, const void* ref, H5R_type_t ref_type, const PropList& plist) : H5Object(), AbstractDs(), id(H5I_INVALID_HID)
 {
-    id = H5Location::p_dereference(attr.getId(), ref, ref_type, "constructor - by dereference");
+    id = H5Location::p_dereference(attr.getId(), ref, ref_type, plist, "constructor - by dereference");
 }
 
 //--------------------------------------------------------------------------
@@ -129,7 +126,7 @@ DataSpace DataSet::getSpace() const
     hid_t dataspace_id = H5Dget_space(id);
 
     // If the dataspace id is invalid, throw an exception
-    if(dataspace_id < 0)
+    if (dataspace_id < 0)
     {
         throw DataSetIException("DataSet::getSpace", "H5Dget_space failed");
     }
@@ -145,7 +142,7 @@ DataSpace DataSet::getSpace() const
 hid_t DataSet::p_get_type() const
 {
     hid_t type_id = H5Dget_type(id);
-    if(type_id > 0)
+    if (type_id > 0)
         return(type_id);
     else
     {
@@ -163,7 +160,7 @@ hid_t DataSet::p_get_type() const
 DSetCreatPropList DataSet::getCreatePlist() const
 {
     hid_t create_plist_id = H5Dget_create_plist(id);
-    if(create_plist_id < 0)
+    if (create_plist_id < 0)
     {
         throw DataSetIException("DataSet::getCreatePlist", "H5Dget_create_plist failed");
     }
@@ -202,7 +199,7 @@ size_t DataSet::getInMemDataSize() const
 
     // Get the data type of this dataset
     hid_t mem_type_id = H5Dget_type(id);
-    if(mem_type_id < 0)
+    if (mem_type_id < 0)
     {
         throw DataSetIException(func, "H5Dget_type failed");
     }
@@ -266,7 +263,7 @@ haddr_t DataSet::getOffset() const
     haddr_t ds_addr; // for address of dataset
 
     ds_addr = H5Dget_offset(id);
-    if(ds_addr == HADDR_UNDEF)
+    if (ds_addr == HADDR_UNDEF)
     {
         throw DataSetIException("DataSet::getOffset", "H5Dget_offset returned HADDR_UNDEF");
     }
@@ -283,7 +280,7 @@ haddr_t DataSet::getOffset() const
 void DataSet::getSpaceStatus(H5D_space_status_t& status) const
 {
     herr_t ret_value = H5Dget_space_status(id, &status);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataSetIException("DataSet::getSpaceStatus", "H5Dget_space_status failed");
     }
@@ -297,8 +294,6 @@ void DataSet::getSpaceStatus(H5D_space_status_t& status) const
 ///\return      Amount of storage
 ///\exception   H5::DataSetIException
 // Programmer   Binh-Minh Ribler - 2000
-// Modification
-//              Replaced the version without const parameter - Apr, 2014
 //--------------------------------------------------------------------------
 hsize_t DataSet::getVlenBufSize(const DataType& type, const DataSpace& space) const
 {
@@ -309,7 +304,7 @@ hsize_t DataSet::getVlenBufSize(const DataType& type, const DataSpace& space) co
     hsize_t size; // for amount of storage
 
     herr_t ret_value = H5Dvlen_get_buf_size(id, type_id, space_id, &size);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataSetIException("DataSet::getVlenBufSize", "H5Dvlen_get_buf_size failed");
     }
@@ -331,7 +326,7 @@ hsize_t DataSet::getVlenBufSize(const DataType& type, const DataSpace& space) co
 //--------------------------------------------------------------------------
 //hsize_t DataSet::getVlenBufSize(DataType& type, DataSpace& space) const
 //{
-//  return(getVlenBufSize(type, space));
+//    return(getVlenBufSize(type, space));
 //}
 
 //--------------------------------------------------------------------------
@@ -353,7 +348,7 @@ void DataSet::vlenReclaim(const DataType& type, const DataSpace& space, const DS
     hid_t xfer_plist_id = xfer_plist.getId();
 
     herr_t ret_value = H5Dvlen_reclaim(type_id, space_id, xfer_plist_id, buf);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataSetIException("DataSet::vlenReclaim", "H5Dvlen_reclaim failed");
     }
@@ -411,7 +406,7 @@ void DataSet::read(void* buf, const DataType& mem_type, const DataSpace& mem_spa
     hid_t xfer_plist_id = xfer_plist.getId();
 
     herr_t ret_value = H5Dread(id, mem_type_id, mem_space_id, file_space_id, xfer_plist_id, buf);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataSetIException("DataSet::read", "H5Dread failed");
     }
@@ -433,8 +428,8 @@ void DataSet::read(void* buf, const DataType& mem_type, const DataSpace& mem_spa
 //              Follow the change to Attribute::read and use the following
 //              private functions to read datasets with fixed- and
 //              variable-length string:
-//                      DataSet::p_read_fixed_len and
-//                      DataSet::p_read_variable_len
+//                  DataSet::p_read_fixed_len and
+//                  DataSet::p_read_variable_len
 //--------------------------------------------------------------------------
 void DataSet::read(H5std_string& strg, const DataType& mem_type, const DataSpace& mem_space, const DataSpace& file_space, const DSetMemXferPropList& xfer_plist) const
 {
@@ -487,7 +482,7 @@ void DataSet::write(const void* buf, const DataType& mem_type, const DataSpace&
     hid_t xfer_plist_id = xfer_plist.getId();
 
     herr_t ret_value = H5Dwrite(id, mem_type_id, mem_space_id, file_space_id, xfer_plist_id, buf);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataSetIException("DataSet::write", "H5Dwrite failed");
     }
@@ -563,7 +558,7 @@ int DataSet::iterateElems(void* buf, const DataType& type, const DataSpace& spac
     hid_t type_id = type.getId();
     hid_t space_id = space.getId();
     herr_t ret_value = H5Diterate(buf, type_id, space_id, op, op_data);
-    if(ret_value >= 0)
+    if (ret_value >= 0)
         return(ret_value);
     else  // raise exception when H5Diterate returns a negative value
     {
@@ -577,16 +572,14 @@ int DataSet::iterateElems(void* buf, const DataType& type, const DataSpace& spac
 ///\param       size - IN: Array containing the new magnitude of each dimension
 ///\exception   H5::DataSetIException
 ///\par Description
-///             For more information, please see the Description section in
-///             C layer Reference Manual at:
-///\par
-/// https://support.hdfgroup.org/HDF5/doc/RM/RM_H5D.html#Dataset-Extend
+///             For information, please refer to the H5Dset_extent API in
+///             the HDF5 C Reference Manual.
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DataSet::extend(const hsize_t* size) const
 {
     herr_t ret_value = H5Dset_extent(id, size);
-    if(ret_value < 0)  // raise exception when H5Dset_extent returns a neg value
+    if (ret_value < 0)  // raise exception when H5Dset_extent returns a neg value
         throw DataSetIException("DataSet::extend", "H5Dset_extent failed");
 }
 
@@ -600,7 +593,6 @@ void DataSet::extend(const hsize_t* size) const
 ///\param       space - IN: Dataspace describing memory buffer & containing selection to use
 ///\exception   H5::DataSetIException
 // Programmer   Binh-Minh Ribler - 2014
-// Modification
 //--------------------------------------------------------------------------
 void DataSet::fillMemBuf(const void *fill, const DataType& fill_type, void *buf, const DataType& buf_type, const DataSpace& space) const
 {
@@ -608,7 +600,7 @@ void DataSet::fillMemBuf(const void *fill, const DataType& fill_type, void *buf,
     hid_t buf_type_id = buf_type.getId();
     hid_t space_id = space.getId();
     herr_t ret_value = H5Dfill(fill, fill_type_id, buf, buf_type_id, space_id);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataSetIException("DataSet::fillMemBuf", "H5Dfill failed");
     }
@@ -644,15 +636,13 @@ void DataSet::fillMemBuf(const void *fill, const DataType& fill_type, void *buf,
 ///\param       space - IN: Dataspace describing memory buffer & containing selection to use
 ///\exception   H5::DataSetIException
 // Programmer   Binh-Minh Ribler - 2000
-// Modification
-//              Replaced the version without const parameter - Apr, 2014
 //--------------------------------------------------------------------------
 void DataSet::fillMemBuf(void *buf, const DataType& buf_type, const DataSpace& space) const
 {
     hid_t buf_type_id = buf_type.getId();
     hid_t space_id = space.getId();
     herr_t ret_value = H5Dfill(NULL, buf_type_id, buf, buf_type_id, space_id);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataSetIException("DataSet::fillMemBuf", "H5Dfill failed");
     }
@@ -696,10 +686,10 @@ hid_t DataSet::getId() const
 
 //--------------------------------------------------------------------------
 // Function:    DataSet::p_read_fixed_len (private)
-// brief        Reads a fixed length \a H5std_string from a dataset.
-// param        mem_type  - IN: DataSet datatype (in memory)
-// param        strg      - IN: Buffer for read string
-// exception    H5::DataSetIException
+// brief       Reads a fixed length \a H5std_string from a dataset.
+// param       mem_type  - IN: DataSet datatype (in memory)
+// param       strg      - IN: Buffer for read string
+// exception   H5::DataSetIException
 // Programmer   Binh-Minh Ribler - Jul, 2009
 // Modification
 //      Jul 2009
@@ -720,7 +710,7 @@ void DataSet::p_read_fixed_len(const hid_t mem_type_id, const hid_t mem_space_id
 
         herr_t ret_value = H5Dread(id, mem_type_id, mem_space_id, file_space_id, xfer_plist_id, strg_C);
 
-        if(ret_value < 0)
+        if (ret_value < 0)
         {
             delete []strg_C;    // de-allocate for fixed-len string
             throw DataSetIException("DataSet::read", "H5Dread failed for fixed length string");
@@ -734,10 +724,10 @@ void DataSet::p_read_fixed_len(const hid_t mem_type_id, const hid_t mem_space_id
 
 //--------------------------------------------------------------------------
 // Function:    DataSet::p_read_variable_len (private)
-// brief        Reads a variable length \a H5std_string from an dataset.
-// param        mem_type  - IN: DataSet datatype (in memory)
-// param        strg      - IN: Buffer for read string
-// exception    H5::DataSetIException
+// brief       Reads a variable length \a H5std_string from an dataset.
+// param       mem_type  - IN: DataSet datatype (in memory)
+// param       strg      - IN: Buffer for read string
+// exception   H5::DataSetIException
 // Programmer   Binh-Minh Ribler - Jul, 2009
 // Modification
 //      Jul 2009
@@ -751,7 +741,7 @@ void DataSet::p_read_variable_len(const hid_t mem_type_id, const hid_t mem_space
     // Read dataset, no allocation for variable-len string; C library will
     herr_t ret_value = H5Dread(id, mem_type_id, mem_space_id, file_space_id, xfer_plist_id, &strg_C);
 
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataSetIException("DataSet::read", "H5Dread failed for variable length string");
     }
@@ -783,8 +773,8 @@ void DataSet::p_setId(const hid_t new_id)
     catch (Exception& close_error) {
         throw DataSetIException(inMemFunc("p_setId"), close_error.getDetailMsg());
     }
-    // reset object's id to the given id
-    id = new_id;
+   // reset object's id to the given id
+   id = new_id;
 }
 
 //--------------------------------------------------------------------------
@@ -816,7 +806,7 @@ void DataSet::close()
     if (p_valid_id(id))
     {
         herr_t ret_value = H5Dclose(id);
-        if(ret_value < 0)
+        if (ret_value < 0)
         {
             throw DataSetIException("DataSet::close", "H5Dclose failed");
         }
@@ -844,4 +834,5 @@ DataSet::~DataSet()
         cerr << "DataSet::~DataSet - " << close_error.getDetailMsg() << endl;
     }
 }
+
 } // end namespace
