@@ -18,28 +18,31 @@
 #endif
 #include <string>
 
-#include "H5private.h"          // for HDcalloc
 #include "H5Include.h"
 #include "H5Exception.h"
 #include "H5IdComponent.h"
+#include "H5DataSpace.h"
 #include "H5PropList.h"
+#include "H5FaccProp.h"
+#include "H5FcreatProp.h"
+#include "H5OcreatProp.h"
+#include "H5DcreatProp.h"
+#include "H5DxferProp.h"
+#include "H5LcreatProp.h"
 #include "H5LaccProp.h"
 #include "H5Location.h"
 #include "H5Object.h"
-#include "H5DxferProp.h"
 #include "H5DataType.h"
 #include "H5AtomType.h"
 #include "H5PredType.h"
+#include "H5private.h"
 #include "H5AbstractDs.h"
-#include "H5DataSpace.h"
 #include "H5DataSet.h"
 #include "H5Attribute.h"
 
 namespace H5 {
-#ifndef H5_NO_STD
-    using std::cerr;
-    using std::endl;
-#endif  // H5_NO_STD
+using std::cerr;
+using std::endl;
 
 //--------------------------------------------------------------------------
 // Function:    DataType default constructor
@@ -57,7 +60,7 @@ DataType::DataType() : H5Object(), id(H5I_INVALID_HID), encoded_buf(NULL), buf_s
 //              its id.
 // Programmer   Binh-Minh Ribler - 2000
 // Modification
-//      Dec, 2005
+//        Dec, 2005
 //              Removed second argument, "predefined", after changing to the
 //              new ref counting mechanism that relies on C's ref counting.
 //--------------------------------------------------------------------------
@@ -78,7 +81,7 @@ DataType::DataType(const H5T_class_t type_class, size_t size) : H5Object(), enco
 {
     // Call C routine to create the new datatype
     id = H5Tcreate(type_class, size);
-    if(id < 0)
+    if (id < 0)
     {
         throw DataTypeIException("DataType constructor", "H5Tcreate failed");
     }
@@ -91,38 +94,38 @@ DataType::DataType(const H5T_class_t type_class, size_t size) : H5Object(), enco
 ///\param       loc - IN: Location referenced object is in
 ///\param       ref - IN: Reference pointer
 ///\param       ref_type - IN: Reference type - default to H5R_OBJECT
+///\param       plist - IN: Property list - default to PropList::DEFAULT
 ///\exception   H5::ReferenceException
 // Programmer   Binh-Minh Ribler - Oct, 2006
-// Modification
-//      Jul, 2008
-//              Added for application convenience.
 //--------------------------------------------------------------------------
-DataType::DataType(const H5Location& loc, const void* ref, H5R_type_t ref_type) : H5Object(), encoded_buf(NULL), buf_size(0)
+DataType::DataType(const H5Location& loc, const void* ref, H5R_type_t ref_type, const PropList& plist) : H5Object(), encoded_buf(NULL), buf_size(0)
 {
-    id = H5Location::p_dereference(loc.getId(), ref, ref_type, "constructor - by dereference");
+    id = H5Location::p_dereference(loc.getId(), ref, ref_type, plist, "constructor - by dereference");
 }
 
 //--------------------------------------------------------------------------
 // Function:    DataType overload constructor - dereference
-///\brief       Given a reference, ref, to an hdf5 group, creates a
-///             DataType object
-///\param       attr - IN: Specifying location where the referenced object is in
-///\param       ref - IN: Reference pointer
-///\param       ref_type - IN: Reference type - default to H5R_OBJECT
-///\exception   H5::ReferenceException
+// brief       Given a reference, ref, to an hdf5 group, creates a
+//             DataType object
+// param       attr - IN: Specifying location where the referenced object is in
+// param       ref - IN: Reference pointer
+// param       ref_type - IN: Reference type - default to H5R_OBJECT
+// param       plist - IN: Property list - default to PropList::DEFAULT
+// exception   H5::ReferenceException
 // Programmer   Binh-Minh Ribler - Oct, 2006
 // Modification
-//      Jul, 2008
+//        Jul, 2008
 //              Added for application convenience.
 //--------------------------------------------------------------------------
-DataType::DataType(const Attribute& attr, const void* ref, H5R_type_t ref_type) : H5Object(), id(H5I_INVALID_HID), encoded_buf(NULL), buf_size(0)
+ /* DataType::DataType(const Attribute& attr, const void* ref, H5R_type_t ref_type, const PropList& plist) : H5Object(), id(H5I_INVALID_HID), encoded_buf(NULL), buf_size(0)
 {
-    id = H5Location::p_dereference(attr.getId(), ref, ref_type, "constructor - by dereference");
+    id = H5Location::p_dereference(attr.getId(), ref, ref_type, plist, "constructor - by dereference");
 }
+ */ 
 
 //--------------------------------------------------------------------------
 // Function:    DataType copy constructor
-///\brief       Copy constructor: makes a copy of the original DataType object.
+///\brief       Copy constructor: same HDF5 object as \a original
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 DataType::DataType(const DataType& original) : H5Object(), id(original.id), encoded_buf(NULL), buf_size(0)
@@ -145,7 +148,7 @@ DataType::DataType(const DataType& original) : H5Object(), id(original.id), enco
 //--------------------------------------------------------------------------
 DataType::DataType(const PredType& pred_type) : H5Object(), encoded_buf(NULL), buf_size(0)
 {
-    // call C routine to copy the datatype
+    // Call C routine to copy the datatype
     id = H5Tcopy(pred_type.getId());
     if (id < 0)
         throw DataTypeIException("DataType constructor", "H5Tcopy failed");
@@ -155,38 +158,38 @@ DataType::DataType(const PredType& pred_type) : H5Object(), encoded_buf(NULL), b
 // Function:    DataType overloaded constructor
 ///\brief       Creates a DataType instance by opening an HDF5 datatype given
 ///             its name as a char*.
-///\param       loc       - IN: Location of the type
-///\param       type_name - IN: Datatype name
+///\param       loc        - IN: Location of the type
+///\param       dtype_name - IN: Datatype name
 ///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - Sept 2017
+// Programmer   Binh-Minh Ribler - Dec 2016
 // Description
-//              In 1.8.20, this constructor was introduced and may replace the
+//              In 1.10.1, this constructor was introduced and may replace the
 //              existing function CommonFG::openDataType(const char*) to
 //              improve usability.
-//              -BMR, Sept 2017
+//              -BMR, Dec 2016
 //--------------------------------------------------------------------------
-DataType::DataType(const H5Location& loc, const char *type_name) : H5Object(), encoded_buf(NULL), buf_size(0)
+DataType::DataType(const H5Location& loc, const char *dtype_name) : H5Object(), encoded_buf(NULL), buf_size(0)
 {
-    id = p_opentype(loc, type_name);
+    id = p_opentype(loc, dtype_name);
 }
 
 //--------------------------------------------------------------------------
 // Function:    DataType overloaded constructor
 ///\brief       Creates a DataType instance by opening an HDF5 datatype given
 ///             its name as an \c H5std_string.
-///\param       loc       - IN: Location of the type
-///\param       type_name - IN: Datatype name
+///\param       loc        - IN: Location of the type
+///\param       dtype_name - IN: Datatype name
 ///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - Sept 2017
+// Programmer   Binh-Minh Ribler - Dec 2016
 // Description
-//              In 1.8.20, this constructor was introduced and may replace the
-//              existing function CommonFG::openDataType(const H5std_string&)
-//              to improve usability.
-//              -BMR, Sept 2017
+//              In 1.10.1, this constructor was introduced and may replace the
+//              existing function CommonFG::openDataType(const H5std_string&) to
+//              improve usability.
+//              -BMR, Dec 2016
 //--------------------------------------------------------------------------
-DataType::DataType(const H5Location& loc, const H5std_string& type_name) : H5Object(), encoded_buf(NULL), buf_size(0)
+DataType::DataType(const H5Location& loc, const H5std_string& dtype_name) : H5Object(), encoded_buf(NULL), buf_size(0)
 {
-    id = p_opentype(loc, type_name.c_str());
+    id = p_opentype(loc, dtype_name.c_str());
 }
 
 //--------------------------------------------------------------------------
@@ -213,7 +216,7 @@ void DataType::copy(const DataType& like_type)
 
     // call C routine to copy the datatype
     id = H5Tcopy(like_type.getId());
-    if(id < 0)
+    if (id < 0)
         throw DataTypeIException(inMemFunc("copy"), "H5Tcopy failed");
 }
 
@@ -238,16 +241,47 @@ void DataType::copy(const DataSet& dset)
 
     // call C routine to copy the datatype
     id = H5Tcopy(dset.getId());
-    if(id < 0)
+    if (id < 0)
         throw DataTypeIException(inMemFunc("copy"), "H5Tcopy failed");
 }
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+//--------------------------------------------------------------------------
+// Function:    DataType::p_decode
+// Purpose      Returns an id of a type by decoding the binary object
+///             description of this datatype.
+///\exception   H5::DataTypeIException
+// Programmer   Binh-Minh Ribler - Aug 2017
+//--------------------------------------------------------------------------
+hid_t DataType::p_decode() const
+{
+    // Make sure that the buffer can be decoded
+    if (encoded_buf == NULL)
+    {
+        throw DataTypeIException("DataType::p_decode", "No encoded buffer");
+    }
+
+    // Call C function to decode the binary object description
+    hid_t encoded_dtype_id = H5Tdecode(encoded_buf);
+
+    // If H5Tdecode fails, raise exception
+    if (encoded_dtype_id < 0)
+    {
+        throw DataTypeIException("DataType::p_decode", "H5Tdecode failed");
+    }
+    else
+    {
+        return(encoded_dtype_id);
+    }
+}
+#endif // DOXYGEN_SHOULD_SKIP_THIS
+
 //--------------------------------------------------------------------------
 // Function:    DataType::decode
 ///\brief       Returns a DataType instance by decoding the binary object
 ///             description of this datatype.
 ///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - Sept 2017
+// Programmer   Binh-Minh Ribler - Aug 2017
 //--------------------------------------------------------------------------
 DataType* DataType::decode() const
 {
@@ -267,7 +301,7 @@ DataType* DataType::decode() const
 // Function:    DataType::encode
 ///\brief       Creates a binary object description of this datatype.
 ///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - Sept 2017
+// Programmer   Binh-Minh Ribler - Aug 2017
 //--------------------------------------------------------------------------
 void DataType::encode()
 {
@@ -299,7 +333,7 @@ void DataType::encode()
 ///\brief       Determines whether this datatype has a binary object
 ///             description.
 ///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - Sept 2017
+// Programmer   Binh-Minh Ribler - Aug 2017
 //--------------------------------------------------------------------------
 bool DataType::hasBinaryDesc() const
 {
@@ -351,9 +385,9 @@ bool DataType::operator==(const DataType& compared_type) const
     // Call C routine H5Tequal to determines whether two datatype
     // identifiers refer to the same datatype
     htri_t ret_value = H5Tequal(id, compared_type.getId());
-    if(ret_value > 0)
+    if (ret_value > 0)
         return true;
-    else if(ret_value == 0)
+    else if (ret_value == 0)
         return false;
     else
     {
@@ -361,6 +395,28 @@ bool DataType::operator==(const DataType& compared_type) const
     }
 }
 
+//--------------------------------------------------------------------------
+// Function:    DataType::p_commit (private)
+//\brief        Commits a transient datatype to a file, creating a new
+//              named datatype
+//\param        loc_id - IN: The id of either a file, group, dataset, named
+//                           datatype, or attribute.
+//\param        name - IN: Name of the datatype
+//\exception    H5::DataTypeIException
+// Programmer   Binh-Minh Ribler - 2000
+// Modification:
+//              Copied from DataType::commit and made into private function
+//              to be commonly used by several overloads of DataType::commit.
+//              BMR - Jan, 2007
+//--------------------------------------------------------------------------
+void DataType::p_commit(hid_t loc_id, const char* name)
+{
+    // Call C routine to commit the transient datatype
+    herr_t ret_value = H5Tcommit2(loc_id, name, id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    if (ret_value < 0)
+        throw DataTypeIException(inMemFunc("p_commit"), "H5Tcommit2 failed");
+}
+
 //--------------------------------------------------------------------------
 // Function:    DataType::commit
 ///\brief       Commits a transient datatype to a file, creating a new
@@ -368,7 +424,7 @@ bool DataType::operator==(const DataType& compared_type) const
 ///\param       loc - IN: A location (file, dataset, datatype, or group)
 ///\param       name - IN: Name of the datatype
 ///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - 2000
+// Programmer   Binh-Minh Ribler - Jan, 2007
 //--------------------------------------------------------------------------
 void DataType::commit(const H5Location& loc, const char* name)
 {
@@ -462,7 +518,7 @@ H5T_conv_t DataType::find(const DataType& dest, H5T_cdata_t **pcdata) const
 {
     // Call C routine to find the conversion function
     H5T_conv_t func = H5Tfind(id, dest.getId(), pcdata);
-    if(func == NULL)
+    if (func == NULL)
     {
         throw DataTypeIException(inMemFunc("find"), "H5Tfind returns a NULL function");
     }
@@ -475,7 +531,7 @@ H5T_conv_t DataType::find(const DataType& dest, H5T_cdata_t **pcdata) const
 ///\param       dest       - IN: Destination datatype
 ///\param       nelmts     - IN: Size of array \a buf
 ///\param       buf        - IN/OUT: Array containing pre- and post-conversion
-///                             values
+///                            values
 ///\param       background - IN: Optional backgroud buffer
 ///\param       plist - IN: Property list - default to PropList::DEFAULT
 ///\return      Pointer to a suitable conversion function
@@ -491,7 +547,7 @@ void DataType::convert(const DataType& dest, size_t nelmts, void *buf, void *bac
     // Call C routine H5Tconvert to convert the data
     herr_t ret_value;
     ret_value = H5Tconvert(id, dest_id, nelmts, buf, background, plist_id);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataTypeIException(inMemFunc("convert"), "H5Tconvert failed");
     }
@@ -515,7 +571,7 @@ void DataType::lock() const
 {
     // Call C routine to lock the datatype
     herr_t ret_value = H5Tlock(id);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataTypeIException(inMemFunc("lock"), "H5Tlock failed");
     }
@@ -533,7 +589,7 @@ H5T_class_t DataType::getClass() const
     H5T_class_t type_class = H5Tget_class(id);
 
     // Return datatype class identifier if successful
-    if(type_class == H5T_NO_CLASS)
+    if (type_class == H5T_NO_CLASS)
     {
         throw DataTypeIException(inMemFunc("getClass"), "H5Tget_class returns H5T_NO_CLASS");
     }
@@ -551,7 +607,7 @@ size_t DataType::getSize() const
 {
     // Call C routine to get the datatype size
     size_t type_size = H5Tget_size(id);
-    if(type_size <= 0) // valid data types are never zero size
+    if (type_size <= 0)    // valid data types are never zero size
     {
         throw DataTypeIException(inMemFunc("getSize"), "H5Tget_size returns invalid datatype size");
     }
@@ -573,7 +629,7 @@ DataType DataType::getSuper() const
 
     // If H5Tget_super returns a valid datatype id, create and return
     // the base type, otherwise, raise exception
-    if(base_type_id > 0)
+    if (base_type_id > 0)
     {
         DataType base_type;
         base_type.p_setId(base_type_id);
@@ -597,8 +653,8 @@ DataType DataType::getSuper() const
 ///             destination datatypes.
 ///\exception   H5::DataTypeIException
 ///\par Description
-///             For more information, please see:
-/// https://support.hdfgroup.org/HDF5/doc/RM/RM_H5T.html#Datatype-Register
+///             For information, please refer to the H5Tregister API in
+///             the HDF5 C Reference Manual.
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DataType::registerFunc(H5T_pers_t pers, const char* name, const DataType& dest, H5T_conv_t func) const
@@ -607,7 +663,7 @@ void DataType::registerFunc(H5T_pers_t pers, const char* name, const DataType& d
 
     // Call C routine H5Tregister to register the conversion function
     herr_t ret_value = H5Tregister(pers, name, id, dest_id, func);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataTypeIException(inMemFunc("registerFunc"), "H5Tregister failed");
     }
@@ -644,7 +700,7 @@ void DataType::unregister(H5T_pers_t pers, const char* name, const DataType& des
 
     // Call C routine H5Tunregister to remove the conversion function
     herr_t ret_value = H5Tunregister(pers, name, id, dest_id, func);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataTypeIException(inMemFunc("unregister"), "H5Tunregister failed");
     }
@@ -674,7 +730,7 @@ void DataType::setTag(const char* tag) const
 {
     // Call C routine H5Tset_tag to tag an opaque datatype.
     herr_t ret_value = H5Tset_tag(id, tag);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw DataTypeIException(inMemFunc("setTag"), "H5Tset_tag failed");
     }
@@ -705,7 +761,7 @@ H5std_string DataType::getTag() const
 
     // if the tag C-string returned is not NULL, convert it to C++ string
     // and return it, otherwise, raise an exception
-    if(tag_Cstr != NULL)
+    if (tag_Cstr != NULL)
     {
         H5std_string tag = H5std_string(tag_Cstr); // C string to string object
         H5free_memory(tag_Cstr); // free the C string
@@ -729,14 +785,36 @@ H5std_string DataType::getTag() const
 bool DataType::detectClass(H5T_class_t cls) const
 {
     htri_t ret_value = H5Tdetect_class(id, cls);
-    if(ret_value > 0)
+    if (ret_value > 0)
         return true;
-    else if(ret_value == 0)
+    else if (ret_value == 0)
         return false;
     else
     {
         throw DataTypeIException(inMemFunc("detectClass"),
-                                "H5Tdetect_class returns negative value");
+            "H5Tdetect_class returns negative value");
+    }
+}
+
+//--------------------------------------------------------------------------
+// Function:    DataType::detectClass (static)
+///\brief       Checks whether a predtype is a certain class of datatype.
+///\return      true if this predtype is the specified type class, and false,
+///             otherwise.
+///\exception   H5::DataTypeIException
+// Programmer   Binh-Minh Ribler - August, 2017
+//--------------------------------------------------------------------------
+bool DataType::detectClass(const PredType& pred_type, H5T_class_t cls)
+{
+    htri_t ret_value = H5Tdetect_class(pred_type.getId(), cls);
+    if (ret_value > 0)
+        return true;
+    else if (ret_value == 0)
+        return false;
+    else
+    {
+        throw DataTypeIException("detectClass on PredType",
+            "H5Tdetect_class returns negative value");
     }
 }
 
@@ -751,14 +829,14 @@ bool DataType::detectClass(H5T_class_t cls) const
 bool DataType::isVariableStr() const
 {
     htri_t is_varlen_str = H5Tis_variable_str(id);
-    if(is_varlen_str == 1)
+    if (is_varlen_str == 1)
         return true;
-    else if(is_varlen_str == 0)
+    else if (is_varlen_str == 0)
         return false;
     else
     {
         throw DataTypeIException(inMemFunc("isVariableStr"),
-                                "H5Tis_variable_str returns negative value");
+            "H5Tis_variable_str returns negative value");
     }
 }
 
@@ -768,7 +846,7 @@ bool DataType::isVariableStr() const
 ///             creation.
 ///\return      A property list object
 ///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - May, 2017
+// Programmer   Binh-Minh Ribler - March, 2017
 // Description
 //              Currently, there is no datatype creation property list class
 //              in the C++ API because there is no associated functionality.
@@ -806,78 +884,27 @@ hid_t DataType::getId() const
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 //--------------------------------------------------------------------------
-// Function:    DataType::p_commit (private)
-//\brief        Commits a transient datatype to a file, creating a new
-//              named datatype
-//\param        loc_id - IN: The id of either a file, group, dataset, named
-//                       datatype, or attribute.
-//\param        name - IN: Name of the datatype
-//\exception    H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - 2000
-// Modification:
-//              Copied from DataType::commit and made into private function
-//              to be commonly used by several overloads of DataType::commit.
-//              BMR - Jan, 2007
-//--------------------------------------------------------------------------
-void DataType::p_commit(hid_t loc_id, const char* name)
-{
-    // Call C routine to commit the transient datatype
-    herr_t ret_value = H5Tcommit2(loc_id, name, id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
-    if(ret_value < 0)
-        throw DataTypeIException(inMemFunc("p_commit"), "H5Tcommit2 failed");
-}
-
-//--------------------------------------------------------------------------
-// Function:    DataType::p_decode (protected)
-// Purpose      Returns an id of a type by decoding the binary object
-///             description of this datatype.
-///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - Sept 2017
-//--------------------------------------------------------------------------
-hid_t DataType::p_decode() const
-{
-    // Make sure that the buffer can be decoded
-    if (encoded_buf == NULL)
-    {
-        throw DataTypeIException("DataType::p_decode", "No encoded buffer");
-    }
-
-    // Call C function to decode the binary object description
-    hid_t encoded_dtype_id = H5Tdecode(encoded_buf);
-
-    // If H5Tdecode fails, raise exception
-    if (encoded_dtype_id < 0)
-    {
-        throw DataTypeIException("DataType::p_decode", "H5Tdecode failed");
-    }
-    else
-    {
-        return(encoded_dtype_id);
-    }
-}
-
-//--------------------------------------------------------------------------
-// Function:    DataType::p_opentype (protected)
+// Function:    DataType::p_opentype (private)
 ///\brief       Opens an HDF5 datatype given its name
 ///\param       loc        - IN: Location of the type
-///\param       type_name - IN: Datatype name
+///\param       dtype_name - IN: Datatype name
 ///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - Sept 2017
+// Programmer   Binh-Minh Ribler - Dec 2016
 // Description
-//              This function was introduced in 1.8.20 to be used by the
-//              new XxxType constructors that open a datatype. -BMR, Sept 2017
+//              This function was introduced in 1.10.1 to be used by the new
+//              XxxType constructors that open a datatype. -BMR, Dec 2016
 //--------------------------------------------------------------------------
-hid_t DataType::p_opentype(const H5Location& loc, const char *type_name) const
+hid_t DataType::p_opentype(const H5Location& loc, const char *dtype_name) const
 {
     // Call C function to open the named datatype at this location
-    hid_t ret_value = H5Topen2(loc.getId(), type_name, H5P_DEFAULT);
+    hid_t ret_value = H5Topen2(loc.getId(), dtype_name, H5P_DEFAULT);
     if (ret_value < 0)
         throw DataTypeIException(inMemFunc("constructor"), "H5Topen2 failed");
     return(ret_value);
 }
 
 //--------------------------------------------------------------------------
-// Function:    DataType::p_setId (protected)
+// Function:    DataType::p_setId
 ///\brief       Sets the identifier of this object to a new value.
 ///
 ///\exception   H5::IdComponentException when the attempt to close the HDF5
@@ -914,12 +941,19 @@ void DataType::close()
     if (p_valid_id(id))
     {
         herr_t ret_value = H5Tclose(id);
-        if(ret_value < 0)
+        if (ret_value < 0)
         {
             throw DataTypeIException(inMemFunc("close"), "H5Tclose failed");
         }
-        // reset the id
+        // Reset the id
         id = H5I_INVALID_HID;
+
+        // Free and reset buffer of encoded object description if it's been used
+        if (encoded_buf != NULL)
+        {
+            HDfree(encoded_buf);
+            buf_size = 0;
+        }
     }
 }
 
@@ -951,4 +985,5 @@ DataType::~DataType()
         cerr << inMemFunc("~DataType - ") << close_error.getDetailMsg() << endl;
     }
 }
+
 } // end namespace
