@@ -16,13 +16,15 @@
 #include "H5Include.h"
 #include "H5Exception.h"
 #include "H5IdComponent.h"
+#include "H5DataSpace.h"
 #include "H5PropList.h"
+#include "H5OcreatProp.h"
+#include "H5DcreatProp.h"
+#include "H5LcreatProp.h"
 #include "H5LaccProp.h"
 #include "H5Location.h"
 #include "H5Object.h"
 #include "H5DataType.h"
-#include "H5OcreatProp.h"
-#include "H5DcreatProp.h"
 
 namespace H5 {
 
@@ -93,7 +95,7 @@ DSetCreatPropList::DSetCreatPropList() : ObjCreatPropList(H5P_DATASET_CREATE) {}
 
 //--------------------------------------------------------------------------
 // Function:    DSetCreatPropList copy constructor
-///\brief       Copy constructor: makes a copy of the original
+///\brief       Copy constructor: same HDF5 object as \a original
 ///             DSetCreatPropList object
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
@@ -125,7 +127,7 @@ DSetCreatPropList::DSetCreatPropList(const hid_t plist_id) : ObjCreatPropList(pl
 void DSetCreatPropList::setChunk(int ndims, const hsize_t* dim) const
 {
     herr_t ret_value = H5Pset_chunk(id, ndims, dim);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setChunk", "H5Pset_chunk failed");
     }
@@ -143,10 +145,10 @@ void DSetCreatPropList::setChunk(int ndims, const hsize_t* dim) const
 int DSetCreatPropList::getChunk(int max_ndims, hsize_t* dim) const
 {
     int chunk_size = H5Pget_chunk(id, max_ndims, dim);
-    if(chunk_size < 0)
+    if (chunk_size < 0)
     {
         throw PropListIException("DSetCreatPropList::getChunk",
-                "H5Pget_chunk returns negative chunk size");
+            "H5Pget_chunk returns negative chunk size");
     }
     return(chunk_size);
 }
@@ -157,17 +159,17 @@ int DSetCreatPropList::getChunk(int max_ndims, hsize_t* dim) const
 ///\param       layout - IN: Type of storage layout for raw data
 ///\exception   H5::PropListIException
 ///\par Description
-///             For information on valid layout types, please refer to
-/// https://support.hdfgroup.org/HDF5/doc/RM/RM_H5P.html#Property-SetLayout
+///             For information, please refer to the H5Pset_layout API in
+///             the HDF5 C Reference Manual.
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DSetCreatPropList::setLayout(H5D_layout_t layout) const
 {
     herr_t ret_value = H5Pset_layout(id, layout);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setLayout",
-                "H5Pset_layout failed");
+            "H5Pset_layout failed");
     }
 }
 
@@ -190,10 +192,10 @@ void DSetCreatPropList::setLayout(H5D_layout_t layout) const
 H5D_layout_t DSetCreatPropList::getLayout() const
 {
     H5D_layout_t layout = H5Pget_layout(id);
-    if(layout == H5D_LAYOUT_ERROR)
+    if (layout == H5D_LAYOUT_ERROR)
     {
         throw PropListIException("DSetCreatPropList::getLayout",
-                "H5Pget_layout returns H5D_LAYOUT_ERROR");
+            "H5Pget_layout returns H5D_LAYOUT_ERROR");
     }
     return(layout);
 }
@@ -213,10 +215,10 @@ H5D_layout_t DSetCreatPropList::getLayout() const
 void DSetCreatPropList::setDeflate(int level) const
 {
     herr_t ret_value = H5Pset_deflate(id, level);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setDeflate",
-                "H5Pset_deflate failed");
+            "H5Pset_deflate failed");
     }
 }
 
@@ -232,18 +234,17 @@ void DSetCreatPropList::setDeflate(int level) const
 ///\par Description
 ///             The associate C function sets an SZIP compression filter,
 ///             H5Z_FILTER_SZIP, for a dataset.  For more information about
-///             SZIP and usage, please refer to the C layer Reference
-///             Manual at:
-/// https://support.hdfgroup.org/HDF5/doc/RM_H5P.html#Property-SetSzip
+///             SZIP and usage, please refer to the H5Pset_szip API in
+///             the HDF5 C Reference Manual.
 // Programmer   Binh-Minh Ribler - Jan, 2007
 //--------------------------------------------------------------------------
 void DSetCreatPropList::setSzip(unsigned int options_mask, unsigned int pixels_per_block) const
 {
     herr_t ret_value = H5Pset_szip(id, options_mask, pixels_per_block);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setSzip",
-                "H5Pset_szip failed");
+            "H5Pset_szip failed");
     }
 }
 
@@ -254,18 +255,17 @@ void DSetCreatPropList::setSzip(unsigned int options_mask, unsigned int pixels_p
 ///\par Description
 ///             The associate C function sets an Nbit compression filter,
 ///             H5Z_FILTER_NBIT, for a dataset.  For more information about
-///             Nbit compression, please refer to the C layer Reference
-///             Manual at:
-/// https://support.hdfgroup.org/HDF5/doc/RM/RM_H5P.html#Property-setNbit
+///             Nbit compression, please refer to the H5Pset_nbit API in
+///             the HDF5 C Reference Manual.
 // Programmer   Binh-Minh Ribler - Apr, 2016
 //--------------------------------------------------------------------------
 void DSetCreatPropList::setNbit() const
 {
     herr_t ret_value = H5Pset_nbit(id);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setNbit",
-                "H5Pset_nbit failed");
+            "H5Pset_nbit failed");
     }
 }
 
@@ -283,14 +283,13 @@ void DSetCreatPropList::setNbit() const
 ///             according to the actual dataset datatype.
 ///\par
 ///             For information on setting fill value, please refer to the
-///             C layer Reference Manual at:
-/// https://support.hdfgroup.org/HDF5/doc/RM/RM_H5P.html#Property-SetFillValue
+///             H5Pset_fill_value API in the HDF5 C Reference Manual.
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DSetCreatPropList::setFillValue(const DataType& fvalue_type, const void* value) const
 {
     herr_t ret_value = H5Pset_fill_value(id, fvalue_type.getId(), value);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setFillValue",
                 "H5Pset_fill_value failed");
@@ -313,7 +312,7 @@ void DSetCreatPropList::setFillValue(const DataType& fvalue_type, const void* va
 void DSetCreatPropList::getFillValue(const DataType& fvalue_type, void* value) const
 {
     herr_t ret_value = H5Pget_fill_value(id, fvalue_type.getId(), value);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::getFillValue",
                 "H5Pget_fill_value failed");
@@ -334,7 +333,7 @@ H5D_fill_value_t DSetCreatPropList::isFillValueDefined() const
 {
     H5D_fill_value_t status;
     herr_t ret_value = H5Pfill_value_defined(id, &status);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::isFillValueDefined",
                 "H5Pfill_value_defined returned H5D_FILL_VALUE_ERROR (-1)");
@@ -367,7 +366,7 @@ void DSetCreatPropList::setFilter(H5Z_filter_t filter_id, unsigned int flags,
         size_t cd_nelmts, const unsigned int cd_values[]) const
 {
     herr_t ret_value = H5Pset_filter(id, filter_id, flags, cd_nelmts, cd_values);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setFilter",
                 "H5Pset_filter failed");
@@ -387,7 +386,7 @@ void DSetCreatPropList::setFilter(H5Z_filter_t filter_id, unsigned int flags,
 void DSetCreatPropList::removeFilter(H5Z_filter_t filter_id) const
 {
     herr_t ret_value = H5Premove_filter(id, filter_id);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::removeFilter",
                 "H5Premove_filter failed");
@@ -404,7 +403,7 @@ void DSetCreatPropList::removeFilter(H5Z_filter_t filter_id) const
 int DSetCreatPropList::getNfilters() const
 {
     int num_filters = H5Pget_nfilters(id);
-    if(num_filters < 0)
+    if (num_filters < 0)
     {
         throw PropListIException("DSetCreatPropList::getNfilters",
                 "H5Pget_nfilters returned negative number of filters");
@@ -439,11 +438,11 @@ H5Z_filter_t DSetCreatPropList::getFilter(int filter_number,
 {
     H5Z_filter_t filter_id;
     filter_id = H5Pget_filter2(id, filter_number, &flags, &cd_nelmts,
-                                cd_values, namelen, name, &filter_config);
-    if(filter_id == H5Z_FILTER_ERROR)
+                                  cd_values, namelen, name, &filter_config);
+    if (filter_id == H5Z_FILTER_ERROR)
         throw PropListIException("DSetCreatPropList::getFilter",
                 "H5Pget_filter2 returned H5Z_FILTER_ERROR");
-   else
+    else
         return(filter_id);
 }
 
@@ -467,7 +466,7 @@ void DSetCreatPropList::getFilterById(H5Z_filter_t filter_id,
         size_t namelen, char name[], unsigned int &filter_config) const
 {
     herr_t ret_value = H5Pget_filter_by_id2(id, filter_id, &flags, &cd_nelmts,
-                                cd_values, namelen, name, &filter_config);
+                                  cd_values, namelen, name, &filter_config);
     if (ret_value < 0)
         throw PropListIException("DSetCreatPropList::getFilterById",
                 "H5Pget_filter_by_id2 failed");
@@ -498,7 +497,7 @@ void DSetCreatPropList::modifyFilter(H5Z_filter_t filter_id, unsigned int
         flags, size_t cd_nelmts, const unsigned int cd_values[]) const
 {
     herr_t ret_value = H5Pmodify_filter(id, filter_id, flags, cd_nelmts, cd_values);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::modifyFilter",
                 "H5Pmodify_filter failed");
@@ -517,9 +516,9 @@ void DSetCreatPropList::modifyFilter(H5Z_filter_t filter_id, unsigned int
 bool DSetCreatPropList::allFiltersAvail() const
 {
     htri_t ret_value = H5Pall_filters_avail(id);
-    if(ret_value > 0)
+    if (ret_value > 0)
         return true;
-    else if(ret_value == 0)
+    else if (ret_value == 0)
         return false;
     else // Raise exception when H5Pall_filters_avail returns a negative value
     {
@@ -533,15 +532,14 @@ bool DSetCreatPropList::allFiltersAvail() const
 ///
 ///\exception   H5::PropListIException
 ///\par Description
-///             Please refer to the Reference Manual of \c H5Pset_shuffle for
-///             details.
-/// https://support.hdfgroup.org/HDF5/doc/RM/RM_H5P.html#Property-SetShuffle
+///             For information, please refer to the H5Pset_shuffle API in
+///             the HDF5 C Reference Manual.
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DSetCreatPropList::setShuffle() const
 {
     herr_t ret_value = H5Pset_shuffle(id);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setShuffle",
                 "H5Pset_shuffle failed");
@@ -566,7 +564,7 @@ H5D_alloc_time_t DSetCreatPropList::getAllocTime() const
 {
     H5D_alloc_time_t alloc_time;
     herr_t ret_value = H5Pget_alloc_time(id, &alloc_time);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::getAllocTime",
                 "H5Pget_alloc_time failed");
@@ -590,7 +588,7 @@ H5D_fill_time_t DSetCreatPropList::getFillTime() const
 {
     H5D_fill_time_t fill_time;
     herr_t ret_value = H5Pget_fill_time(id, &fill_time);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::getFillTime",
                 "H5Pget_fill_time failed");
@@ -615,7 +613,7 @@ H5D_fill_time_t DSetCreatPropList::getFillTime() const
 void DSetCreatPropList::setAllocTime(H5D_alloc_time_t alloc_time) const
 {
     herr_t ret_value = H5Pset_alloc_time(id, alloc_time);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setAllocTime",
                 "H5Pset_alloc_time failed");
@@ -636,7 +634,7 @@ void DSetCreatPropList::setAllocTime(H5D_alloc_time_t alloc_time) const
 void DSetCreatPropList::setFillTime(H5D_fill_time_t fill_time) const
 {
     herr_t ret_value = H5Pset_fill_time(id, fill_time);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setFillTime",
                 "H5Pset_fill_time failed");
@@ -653,7 +651,7 @@ void DSetCreatPropList::setFillTime(H5D_fill_time_t fill_time) const
 void DSetCreatPropList::setFletcher32() const
 {
     herr_t ret_value = H5Pset_fletcher32(id);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setFletcher32",
                 "H5Pset_fletcher32 failed");
@@ -679,7 +677,7 @@ void DSetCreatPropList::setFletcher32() const
 void DSetCreatPropList::setExternal(const char* name, off_t offset, hsize_t size) const
 {
     herr_t ret_value = H5Pset_external(id, name, offset, size);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::setExternal",
                 "H5Pset_external failed");
@@ -696,7 +694,7 @@ void DSetCreatPropList::setExternal(const char* name, off_t offset, hsize_t size
 int DSetCreatPropList::getExternalCount() const
 {
     int num_ext_files = H5Pget_external_count(id);
-    if(num_ext_files < 0)
+    if (num_ext_files < 0)
     {
         throw PropListIException("DSetCreatPropList::getExternalCount",
                 "H5Pget_external_count returns negative number of external files");
@@ -730,18 +728,69 @@ int DSetCreatPropList::getExternalCount() const
 void DSetCreatPropList::getExternal(unsigned idx, size_t name_size, char* name, off_t& offset, hsize_t& size) const
 {
     herr_t ret_value = H5Pget_external(id, idx, name_size, name, &offset, &size);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException("DSetCreatPropList::getExternal",
                 "H5Pget_external failed");
     }
 }
 
+//--------------------------------------------------------------------------
+// Function:    DSetCreatPropList::setVirtual
+///\brief       Maps elements of a virtual dataset to elements of the source
+///             dataset.
+///\param       vspace     - IN: Dataspace the virtual dataset, possibly an
+///                             unlimited selection
+///\param       src_fname  - IN: Name of the HDF5 file where the source dataset
+///                             is located (\a char*)
+///\param       src_dsname - IN: Path to the dataset in the file specified by
+///                             \a src_file_name (\a char*)
+///\param       sspace     - IN: Dataspace with a selection applied, possibly
+///                             an unlimited selection
+///\exception   H5::PropListIException
+///\par Description
+///             For information, please refer to the H5Pset_virtual API in
+///             the HDF5 C Reference Manual.
+// Programmer   Binh-Minh Ribler - Mar, 2017
+//--------------------------------------------------------------------------
+void DSetCreatPropList::setVirtual(const DataSpace& vspace, const char *src_fname, const char *src_dsname, const DataSpace& sspace) const
+{
+    herr_t ret_value = H5Pset_virtual(id, vspace.getId(), src_fname, src_dsname, sspace.getId());
+    if (ret_value < 0)
+    {
+        throw PropListIException("DSetCreatPropList::setVirtual",
+            "H5Pset_virtual failed");
+    }
+}
+
+//--------------------------------------------------------------------------
+// Function:    DSetCreatPropList::setVirtual
+///\brief       Maps elements of a virtual dataset to elements of the source
+///             dataset.
+///\param       vspace     - IN: Dataspace the virtual dataset, possibly an
+///                             unlimited selection
+///\param       src_fname  - IN: Name of the HDF5 file where the source dataset
+///                             is located (\a H5std_string)
+///\param       src_dsname - IN: Path to the dataset in the file specified by
+///                             \a src_file_name (\a H5std_string)
+///\param       sspace     - IN: Dataspace with a selection applied, possibly
+///                             an unlimited selection
+///\exception   H5::PropListIException
+///\par Description
+///             For information, please refer to the H5Pset_virtual API in
+///             the HDF5 C Reference Manual.
+// Programmer   Binh-Minh Ribler - Mar, 2017
+//--------------------------------------------------------------------------
+void DSetCreatPropList::setVirtual(const DataSpace& vspace, const H5std_string src_fname, const H5std_string src_dsname, const DataSpace& sspace) const
+{
+    setVirtual(vspace, src_fname.c_str(), src_dsname.c_str(), sspace);
+}
+
 //--------------------------------------------------------------------------
 // Function:    DSetCreatPropList destructor
 ///\brief       Noop destructor.
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
-DSetCreatPropList::~DSetCreatPropList () {}
+DSetCreatPropList::~DSetCreatPropList() {}
 
 } // end namespace
