@@ -22,21 +22,27 @@
 #include "H5Exception.h"
 #include "H5IdComponent.h"
 #include "H5PropList.h"
+#include "H5FaccProp.h"
+#include "H5FcreatProp.h"
 #include "H5OcreatProp.h"
 #include "H5DcreatProp.h"
+#include "H5DxferProp.h"
+#include "H5LcreatProp.h"
 #include "H5LaccProp.h"
 #include "H5Location.h"
 #include "H5Object.h"
 #include "H5AbstractDs.h"
+#include "H5DataSpace.h"
+#include "H5DataSet.h"
 #include "H5CommonFG.h"
 #include "H5Attribute.h"
 #include "H5Group.h"
+#include "H5File.h"
+#include "H5Alltypes.h"
 
 namespace H5 {
-#ifndef H5_NO_STD
     using std::cerr;
     using std::endl;
-#endif  // H5_NO_STD
 
 //--------------------------------------------------------------------------
 // Function:    Group default constructor
@@ -47,7 +53,7 @@ Group::Group() : H5Object(), CommonFG(), id(H5I_INVALID_HID) {}
 
 //--------------------------------------------------------------------------
 // Function:    Group copy constructor
-///\brief       Copy constructor: makes a copy of the original Group object.
+///\brief       Copy constructor: same HDF5 object as \a original
 ///\param       original - IN: Original group to copy
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
@@ -56,11 +62,32 @@ Group::Group(const Group& original) : H5Object(), CommonFG(), id(original.id)
     incRefCount(); // increment number of references to this id
 }
 
+//--------------------------------------------------------------------------
+// Function:    Group::closeObjId
+///\brief       Closes an object, which was opened with Group::getObjId
+///\exception   H5::FileIException or H5::GroupIException
+// Programmer   Binh-Minh Ribler - March, 2017
+//--------------------------------------------------------------------------
+void Group::closeObjId(hid_t obj_id) const
+{
+    herr_t ret_value = H5Oclose(obj_id);
+    if (ret_value < 0)
+    {
+        throwException("Group::closeObjId", "H5Oclose failed");
+    }
+}
+
 //--------------------------------------------------------------------------
 // Function:    Group::getLocId
-///\brief       Returns the id of this group.
-///\return      Id of this group
+// Purpose:     Get the id of this group
 // Programmer   Binh-Minh Ribler - 2000
+// Description
+//              This function is a redefinition of CommonFG::getLocId.  It
+//              is used by CommonFG member functions to get the file id.
+// Deprecated:
+//      Aug 18, 2016 -BMR
+//              After HDFFV-9920, the Group's methods can use getId() and
+//              getLocId() is kept for backward compatibility.
 //--------------------------------------------------------------------------
 hid_t Group::getLocId() const
 {
@@ -84,35 +111,72 @@ Group::Group(const hid_t existing_id) : H5Object(), CommonFG(), id(existing_id)
 ///\param       loc - IN: Specifying location referenced object is in
 ///\param       ref - IN: Reference pointer
 ///\param       ref_type - IN: Reference type - default to H5R_OBJECT
+///\param       plist - IN: Property list - default to PropList::DEFAULT
 ///\exception   H5::ReferenceException
 ///\par Description
 ///             \c obj can be DataSet, Group, or named DataType, that
 ///             is a datatype that has been named by DataType::commit.
 // Programmer   Binh-Minh Ribler - Oct, 2006
 //--------------------------------------------------------------------------
-Group::Group(const H5Location& loc, const void* ref, H5R_type_t ref_type) : H5Object(), id(H5I_INVALID_HID)
+Group::Group(const H5Location& loc, const void* ref, H5R_type_t ref_type, const PropList& plist) : H5Object(), CommonFG(), id(H5I_INVALID_HID)
 {
-    id = H5Location::p_dereference(loc.getId(), ref, ref_type, "constructor - by dereference");
+    id = H5Location::p_dereference(loc.getId(), ref, ref_type, plist, "constructor - by dereference");
 }
 
 //--------------------------------------------------------------------------
-// Function:    Group overload constructor - dereference
-// Purpose      Given a reference, ref, to an hdf5 group, creates a Group object
-// \param       attr - IN: Specifying location where the referenced object is in
-// \param       ref - IN: Reference pointer
-// \param       ref_type - IN: Reference type - default to H5R_OBJECT
-// \exception   H5::ReferenceException
-// Programmer   Binh-Minh Ribler - Oct, 2006
-// Modification
-//      May, 2017
-//              Removed in 1.8.19 because H5Location is Attribute's baseclass
-//              now. -BMR
+// Function:    Group::getNumObjs
+///\brief       Returns the number of objects in this group.
+///\return      Number of objects
+///\exception   H5::FileIException or H5::GroupIException
+// Programmer   Binh-Minh Ribler - January, 2003
 //--------------------------------------------------------------------------
-/*Group::Group(const Attribute& attr, const void* ref, H5R_type_t ref_type) : H5Object(), id(H5I_INVALID_HID)
+hsize_t Group::getNumObjs() const
 {
-    id = H5Location::p_dereference(attr.getId(), ref, ref_type, "constructor - by dereference");
+    H5G_info_t ginfo;      // Group information
+
+    herr_t ret_value = H5Gget_info(getId(), &ginfo);
+    if(ret_value < 0)
+        throwException("getNumObjs", "H5Gget_info failed");
+    return (ginfo.nlinks);
+}
+
+//--------------------------------------------------------------------------
+// Function:    Group::getObjId
+///\brief       Opens an object via object header.
+///\param       obj_name - IN: Path to the object
+///\param       plist    - IN: Access property list for the link pointing to
+///                            the object
+///\exception   H5::FileIException or H5::GroupIException
+///\par Description
+///             This function opens an object in a group or file, using
+///             H5Oopen.  Thus, an object can be opened without knowing
+///             the object's type.
+// Programmer   Binh-Minh Ribler - March, 2017
+//--------------------------------------------------------------------------
+hid_t Group::getObjId(const char* obj_name, const PropList& plist) const
+{
+    hid_t ret_value = H5Oopen(getId(), obj_name, plist.getId());
+    if (ret_value < 0)
+    {
+        throwException("Group::getObjId", "H5Oopen failed");
+    }
+    return(ret_value);
+}
+
+//--------------------------------------------------------------------------
+// Function:    Group::getObjId
+///\brief       This is an overloaded member function, provided for convenience.
+///             It takes a reference to a \c H5std_string for the object's name.
+///\param       obj_name - IN: Path to the object
+///\param       plist    - IN: Access property list for the link pointing to
+///                            the object
+///\exception   H5::FileIException or H5::GroupIException
+// Programmer   Binh-Minh Ribler - March, 2017
+//--------------------------------------------------------------------------
+hid_t Group::getObjId(const H5std_string& obj_name, const PropList& plist) const
+{
+    return(getObjId(obj_name.c_str(), plist));
 }
-*/
 
 //--------------------------------------------------------------------------
 // Function:    Group::getId
@@ -151,7 +215,7 @@ void Group::p_setId(const hid_t new_id)
         close();
     }
     catch (Exception& close_error) {
-        throw GroupIException("Group::p_setId", close_error.getDetailMsg());
+        throwException("Group::p_setId", close_error.getDetailMsg());
     }
     // reset object's id to the given id
     id = new_id;
@@ -170,9 +234,9 @@ void Group::close()
     if (p_valid_id(id))
     {
         herr_t ret_value = H5Gclose(id);
-        if(ret_value < 0)
+        if (ret_value < 0)
         {
-            throw GroupIException("Group::close", "H5Gclose failed");
+            throwException("Group::close", "H5Gclose failed");
         }
         // reset the id
         id = H5I_INVALID_HID;
@@ -186,11 +250,10 @@ void Group::close()
 ///\param       msg       - Message describing the failure
 ///\exception   H5::GroupIException
 // Description
-//              This function is used in CommonFG implementation so that
+//              This function is also used in H5Location's methods so that
 //              proper exception can be thrown for file or group.  The
-//              argument func_name is a member of CommonFG and "Group::"
-//              will be inserted to indicate the function called is an
-//              implementation of Group.
+//              "Group::" will be inserted to indicate the function called is
+//              an implementation of Group.
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void Group::throwException(const H5std_string& func_name, const H5std_string& msg) const
