@@ -22,8 +22,9 @@
 #include "H5FaccProp.h"
 #include "H5FcreatProp.h"
 #include "H5OcreatProp.h"
-#include "H5DcreatProp.h"
 #include "H5DxferProp.h"
+#include "H5DcreatProp.h"
+#include "H5LcreatProp.h"
 #include "H5LaccProp.h"
 #include "H5Location.h"
 #include "H5Object.h"
@@ -45,7 +46,7 @@ namespace H5 {
 void H5Library::open()
 {
     herr_t ret_value = H5open();
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw LibraryIException("H5Library::open", "H5open failed");
     }
@@ -61,7 +62,7 @@ void H5Library::open()
 void H5Library::close()
 {
     herr_t ret_value = H5close();
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw LibraryIException("H5Library::close", "H5close failed");
     }
@@ -79,7 +80,7 @@ void H5Library::close()
 //--------------------------------------------------------------------------
 void H5Library::dontAtExit()
 {
-    herr_t ret_value = H5dont_atexit();
+   (void)H5dont_atexit();
 }
 
 //--------------------------------------------------------------------------
@@ -94,7 +95,7 @@ void H5Library::dontAtExit()
 void H5Library::getLibVersion(unsigned& majnum, unsigned& minnum, unsigned& relnum)
 {
     herr_t ret_value = H5get_libversion(&majnum, &minnum, &relnum);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw LibraryIException("H5Library::getLibVersion", "H5get_libversion failed");
     }
@@ -110,14 +111,13 @@ void H5Library::getLibVersion(unsigned& majnum, unsigned& minnum, unsigned& reln
 ///\exception   H5::LibraryIException
 ///\par Description
 ///             For information about library version, please refer to
-///             the C layer Reference Manual at:
-/// https://support.hdfgroup.org/HDF5/doc/RM/RM_H5.html#Library-VersCheck
+///             the H5check_version API in the HDF5 C Reference Manual.
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void H5Library::checkVersion(unsigned majnum, unsigned minnum, unsigned relnum)
 {
     herr_t ret_value = H5check_version(majnum, minnum, relnum);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw LibraryIException("H5Library::checkVersion", "H5check_version failed");
     }
@@ -146,7 +146,7 @@ void H5Library::checkVersion(unsigned majnum, unsigned minnum, unsigned relnum)
 void H5Library::garbageCollect()
 {
     herr_t ret_value = H5garbage_collect();
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw LibraryIException("H5Library::garbageCollect", "H5garbage_collect failed");
     }
@@ -188,6 +188,10 @@ void H5Library::initH5cpp()
     if (ret_value != 0)
         throw LibraryIException("H5Library::initH5cpp", "Registrating LinkAccPropList::deleteConstants failed");
 
+    ret_value = std::atexit(LinkCreatPropList::deleteConstants);
+    if (ret_value != 0)
+        throw LibraryIException("H5Library::initH5cpp", "Registrating LinkCreatPropList::deleteConstants failed");
+
     ret_value = std::atexit(FileAccPropList::deleteConstants);
     if (ret_value != 0)
         throw LibraryIException("H5Library::initH5cpp", "Registrating FileAccPropList::deleteConstants failed");
@@ -240,9 +244,8 @@ void H5Library::termH5cpp()
 ///\exception   H5::LibraryIException
 ///\par Description
 ///             Setting a value of -1 for a limit means no limit of that type.
-///             For more information on free list limits, please refer to C
-///             layer Reference Manual at:
-/// https://support.hdfgroup.org/HDF5/doc/RM/RM_H5.html#Library-SetFreeListLimits
+///             For more information on free list limits, please refer to
+///             the H5set_free_list_limits API in the HDF5 C Reference Manual.
 // Programmer   Binh-Minh Ribler - May, 2004
 //--------------------------------------------------------------------------
 void H5Library::setFreeListLimits(int reg_global_lim, int reg_list_lim,
@@ -250,17 +253,25 @@ void H5Library::setFreeListLimits(int reg_global_lim, int reg_list_lim,
         int blk_list_lim)
 {
     herr_t ret_value = H5set_free_list_limits(reg_global_lim, reg_list_lim, arr_global_lim, arr_list_lim, blk_global_lim, blk_list_lim);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw LibraryIException("H5Library::setFreeListLimits", "H5set_free_list_limits failed");
     }
 }
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
-// Default constructor - private
+//--------------------------------------------------------------------------
+// Function:    H5Library default constructor - private
+///\brief       Default constructor: Creates a stub H5Library object
+// Programmer   Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
 H5Library::H5Library(){}
 
-// Destructor - private
+//--------------------------------------------------------------------------
+// Function:    H5Library destructor
+///\brief       Noop destructor
+// Programmer   Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
 H5Library::~H5Library(){}
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
