@@ -13,18 +13,28 @@
 
 #include <string>
 
-#include "H5private.h"          // for HDmemset
+#include "H5private.h"        // for HDmemset
 #include "H5Include.h"
 #include "H5Exception.h"
 #include "H5IdComponent.h"
 #include "H5PropList.h"
-#include "H5DataSpace.h"
-#include "H5AbstractDs.h"
+#include "H5FaccProp.h"
+#include "H5FcreatProp.h"
+#include "H5OcreatProp.h"
+#include "H5DcreatProp.h"
+#include "H5DxferProp.h"
+#include "H5LcreatProp.h"
 #include "H5LaccProp.h"
 #include "H5Location.h"
-#include "H5Attribute.h"
 #include "H5Object.h"
 #include "H5DataType.h"
+#include "H5DataSpace.h"
+#include "H5AbstractDs.h"
+#include "H5CommonFG.h"
+#include "H5Group.h"
+#include "H5File.h"
+#include "H5DataSet.h"
+#include "H5Attribute.h"
 
 namespace H5 {
 
@@ -32,19 +42,14 @@ namespace H5 {
 // userAttrOpWrpr simply interfaces between the user's function and the
 // C library function H5Aiterate2; used to resolve the different prototype
 // problem.  May be moved to Iterator later.
- /* extern "C" herr_t userAttrOpWrpr(hid_t loc_id, const char *attr_name,
+extern "C" herr_t userAttrOpWrpr(hid_t loc_id, const char *attr_name,
     const H5A_info_t *ainfo, void *op_data)
 {
     H5std_string s_attr_name = H5std_string(attr_name);
-#ifdef NO_STATIC_CAST
-    UserData4Aiterate* myData = (UserData4Aiterate *) op_data;
-#else
-    UserData4Aiterate* myData = static_cast <UserData4Aiterate *> (op_data);
-#endif
+    UserData4Aiterate* myData = reinterpret_cast<UserData4Aiterate *> (op_data);
     myData->op(*myData->location, s_attr_name, myData->opData);
     return 0;
 }
- */ 
 
 //--------------------------------------------------------------------------
 // Function:    H5Object default constructor (protected)
@@ -53,42 +58,20 @@ namespace H5 {
 H5Object::H5Object() : H5Location() {}
 
 //--------------------------------------------------------------------------
-// Function:    H5Object overloaded constructor (protected)
-// Purpose      Creates an H5Object object using the id of an existing HDF5
-//              object.
-// Parameters   object_id - IN: Id of an existing HDF5 object
-// Programmer   Binh-Minh Ribler - 2000
-// *** Deprecation warning ***
-// This constructor is no longer appropriate because the data member "id" had
-// been moved to the sub-classes.  It will be removed in 1.10 release.  If its
-// removal does not raise any problems in 1.10, it will be removed from 1.8 in
-// subsequent releases.
-// Removed in 1.8.18 and 1.10.1 - Aug 2016
+// Function:    f_Attribute_setId - friend
+// Purpose:     This function is friend to class H5::Attribute so that it
+//              can set Attribute::id in order to work around a problem
+//              described in the JIRA issue HDFFV-7947.
+//              Applications shouldn't need to use it.
+// param        attr   - IN/OUT: Attribute object to be changed
+// param        new_id - IN: New id to set
+// Programmer   Binh-Minh Ribler - 2015
 //--------------------------------------------------------------------------
-//H5Object::H5Object(const hid_t object_id) : H5Location() {}
-
-//--------------------------------------------------------------------------
-// Function:    H5Object copy constructor
-///\brief       Copy constructor: makes a copy of the original H5Object
-///             instance.
-///\param       original - IN: H5Object instance to copy
-// Programmer   Binh-Minh Ribler - 2000
-// *** Deprecation warning ***
-// This constructor is no longer appropriate because the data member "id" had
-// been moved to the sub-classes.  It is removed from 1.8.15 because it is
-// a noop and it can be generated by the compiler if needed.
-//--------------------------------------------------------------------------
-// H5Object::H5Object(const H5Object& original) : H5Location() {}
-
-/***************************************************************************
-                           Notes for H5A wrappers
-                           ======================
-May, 2017 (1.8.19)
-        These H5A wrappers are copied from H5Location to prevent an attribute
-        id being passed in to H5A APIs.  The original H5A wrapper functions
-        in H5Location will be deprecated in future releases.
-
-***************************************************************************/
+void f_Attribute_setId(Attribute* attr, hid_t new_id)
+{
+    attr->p_setId(new_id);
+}
+#endif
 
 //--------------------------------------------------------------------------
 // Function:    H5Object::createAttribute
@@ -200,6 +183,99 @@ Attribute H5Object::openAttribute(const unsigned int idx) const
     }
 }
 
+//--------------------------------------------------------------------------
+// Function:    H5Object::iterateAttrs
+///\brief       Iterates a user's function over all the attributes of an H5
+///             object, which may be a group, dataset or named datatype.
+///\param       user_op - IN: User's function to operate on each attribute
+///\param       _idx - IN/OUT: Starting (IN) and ending (OUT) attribute indices
+///\param       op_data - IN: User's data to pass to user's operator function
+///\return      Returned value of the last operator if it was non-zero, or
+///             zero if all attributes were processed
+///\exception   H5::AttributeIException
+///\par Description
+///             The signature of user_op is
+///             void (*)(H5::H5Location&, H5std_string, void*).
+///             For information, please refer to the H5Aiterate2 API in
+///             the HDF5 C Reference Manual.
+// Programmer   Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
+int H5Object::iterateAttrs(attr_operator_t user_op, unsigned *_idx, void *op_data)
+{
+    // store the user's function and data
+    UserData4Aiterate* userData = new UserData4Aiterate;
+    userData->opData = op_data;
+    userData->op = user_op;
+    userData->location = this;
+
+    // call the C library routine H5Aiterate2 to iterate the attributes
+    hsize_t idx = _idx ? static_cast<hsize_t>(*_idx) : 0;
+    int ret_value = H5Aiterate2(getId(), H5_INDEX_NAME, H5_ITER_INC, &idx,
+                         userAttrOpWrpr, static_cast<void *>(userData));
+
+    // release memory
+    delete userData;
+
+    if (ret_value >= 0) {
+        /* Pass back update index value to calling code */
+        if (_idx)
+            *_idx = static_cast<unsigned>(idx);
+        return(ret_value);
+    }
+    else  // raise exception when H5Aiterate returns a negative value
+        throw AttributeIException(inMemFunc("iterateAttrs"), "H5Aiterate2 failed");
+}
+
+//--------------------------------------------------------------------------
+// Function:    H5Object::objVersion
+///\brief       Returns the header version of this HDF5 object.
+///\return      Object version, which can have the following values:
+///             \li \c H5O_VERSION_1
+///             \li \c H5O_VERSION_2
+///\exception   H5::ObjHeaderIException
+///             Exception will be thrown when:
+///             - an error returned by the C API
+///             - version number is not one of the valid values above
+// Programmer   Binh-Minh Ribler - December, 2016
+//--------------------------------------------------------------------------
+unsigned H5Object::objVersion() const
+{
+    H5O_info_t objinfo;
+    unsigned version = 0;
+
+    // Use C API to get information of the object
+    herr_t ret_value = H5Oget_info(getId(), &objinfo);
+
+    // Throw exception if C API returns failure
+    if (ret_value < 0)
+        throw Exception(inMemFunc("objVersion"), "H5Oget_info failed");
+    // Return a valid version or throw an exception for invalid value
+    else
+    {
+        version = objinfo.hdr.version;
+        if (version != H5O_VERSION_1 && version != H5O_VERSION_2)
+            throw ObjHeaderIException("objVersion", "Invalid version for object");
+    }
+    return(version);
+}
+
+//--------------------------------------------------------------------------
+// Function:    H5Object::getNumAttrs
+///\brief       Returns the number of attributes attached to this HDF5 object.
+///\return      Number of attributes
+///\exception   H5::AttributeIException
+// Programmer   Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
+int H5Object::getNumAttrs() const
+{
+    H5O_info_t oinfo;    /* Object info */
+
+    if(H5Oget_info(getId(), &oinfo) < 0)
+        throw AttributeIException(inMemFunc("getNumAttrs"), "H5Oget_info failed");
+    else
+        return(static_cast<int>(oinfo.num_attrs));
+}
+
 //--------------------------------------------------------------------------
 // Function:    H5Object::attrExists
 ///\brief       Checks whether the named attribute exists at this location.
@@ -213,7 +289,7 @@ bool H5Object::attrExists(const char* name) const
     // at this location, which could be specified by a file, group, dataset,
     // or named datatype.
     herr_t ret_value = H5Aexists(getId(), name);
-    if(ret_value > 0)
+    if (ret_value > 0)
         return true;
     else if(ret_value == 0)
         return false;
@@ -243,7 +319,7 @@ bool H5Object::attrExists(const H5std_string& name) const
 void H5Object::removeAttr(const char* name) const
 {
     herr_t ret_value = H5Adelete(getId(), name);
-    if(ret_value < 0)
+    if (ret_value < 0)
         throw AttributeIException(inMemFunc("removeAttr"), "H5Adelete failed");
 }
 
@@ -283,28 +359,7 @@ void H5Object::renameAttr(const char* oldname, const char* newname) const
 //--------------------------------------------------------------------------
 void H5Object::renameAttr(const H5std_string& oldname, const H5std_string& newname) const
 {
-   renameAttr (oldname.c_str(), newname.c_str());
-}
-
-// end of Notes for H5A wrappers
-
-//--------------------------------------------------------------------------
-// Function:    H5Object::getNumAttrs
-///\brief       Returns the number of attributes attached to this HDF5 object.
-///\return      Number of attributes
-///\exception   H5::AttributeIException
-// Programmer   Binh-Minh Ribler - 2000
-// Modification
-//          - Moved from H5Location in 1.8.20. -BMR Oct, 2017
-//--------------------------------------------------------------------------
-int H5Object::getNumAttrs() const
-{
-   H5O_info_t oinfo;    /* Object info */
-
-    if(H5Oget_info(getId(), &oinfo) < 0)
-        throw AttributeIException(inMemFunc("getNumAttrs"), "H5Oget_info failed");
-    else
-        return(static_cast<int>(oinfo.num_attrs));
+    renameAttr (oldname.c_str(), newname.c_str());
 }
 
 //--------------------------------------------------------------------------
@@ -320,11 +375,11 @@ ssize_t H5Object::getObjName(char *obj_name, size_t buf_size) const
 
     // If H5Iget_name returns a negative value, raise an exception
     if (name_size < 0)
-    {
+     {
         throw Exception(inMemFunc("getObjName"), "H5Iget_name failed");
     }
     else if (name_size == 0)
-    {
+     {
         throw Exception(inMemFunc("getObjName"), "Object must have a name, but name length is 0");
     }
     // Return length of the name
@@ -337,7 +392,6 @@ ssize_t H5Object::getObjName(char *obj_name, size_t buf_size) const
 ///\return      Name of the object
 ///\exception   H5::Exception
 // Programmer   Binh-Minh Ribler - Mar, 2014
-// Modification
 //--------------------------------------------------------------------------
 H5std_string H5Object::getObjName() const
 {
@@ -348,16 +402,16 @@ H5std_string H5Object::getObjName() const
 
     // If H5Iget_name failed, throw exception
     if (name_size < 0)
-    {
+     {
         throw Exception(inMemFunc("getObjName"), "H5Iget_name failed");
     }
     else if (name_size == 0)
-    {
+     {
         throw Exception(inMemFunc("getObjName"), "Object must have a name, but name length is 0");
     }
     // Object's name exists, retrieve it
     else if (name_size > 0)
-    {
+     {
         char* name_C = new char[name_size+1];  // temporary C-string
         HDmemset(name_C, 0, name_size+1); // clear buffer
 
@@ -393,13 +447,13 @@ ssize_t H5Object::getObjName(H5std_string& obj_name, size_t len) const
 
     // If no length is provided, get the entire object name
     if (len == 0)
-    {
+     {
         obj_name = getObjName();
         name_size = obj_name.length();
     }
     // If length is provided, get that number of characters in name
     else
-    {
+     {
         char* name_C = new char[len+1];  // temporary C-string
         HDmemset(name_C, 0, len+1); // clear buffer
 
@@ -418,6 +472,7 @@ ssize_t H5Object::getObjName(H5std_string& obj_name, size_t len) const
     return(name_size);
 }
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 //--------------------------------------------------------------------------
 // Function:    H5Object destructor
 ///\brief       Noop destructor.
