@@ -17,6 +17,9 @@
 #include "H5Exception.h"
 #include "H5IdComponent.h"
 #include "H5PropList.h"
+#include "H5OcreatProp.h"
+#include "H5DcreatProp.h"
+#include "H5LcreatProp.h"
 #include "H5LaccProp.h"
 #include "H5Location.h"
 #include "H5Object.h"
@@ -52,7 +55,7 @@ PredType::PredType() : AtomType() {}
 
 //--------------------------------------------------------------------------
 // Function:    PredType copy constructor
-///\brief       Copy constructor: makes a copy of the original PredType object.
+///\brief       Copy constructor: same HDF5 object as \a original
 ///\param       original - IN: PredType instance to copy
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
@@ -289,7 +292,7 @@ PredType* PredType::NATIVE_UINT_FAST64_;
 
 //--------------------------------------------------------------------------
 // Function:    PredType::getPredTypes
-// Purpose:     Returns the dummy PredType constant object pointer
+// Purpose      Returns the dummy PredType constant object pointer
 // Return:      PredType object pointer
 // Description
 //              If the dummy constant PREDTYPE_CONST_ is not allocated yet,
@@ -322,7 +325,7 @@ PredType* PredType::getPredTypes()
 
 //--------------------------------------------------------------------------
 // Function:    PredType::makePredTypes
-// Purpose:     Allocate all PredType constants.
+// Purpose      Allocate all PredType constants.
 // Programmer   Binh-Minh Ribler - September 2015
 //--------------------------------------------------------------------------
 void PredType::makePredTypes()
@@ -512,7 +515,7 @@ void PredType::makePredTypes()
 
 //--------------------------------------------------------------------------
 // Function:    PredType::deleteConstants
-// Purpose:     Deletes all PredType constant pointers.
+// Purpose      Deletes all PredType constant pointers.
 // Programmer   Binh-Minh Ribler - September 2015
 //--------------------------------------------------------------------------
 void PredType::deleteConstants()
@@ -915,7 +918,6 @@ September 2015:
                 + FileCreatPropList
                 + DSetMemXferPropList
                 + DSetCreatPropList
-        
 
         The new method includes these main points:
 
@@ -1003,7 +1005,7 @@ September 2015:
                 static void deleteConstants(); // public
 
         3.  This section shows the differences between the old and new methods
-        for allocating the DataSpace constant, DataSpace::ALL.
+            for allocating the DataSpace constant, DataSpace::ALL.
 
         Old Method:
         ----------
@@ -1038,7 +1040,7 @@ September 2015:
                 static void deleteConstants(); // public
 
         4.  This section shows the differences between the old and new methods
-        for allocating the following constants
+            for allocating the following constants
                 - PropList constant, PropList::DEFAULT.
                 - DSetCreatPropList constant, DSetCreatPropList::DEFAULT.
                 - DSetMemXferPropList constant, DSetMemXferPropList::DEFAULT.
