@@ -19,18 +19,16 @@
 
 #include <string>
 
-#include "H5private.h"          // for HDmemset
+#include "H5private.h"        // for HDmemset
 #include "H5Include.h"
 #include "H5Exception.h"
 #include "H5IdComponent.h"
 #include "H5PropList.h"
 
 
-namespace H5{
-#ifndef H5_NO_STD
+namespace H5 {
     using std::cerr;
     using std::endl;
-#endif  // H5_NO_STD
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 // This DOXYGEN_SHOULD_SKIP_THIS block is a work-around approach to control
@@ -42,7 +40,7 @@ PropList* PropList::DEFAULT_ = 0;
 
 //--------------------------------------------------------------------------
 // Function:    PropList::getConstant
-// Purpose:     Creates a PropList object representing the HDF5 constant
+// Purpose      Creates a PropList object representing the HDF5 constant
 //              H5P_DEFAULT, pointed to by PropList::DEFAULT_.
 // Exception    H5::PropListIException
 // Description
@@ -71,7 +69,7 @@ PropList* PropList::getConstant()
 
 //--------------------------------------------------------------------------
 // Function:    PropList::deleteConstants
-// Purpose:     Deletes the constant object that PropList::DEFAULT_ points to.
+// Purpose      Deletes the constant object that PropList::DEFAULT_ points to.
 // Programmer   Binh-Minh Ribler - 2015
 //--------------------------------------------------------------------------
 void PropList::deleteConstants()
@@ -88,7 +86,7 @@ const PropList& PropList::DEFAULT = *getConstant();
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
 //--------------------------------------------------------------------------
-// Function     Default constructor
+// Function:    Default constructor
 ///\brief       Default constructor: creates a stub property list object.
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
@@ -96,7 +94,7 @@ PropList::PropList() : IdComponent(), id(H5P_DEFAULT) {}
 
 //--------------------------------------------------------------------------
 // Function:    PropList copy constructor
-///\brief       Copy constructor
+///\brief       Copy constructor: same HDF5 object as \a original
 ///\param       original - IN: The original property list to copy
 // Programmer   Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
@@ -127,7 +125,7 @@ PropList::PropList(const hid_t plist_id) : IdComponent()
         case H5I_GENPROP_CLS:
           // call C routine to create a new property from the given prop class
           id = H5Pcreate(plist_id);
-          if(id < 0)
+          if (id < 0)
           {
               throw PropListIException("PropList constructor", "H5Pcreate failed");
           }
@@ -135,11 +133,29 @@ PropList::PropList(const hid_t plist_id) : IdComponent()
         case H5I_GENPROP_LST:
           // call C routine to make a copy of the given property list
           id = H5Pcopy(plist_id);
-          if(id < 0)
+          if (id < 0)
           {
               throw PropListIException("PropList constructor", "H5Pcopy failed");
           }
           break;
+        /* These should really be error cases, but changing that breaks
+         * the stated behavior and causes test failures.
+         * (DER, July 2017)
+         */
+        case H5I_BADID:
+        case H5I_FILE:
+        case H5I_GROUP:
+        case H5I_DATATYPE:
+        case H5I_DATASPACE:
+        case H5I_DATASET:
+        case H5I_ATTR:
+        case H5I_REFERENCE:
+        case H5I_VFL:
+        case H5I_ERROR_CLASS:
+        case H5I_ERROR_MSG:
+        case H5I_ERROR_STACK:
+        case H5I_NTYPES:
+        case H5I_UNINIT:
         default:
           id = H5P_DEFAULT;
           break;
@@ -171,7 +187,7 @@ void PropList::copy(const PropList& like_plist)
 
     // call C routine to copy the property list
     id = H5Pcopy(like_plist.getId());
-    if(id < 0)
+    if (id < 0)
         throw PropListIException(inMemFunc("copy"), "H5Pcopy failed");
 }
 
@@ -205,7 +221,7 @@ void PropList::copyProp(PropList& dest, const char *name) const
 {
     hid_t dst_id = dest.getId();
     herr_t ret_value = H5Pcopy_prop(dst_id, id, name);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException(inMemFunc("copyProp"), "H5Pcopy_prop failed");
     }
@@ -222,7 +238,7 @@ void PropList::copyProp(PropList& dest, const char *name) const
 //--------------------------------------------------------------------------
 void PropList::copyProp(PropList& dest, const H5std_string& name) const
 {
-   copyProp(dest, name.c_str());
+    copyProp(dest, name.c_str());
 }
 
 //--------------------------------------------------------------------------
@@ -240,7 +256,7 @@ void PropList::copyProp(PropList& dest, PropList& src, const char *name) const
     hid_t dst_id = dest.getId();
     hid_t src_id = src.getId();
     herr_t ret_value = H5Pcopy_prop(dst_id, src_id, name);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException(inMemFunc("copyProp"), "H5Pcopy_prop failed");
     }
@@ -300,8 +316,8 @@ void PropList::p_setId(const hid_t new_id)
     catch (Exception& close_error) {
         throw PropListIException(inMemFunc("p_setId"), close_error.getDetailMsg());
     }
-    // reset object's id to the given id
-    id = new_id;
+   // reset object's id to the given id
+   id = new_id;
 }
 
 #endif // DOXYGEN_SHOULD_SKIP_THIS
@@ -318,7 +334,7 @@ void PropList::close()
     if (p_valid_id(id))
     {
         herr_t ret_value = H5Pclose(id);
-        if(ret_value < 0)
+        if (ret_value < 0)
         {
             throw PropListIException(inMemFunc("close"), "H5Pclose failed");
         }
@@ -337,17 +353,17 @@ void PropList::close()
 hid_t PropList::getClass() const
 {
     hid_t plist_class = H5Pget_class(id);
-    if(plist_class == H5P_ROOT)
+    if (plist_class == H5P_ROOT)
     {
         throw PropListIException(inMemFunc("getClass"),
-                "H5Pget_class failed - returned H5P_ROOT");
+            "H5Pget_class failed - returned H5P_ROOT");
     }
-    return(plist_class);
+   return(plist_class);
 }
 
 //--------------------------------------------------------------------------
 // Function:    PropList::propExist
-///\brief       Query the existance of a property in a property object.
+///\brief       Queries the existence of a property in a property object.
 ///\param       name - IN: Name of property to check for - \c char pointer
 ///\return      true if the property exists in the property object, and
 ///             false, otherwise.
@@ -363,9 +379,9 @@ bool PropList::propExist(const char* name) const
     // within a property list or class.  It returns a positive value, 0,
     // or a negative value
     htri_t ret_value = H5Pexist(id, name);
-    if(ret_value > 0)
+    if (ret_value > 0)
         return true;
-    else if(ret_value == 0)
+    else if (ret_value == 0)
         return false;
     else // Raise exception when H5Pexist returns a negative value
     {
@@ -382,7 +398,7 @@ bool PropList::propExist(const char* name) const
 //--------------------------------------------------------------------------
 bool PropList::propExist(const H5std_string& name) const
 {
-    return(propExist(name.c_str()));
+    return(propExist( name.c_str()));
 }
 
 //--------------------------------------------------------------------------
@@ -398,7 +414,7 @@ bool PropList::propExist(const H5std_string& name) const
 void PropList::closeClass() const
 {
     herr_t ret_value = H5Pclose_class(id);
-    if(ret_value < 0)
+    if (ret_value < 0)
     {
         throw PropListIException(inMemFunc("closeClass"), "H5Pclose_class failed");
     }
@@ -531,9 +547,7 @@ size_t PropList::getPropSize(const H5std_string& name) const
 H5std_string PropList::getClassName() const
 {
     char* temp_str;
-    temp_str = H5Pget_class_name(id); // this API specified that temp_str must
-                                     // be freed.
-
+    temp_str = H5Pget_class_name(id);
     if (temp_str != NULL)
     {
         H5std_string class_name(temp_str);
@@ -567,8 +581,27 @@ size_t PropList::getNumProps() const
 ///\param       name  - IN: Name of property to set - \c char pointer
 ///\param       value - IN: Void pointer to the value for the property
 ///\exception   H5::PropListIException
+// Description
+//              Revision svn r29815 changed 'value' to const, hence, deprecated
+//              the non-const setProperty.
 // Programmer:  Binh-Minh Ribler - April, 2004
 //--------------------------------------------------------------------------
+void PropList::setProperty(const char* name, const void* value) const
+{
+    herr_t ret_value = H5Pset(id, name, value);
+    if (ret_value < 0)
+    {
+        throw PropListIException(inMemFunc("setProperty"), "H5Pset failed");
+    }
+}
+
+//--------------------------------------------------------------------------
+// Function:    PropList::setProperty
+///\brief       Deprecated due to missing const in prototype. (1.10.1)
+// Programmer:  Binh-Minh Ribler - March, 2017
+// Modification
+//              Planned for removal. -BMR, 2017/03/17 1.10.1
+//--------------------------------------------------------------------------
 void PropList::setProperty(const char* name, void* value) const
 {
     herr_t ret_value = H5Pset(id, name, value);
@@ -577,6 +610,7 @@ void PropList::setProperty(const char* name, void* value) const
         throw PropListIException(inMemFunc("setProperty"), "H5Pset failed");
     }
 }
+
 //--------------------------------------------------------------------------
 // Function:    PropList::setProperty
 ///\brief       This is an overloaded member function, provided for convenience.
@@ -584,11 +618,14 @@ void PropList::setProperty(const char* name, void* value) const
 ///             accepts.
 ///\param       name    - IN: Name of property to set - \c char pointer
 ///\param       charptr - IN: Char pointer to the value for the property
+// Description
+//              Revision svn r29815 changed 'value' to const, hence, deprecated
+//              the non-const setProperty.
 // Programmer:  Binh-Minh Ribler - April, 2004
 //--------------------------------------------------------------------------
 void PropList::setProperty(const char* name, const char* charptr) const
 {
-    herr_t ret_value = H5Pset(id, name, (void*)charptr);
+    herr_t ret_value = H5Pset(id, name, (const void*)charptr);
     if (ret_value < 0)
     {
         throw PropListIException(inMemFunc("setProperty"), "H5Pset failed");
@@ -603,6 +640,18 @@ void PropList::setProperty(const char* name, const char* charptr) const
 ///\param       strg - IN: Value for the property is a \c H5std_string
 // Programmer:  Binh-Minh Ribler - April, 2004
 //--------------------------------------------------------------------------
+void PropList::setProperty(const char* name, const H5std_string& strg) const
+{
+    setProperty(name, strg.c_str());
+}
+
+//--------------------------------------------------------------------------
+// Function:    PropList::setProperty
+///\brief       Deprecated due to missing const in prototype. (1.10.1)
+// Programmer:  Binh-Minh Ribler - March, 2017
+// Modification
+//              Planned for removal. -BMR, 2017/03/17 1.10.1
+//--------------------------------------------------------------------------
 void PropList::setProperty(const char* name, H5std_string& strg) const
 {
     setProperty(name, strg.c_str());
@@ -617,6 +666,18 @@ void PropList::setProperty(const char* name, H5std_string& strg) const
 ///\param       value - IN: Void pointer to the value for the property
 // Programmer:  Binh-Minh Ribler - April, 2004
 //--------------------------------------------------------------------------
+void PropList::setProperty(const H5std_string& name, const void* value) const
+{
+    setProperty(name.c_str(), value);
+}
+
+//--------------------------------------------------------------------------
+// Function:    PropList::setProperty
+///\brief       Deprecated due to missing const in prototype. (1.10.1)
+// Programmer:  Binh-Minh Ribler - March, 2017
+// Modification
+//              Planned for removal. -BMR, 2017/03/17 1.10.1
+//--------------------------------------------------------------------------
 void PropList::setProperty(const H5std_string& name, void* value) const
 {
     setProperty(name.c_str(), value);
@@ -631,6 +692,18 @@ void PropList::setProperty(const H5std_string& name, void* value) const
 ///\param       strg - IN: Value for the property is a \c H5std_string
 // Programmer:  Binh-Minh Ribler - April, 2004
 //--------------------------------------------------------------------------
+void PropList::setProperty(const H5std_string& name, const H5std_string& strg) const
+{
+    setProperty(name.c_str(), strg.c_str());
+}
+
+//--------------------------------------------------------------------------
+// Function:    PropList::setProperty
+///\brief       Deprecated due to missing const in prototype. (1.10.1)
+// Programmer:  Binh-Minh Ribler - March, 2017
+// Modification
+//              Planned for removal. -BMR, 2017/03/17 1.10.1
+//--------------------------------------------------------------------------
 void PropList::setProperty(const H5std_string& name, H5std_string& strg) const
 {
     setProperty(name.c_str(), strg.c_str());
@@ -648,15 +721,14 @@ void PropList::setProperty(const H5std_string& name, H5std_string& strg) const
 bool PropList::isAClass(const PropList& prop_class) const
 {
     htri_t ret_value = H5Pisa_class(id, prop_class.getId());
-    if(ret_value > 0)
+    if (ret_value > 0)
         return true;
-    else if(ret_value == 0)
+    else if (ret_value == 0)
         return false;
     else // Raise exception when H5Pisa_class returns a negative value
     {
         throw PropListIException(inMemFunc("isAClass"), "H5Pisa_class failed");
     }
-
 }
 
 //--------------------------------------------------------------------------
@@ -700,9 +772,9 @@ void PropList::removeProp(const H5std_string& name) const
 bool PropList::operator==(const PropList& rhs) const
 {
     htri_t ret_value = H5Pequal(id, rhs.getId());
-    if(ret_value > 0)
+    if (ret_value > 0)
         return true;
-    else if(ret_value == 0)
+    else if (ret_value == 0)
         return false;
     else // Raise exception when H5Pequal returns a negative value
     {
@@ -724,7 +796,7 @@ PropList PropList::getClassParent() const
     {
         throw PropListIException(inMemFunc("getClassParent"), "H5Pget_class_parent failed");
     }
-   PropList pclass(class_id);
+    PropList pclass(class_id);
     return(pclass);
 }
 
@@ -747,4 +819,5 @@ PropList::~PropList()
         cerr << "PropList::~PropList - " << close_error.getDetailMsg() << endl;
     }
 }
+
 } // end namespace
