@@ -17,6 +17,9 @@
 #include "H5Exception.h"
 #include "H5IdComponent.h"
 #include "H5PropList.h"
+#include "H5OcreatProp.h"
+#include "H5DcreatProp.h"
+#include "H5LcreatProp.h"
 #include "H5LaccProp.h"
 #include "H5Location.h"
 #include "H5Object.h"
@@ -25,7 +28,6 @@
 
 namespace H5 {
 
-
 //--------------------------------------------------------------------------
 // Function:    VarLenType default constructor
 ///\brief       Default constructor: Creates a stub variable-length datatype.
@@ -43,20 +45,21 @@ VarLenType::VarLenType(const hid_t existing_id) : DataType(existing_id) {}
 
 //--------------------------------------------------------------------------
 // Function:    VarLenType copy constructor
-///\brief       Copy constructor: makes a copy of the original VarLenType object.
+///\brief       Copy constructor: same HDF5 object as \a original
 // Programmer   Binh-Minh Ribler - May, 2004
 //--------------------------------------------------------------------------
 VarLenType::VarLenType(const VarLenType& original) : DataType(original) {}
 
 //--------------------------------------------------------------------------
 // Function:    VarLenType overloaded constructor
-///\brief       Creates a new variable-length datatype based on the specified
-///             \a base_type.
+///\brief       Deprecated - will be removed after 1.10.2
 ///\param       base_type - IN: Pointer to existing datatype
 ///\exception   H5::DataTypeIException
 // Description
 //              DataType passed by pointer to avoid clashing with copy
 //              constructor.
+//              Updated: this is unnecessary.
+//              -BMR, Sep, 2017
 // Programmer   Binh-Minh Ribler - May, 2004
 //--------------------------------------------------------------------------
 VarLenType::VarLenType(const DataType* base_type) : DataType()
@@ -69,42 +72,60 @@ VarLenType::VarLenType(const DataType* base_type) : DataType()
     }
 }
 
+//--------------------------------------------------------------------------
+// Function:    VarLenType overloaded constructor
+///\brief       Creates a new variable-length datatype based on the specified
+///             \a base_type.
+///\param       base_type - IN: An existing datatype
+///\exception   H5::DataTypeIException
+// Programmer   Binh-Minh Ribler - May, 2004
+//--------------------------------------------------------------------------
+VarLenType::VarLenType(const DataType& base_type) : DataType()
+{
+    id = H5Tvlen_create(base_type.getId());
+    if (id < 0)
+    {
+        throw DataTypeIException("VarLenType constructor",
+                "H5Tvlen_create returns negative value");
+    }
+}
+
 //--------------------------------------------------------------------------
 // Function:    VarLenType overloaded constructor
 ///\brief       Creates an VarLenType instance by opening an HDF5 variable
 ///             length datatype given its name, provided as a C char*.
-///\param       loc       - IN: Location of the type
-///\param       type_name - IN: Variable length type name
+///\param       loc        - IN: Location of the type
+///\param       dtype_name - IN: Variable length type name
 ///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - Sept 2017
+// Programmer   Binh-Minh Ribler - Dec 2016
 // Description
-//              In 1.8.20, this constructor was introduced and may replace the
+//              In 1.10.1, this constructor was introduced and may replace the
 //              existing function CommonFG::openVarLenType(const char*) to
 //              improve usability.
-//              -BMR, Sept 2017
+//              -BMR, Dec 2016
 //--------------------------------------------------------------------------
-VarLenType::VarLenType(const H5Location& loc, const char *type_name) : DataType()
+VarLenType::VarLenType(const H5Location& loc, const char *dtype_name) : DataType()
 {
-    id = p_opentype(loc, type_name);
+    id = p_opentype(loc, dtype_name);
 }
 
 //--------------------------------------------------------------------------
 // Function:    VarLenType overloaded constructor
 ///\brief       Creates an VarLenType instance by opening an HDF5 variable
 ///             length datatype given its name, provided as an \c H5std_string.
-///\param       loc       - IN: Location of the type
-///\param       type_name - IN: Variable length type name
+///\param       loc        - IN: Location of the type
+///\param       dtype_name - IN: Variable length type name
 ///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - Sept 2017
+// Programmer   Binh-Minh Ribler - Dec 2016
 // Description
-//              In 1.8.20, this constructor was introduced and may replace the
-//              existing function CommonFG::openVarLenType(const H5std_string&) to
-//              improve usability.
-//              -BMR, Sept 2017
+//              In 1.10.1, this constructor was introduced and may replace the
+//              existing function CommonFG::openVarLenType(const H5std_string&)
+//              to improve usability.
+//              -BMR, Dec 2016
 //--------------------------------------------------------------------------
-VarLenType::VarLenType(const H5Location& loc, const H5std_string& type_name) : DataType()
+VarLenType::VarLenType(const H5Location& loc, const H5std_string& dtype_name) : DataType()
 {
-    id = p_opentype(loc, type_name.c_str());
+    id = p_opentype(loc, dtype_name.c_str());
 }
 
 //--------------------------------------------------------------------------
@@ -112,7 +133,7 @@ VarLenType::VarLenType(const H5Location& loc, const H5std_string& type_name) : D
 ///\brief       Returns an VarLenType object via DataType* by decoding the
 ///             binary object description of this type.
 ///\exception   H5::DataTypeIException
-// Programmer   Binh-Minh Ribler - Sept 2017
+// Programmer   Binh-Minh Ribler - Aug 2017
 //--------------------------------------------------------------------------
 DataType* VarLenType::decode() const
 {
