@@ -32,19 +32,20 @@ using std::cerr;
 using std::endl;
 
 #include <string>
-#include "H5Cpp.h"      // C++ API header file
+#include "H5Cpp.h"        // C++ API header file
 using namespace H5;
 
-#include "h5cpputil.h"  // C++ utilility header file
+#include "h5test.h"
+#include "h5cpputil.h"    // C++ utilility header file
 
-const H5std_string      FILE1("dataset.h5");
-const H5std_string      DSET_DEFAULT_NAME("default");
-const H5std_string      DSET_DEFAULT_NAME_PATH("/default");
-const H5std_string      DSET_CHUNKED_NAME("chunked");
-const H5std_string      DSET_SIMPLE_IO_NAME("simple_io");
-const H5std_string      DSET_TCONV_NAME ("tconv");
-const H5std_string      DSET_COMPRESS_NAME("compressed");
-const H5std_string      DSET_BOGUS_NAME ("bogus");
+const H5std_string    FILE1("dataset.h5");
+const H5std_string    DSET_DEFAULT_NAME("default");
+const H5std_string    DSET_DEFAULT_NAME_PATH("/default");
+const H5std_string    DSET_CHUNKED_NAME("chunked");
+const H5std_string    DSET_SIMPLE_IO_NAME("simple_io");
+const H5std_string    DSET_TCONV_NAME("tconv");
+const H5std_string    DSET_COMPRESS_NAME("compressed");
+const H5std_string    DSET_BOGUS_NAME("bogus");
 
 /* Temporary filter IDs used for testing */
 const int H5Z_FILTER_BOGUS = 305;
@@ -57,19 +58,17 @@ static size_t filter_bogus(unsigned int flags, size_t cd_nelmts,
 /*-------------------------------------------------------------------------
  * Function:    test_create
  *
- * Purpose:     Attempts to create a dataset.
+ * Purpose      Attempts to create a dataset.
  *
- * Return:      Success:        0
+ * Return       Success: 0
  *
- *              Failure:        -1
+ *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler (using C version)
+ * Programmer   Binh-Minh Ribler (using C version)
  *              Friday, January 5, 2001
- *
  *-------------------------------------------------------------------------
  */
-static herr_t
-test_create( H5File& file)
+static herr_t test_create( H5File& file)
 {
     SUBTEST("Create, open, close");
 
@@ -100,12 +99,12 @@ test_create( H5File& file)
         // this action by createDataSet, then throw an invalid action exception.
         try {
             dataset = new DataSet (file.createDataSet
-                        (DSET_DEFAULT_NAME, PredType::NATIVE_DOUBLE, space));
+                          (DSET_DEFAULT_NAME, PredType::NATIVE_DOUBLE, space));
 
             // continuation here, that means no exception has been thrown
             throw InvalidActionException("H5File::createDataSet", "Library allowed overwrite of existing dataset");
         }
-        catch (FileIException& E)       // catching invalid creating dataset
+        catch (FileIException& E)        // catching invalid creating dataset
         {} // do nothing, exception expected
 
         // Open the dataset we created above and then close it.  This is one
@@ -148,8 +147,8 @@ test_create( H5File& file)
         csize[1] = 100;
         create_parms.setChunk( 2, csize );
 
-        dataset = new DataSet (file.createDataSet
-                (DSET_CHUNKED_NAME, PredType::NATIVE_DOUBLE, space, create_parms));
+        dataset = new DataSet (file.createDataSet(DSET_CHUNKED_NAME,
+                               PredType::NATIVE_DOUBLE, space, create_parms));
         // Note: this one has no error message in C when failure occurs?
 
         // clean up and return with success
@@ -157,7 +156,7 @@ test_create( H5File& file)
 
         PASSED();
         return 0;
-    }   // outer most try block
+    }        // outer most try block
 
     catch (InvalidActionException& E)
     {
@@ -185,28 +184,26 @@ test_create( H5File& file)
 /*-------------------------------------------------------------------------
  * Function:    test_simple_io
  *
- * Purpose:     Tests simple I/O.  That is, reading and writing a complete
+ * Purpose      Tests simple I/O.  That is, reading and writing a complete
  *              multi-dimensional array without data type or data space
  *              conversions, without compression, and stored contiguously.
  *
- * Return:      Success:        0
+ * Return       Success: 0
  *
- *              Failure:        -1
+ *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler (using C version)
+ * Programmer   Binh-Minh Ribler (using C version)
  *              Friday, January 5, 2001
- *
  *-------------------------------------------------------------------------
  */
-static herr_t
-test_simple_io( H5File& file)
+static herr_t test_simple_io( H5File& file)
 {
 
     SUBTEST("Simple I/O");
 
-    int points[100][200];
-    int check[100][200];
-    int         i, j, n;
+    int        points[100][200];
+    int        check[100][200];
+    int i, j, n;
 
     // Initialize the dataset
     for (i = n = 0; i < 100; i++)
@@ -271,19 +268,17 @@ test_simple_io( H5File& file)
 /*-------------------------------------------------------------------------
  * Function:    test_datasize
  *
- * Purpose:     Tests DataSet::getInMemDataSize().  
+ * Purpose      Tests DataSet::getInMemDataSize().  
  *
- * Return:      Success:        0
+ * Return       Success: 0
  *
- *              Failure:        -1
+ *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler
+ * Programmer   Binh-Minh Ribler
  *              Thursday, March 22, 2012
- *
  *-------------------------------------------------------------------------
  */
-static herr_t
-test_datasize(FileAccPropList &fapl)
+static herr_t test_datasize(FileAccPropList &fapl)
 {
     SUBTEST("DataSet::getInMemDataSize()");
     try
@@ -339,19 +334,17 @@ test_datasize(FileAccPropList &fapl)
 /*-------------------------------------------------------------------------
  * Function:    test_tconv
  *
- * Purpose:     Test some simple data type conversion stuff.
+ * Purpose      Test some simple data type conversion stuff.
  *
- * Return:      Success:        0
+ * Return       Success: 0
  *
- *              Failure:        -1
+ *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler (using C version)
+ * Programmer   Binh-Minh Ribler (using C version)
  *              Friday, January 5, 2001
- *
  *-------------------------------------------------------------------------
  */
-static herr_t
-test_tconv(H5File& file)
+static herr_t test_tconv(H5File& file)
 {
     // Prepare buffers for input/output
     char        *out=NULL, *in=NULL;
@@ -422,28 +415,27 @@ const H5Z_class2_t H5Z_BOGUS[1] = {{
     H5Z_CLASS_T_VERS,           /* H5Z_class_t version number   */
     H5Z_FILTER_BOGUS,           /* Filter id number             */
     1, 1,                       /* Encode and decode enabled    */
-    "bogus",                    /* Filter name for debugging    */
+    "bogus",                    /* Filter name for debugging        */
     NULL,                       /* The "can apply" callback     */
     NULL,                       /* The "set local" callback     */
-    (H5Z_func_t)filter_bogus,   /* The actual filter function   */
+    (H5Z_func_t)filter_bogus,   /* The actual filter function        */
 }};
 
+
 /*-------------------------------------------------------------------------
  * Function:    bogus
  *
- * Purpose:     A bogus compression method that doesn't do anything.
+ * Purpose      A bogus compression method that doesn't do anything.
  *
- * Return:      Success:        Data chunk size
+ * Return       Success: Data chunk size
  *
- *              Failure:        0
+ *              Failure: 0
  *
- * Programmer:  Robb Matzke
+ * Programmer   Robb Matzke
  *              Tuesday, April 21, 1998
- *
  *-------------------------------------------------------------------------
  */
-static size_t
-filter_bogus(unsigned int flags, size_t cd_nelmts,
+static size_t filter_bogus(unsigned int flags, size_t cd_nelmts,
       const unsigned int cd_values[], size_t nbytes,
       size_t *buf_size, void **buf)
 // H5_ATTR_UNUSED variables caused warning, but taking them out caused failure.
@@ -455,36 +447,34 @@ filter_bogus(unsigned int flags, size_t cd_nelmts,
 /*-------------------------------------------------------------------------
  * Function:    test_compression
  *
- * Purpose:     Tests dataset compression. If compression is requested when
+ * Purpose      Tests dataset compression. If compression is requested when
  *              it hasn't been compiled into the library (such as when
  *              updating an existing compressed dataset) then data is sent to
  *              the file uncompressed but no errors are returned.
  *
- * Return:      Success:        0
+ * Return       Success: 0
  *
- *              Failure:        -1
+ *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler (using C version)
+ * Programmer   Binh-Minh Ribler (using C version)
  *              Friday, January 5, 2001
- *
  *-------------------------------------------------------------------------
  */
-static herr_t
-test_compression(H5File& file)
+static herr_t test_compression(H5File& file)
 {
 #ifndef H5_HAVE_FILTER_DEFLATE
-    const char          *not_supported;
+    const char *not_supported;
     not_supported = "    Deflate compression is not enabled.";
 #endif /* H5_HAVE_FILTER_DEFLATE */
-    int         points[100][200];
-    int         check[100][200];
-    hsize_t     i, j, n;
+    int points[100][200];
+    int check[100][200];
+    hsize_t        i, j, n;
 
     // Initialize the dataset
     for (i = n = 0; i < 100; i++)
     {
         for (j = 0; j < 200; j++) {
-            points[i][j] = (int)n++;
+            points[i][j] = static_cast<int>(n++);
         }
     }
 
@@ -492,7 +482,7 @@ test_compression(H5File& file)
     DataSet* dataset = NULL;
     try
     {
-        const hsize_t   size[2] = {100, 200};
+        const hsize_t size[2] = {100, 200};
         // Create the data space
         DataSpace space1(2, size, NULL);
 
@@ -504,7 +494,7 @@ test_compression(H5File& file)
         // Use chunked storage with compression
         DSetCreatPropList dscreatplist;
 
-        const hsize_t   chunk_size[2] = {2, 25};
+        const hsize_t chunk_size[2] = {2, 25};
         dscreatplist.setChunk (2, chunk_size);
         dscreatplist.setDeflate (6);
 
@@ -530,8 +520,8 @@ test_compression(H5File& file)
                 if (0!=check[i][j]) {
                     H5_FAILED();
                     cerr << "    Read a non-zero value." << endl;
-                    cerr << "    At index " << (unsigned long)i << "," <<
-                   (unsigned long)j << endl;
+                    cerr << "    At index " << static_cast<unsigned long>(i)
+                         << "," << static_cast<unsigned long>(j) << endl;
                     throw Exception("test_compression", "Failed in uninitialized read");
                 }
             }
@@ -549,7 +539,7 @@ test_compression(H5File& file)
         {
             for (j=0; j<size[1]; j++)
             {
-                points[i][j] = (int)n++;
+                points[i][j] = static_cast<int>(n++);
             }
         }
 
@@ -590,7 +580,7 @@ test_compression(H5File& file)
         {
             for (j=0; j<size[1]/2; j++)
             {
-                points[i][j] = rand ();
+                    points[i][j] = rand ();
             }
         }
         dataset->write (static_cast<void*>(points), PredType::NATIVE_INT, DataSpace::ALL, DataSpace::ALL, xfer);
@@ -643,8 +633,8 @@ test_compression(H5File& file)
         */
         SUBTEST("Compression (partial I/O)");
 
-        const hsize_t   hs_size[2] = {4, 50};
-        const hsize_t   hs_offset[2] = {7, 30};
+        const hsize_t hs_size[2] = {4, 50};
+        const hsize_t hs_offset[2] = {7, 30};
         for (i = 0; i < hs_size[0]; i++) {
             for (j = 0; j < hs_size[1]; j++) {
                 points[hs_offset[0]+i][hs_offset[1]+j] = rand ();
@@ -661,11 +651,11 @@ test_compression(H5File& file)
                 check[hs_offset[0]+i][hs_offset[1]+j]) {
                 H5_FAILED();
                 cerr << "    Read different values than written.\n" << endl;
-                cerr << "    At index " << (unsigned long)(hs_offset[0]+i) <<
-                   "," << (unsigned long)(hs_offset[1]+j) << endl;
+                cerr << "    At index " << static_cast<unsigned long>((hs_offset[0]+i)) <<
+                    "," << static_cast<unsigned long>((hs_offset[1]+j)) << endl;
 
-                cerr << "    At original: " << (int)points[hs_offset[0]+i][hs_offset[1]+j] << endl;
-                cerr << "    At returned: " << (int)check[hs_offset[0]+i][hs_offset[1]+j] << endl;
+                cerr << "    At original: " << static_cast<int>(points[hs_offset[0]+i][hs_offset[1]+j]) << endl;
+                cerr << "    At returned: " << static_cast<int>(check[hs_offset[0]+i][hs_offset[1]+j]) << endl;
                 throw Exception("test_compression", "Failed in partial I/O");
             }
         } // for j
@@ -740,20 +730,21 @@ test_compression(H5File& file)
 /*-------------------------------------------------------------------------
  * Function:    test_nbit_methods
  *
- * Purpose:     Tests setting nbit compression methods.
+ * Purpose      Tests setting nbit compression methods.
  *
- * Return:      Success:        0
+ * Return       Success: 0
  *
- *              Failure:        -1
+ *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler
+ * Programmer   Binh-Minh Ribler
  *              Friday, April 22, 2016
  *
  *-------------------------------------------------------------------------
  */
-const H5std_string      DSET_NBIT_NAME("nbit_dataset");
+const H5std_string DSET_NBIT_NAME("nbit_dataset");
 const hsize_t DIM1 = 2;
 const hsize_t DIM2 = 5;
+
 static herr_t test_nbit_compression(H5File& file)
 {
     typedef struct {
@@ -856,22 +847,20 @@ static herr_t test_nbit_compression(H5File& file)
 /*-------------------------------------------------------------------------
  * Function:    test_multiopen
  *
- * Purpose:     Tests that a bug no longer exists.  If a dataset is opened
+ * Purpose      Tests that a bug no longer exists.  If a dataset is opened
  *              twice and one of the handles is used to extend the dataset,
  *              then the other handle should return the new size when
  *              queried.
  *
- * Return:      Success:        0
+ * Return       Success: 0
  *
- *              Failure:        -1
+ *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler (using C version)
+ * Programmer   Binh-Minh Ribler (using C version)
  *              Saturday, February 17, 2001
- *
  *-------------------------------------------------------------------------
  */
-static herr_t
-test_multiopen (H5File& file)
+static herr_t test_multiopen (H5File& file)
 {
 
     SUBTEST("Multi-open with extending");
@@ -883,7 +872,7 @@ test_multiopen (H5File& file)
         DSetCreatPropList dcpl;
 
         // Set chunk size to given size
-        hsize_t         cur_size[1] = {10};
+        hsize_t cur_size[1] = {10};
         dcpl.setChunk (1, cur_size);
 
         // Create a simple data space with unlimited size
@@ -907,12 +896,13 @@ test_multiopen (H5File& file)
         // Get the size from the second handle
         space = new DataSpace (dset2.getSpace());
 
-        hsize_t         tmp_size[1];
+        hsize_t tmp_size[1];
         space->getSimpleExtentDims (tmp_size);
         if (cur_size[0]!=tmp_size[0])
         {
-            cerr << "    Got " << (int)tmp_size[0] << " instead of "
-                    << (int)cur_size[0] << "!" << endl;
+            cerr << "    Got " << static_cast<int>(tmp_size[0])
+                 << " instead of " << static_cast<int>(cur_size[0]) 
+                 << "!" << endl;
             throw Exception("test_multiopen", "Failed in multi-open with extending");
         }
 
@@ -939,23 +929,21 @@ test_multiopen (H5File& file)
 /*-------------------------------------------------------------------------
  * Function:    test_types
  *
- * Purpose:     Test various types - should be moved to dtypes.cpp
+ * Purpose      Test various types - should be moved to dtypes.cpp
  *
- * Return:      Success:        0
+ * Return       Success: 0
  *
- *              Failure:        -1
+ *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler (using C version)
+ * Programmer   Binh-Minh Ribler (using C version)
  *              February 17, 2001
- *
  *-------------------------------------------------------------------------
  */
-static herr_t
-test_types(H5File& file)
+static herr_t test_types(H5File& file)
 {
     SUBTEST("Various datatypes");
 
-    size_t              i;
+    size_t i;
     DataSet* dset = NULL;
     try {
 
@@ -963,7 +951,7 @@ test_types(H5File& file)
         Group grp = file.createGroup ("typetests");
 
         /* bitfield_1 */
-        unsigned char   buf[32];
+        unsigned char buf[32];
         hsize_t nelmts = sizeof(buf);
         DataType type;
         try { // block of bitfield_1
@@ -995,7 +983,7 @@ test_types(H5File& file)
 
             // Fill buffer
             for (i=0; i<sizeof buf; i++)
-                buf[i] = (unsigned char)0xff ^ (unsigned char)i;
+                    buf[i] = static_cast<unsigned char>(0xff) ^ static_cast<unsigned char>(i);
 
             // Write data from buf using all default dataspaces and property list
             dset->write (buf, type);
@@ -1024,7 +1012,7 @@ test_types(H5File& file)
 
             // Fill buffer
             for (i=0; i<sizeof(buf); i++)
-                buf[i] = (unsigned char)0xff ^ (unsigned char)i;
+                buf[i] = static_cast<unsigned char>(0xff) ^ static_cast<unsigned char>(i);
 
             // Write data from buf using all default dataspaces and property
             // list; if writing fails, deallocate dset and return.
@@ -1058,7 +1046,7 @@ test_types(H5File& file)
 
             // Fill buffer
             for (i=0; i<sizeof buf; i++)
-                buf[i] = (unsigned char)0xff ^ (unsigned char)i;
+                buf[i] = static_cast<unsigned char>(0xff) ^ static_cast<unsigned char>(i);
 
             // Write data from buf using all default dataspaces and property
             // list; if writing fails, deallocate dset and return.
@@ -1092,7 +1080,7 @@ test_types(H5File& file)
 
             // Fill buffer
             for (i=0; i<sizeof(buf); i++)
-                buf[i] = (unsigned char)0xff ^ (unsigned char)i;
+                buf[i] = static_cast<unsigned char>(0xff) ^ static_cast<unsigned char>(i);
 
             // Write data from buf using all default dataspaces and property
             // list; if writing fails, deallocate dset and return.
@@ -1126,25 +1114,104 @@ test_types(H5File& file)
     }
 }   // test_types
 
+
+/*-------------------------------------------------------------------------
+ * Function:    test_virtual
+ *
+ * Purpose      Tests fixed, unlimited, and printf selections in the same
+ *              VDS
+ *
+ * Return       Success:        0
+ *
+ *              Failure:        number of errors
+ *
+ * Programmer   Binh-Minh Ribler
+ *              Friday, March 10, 2017
+ *
+ *-------------------------------------------------------------------------
+ */
+const int RANK = 2;
+
+static herr_t test_virtual()
+{
+    SUBTEST("DSetCreatPropList::setVirtual");
+
+    try {
+        // Create DCPLs
+        DSetCreatPropList dcpl;
+        DSetCreatPropList srcdcpl;
+
+        // Set fill value
+        char *fill = NULL;
+        dcpl.setFillValue(PredType::NATIVE_INT, &fill);
+
+        // Set chunk dimensions
+        hsize_t cdims[RANK];
+        cdims[0] = 2;
+        cdims[1] = 2;
+        srcdcpl.setChunk(RANK, cdims);
+
+        // Create memory space
+        hsize_t mdims[RANK];
+        mdims[0] = 10;
+        mdims[1] = 10;
+        DataSpace memspace(RANK, mdims);
+
+        // Get the current layout, should be default, H5D_CONTIGUOUS
+        H5D_layout_t layout = dcpl.getLayout();
+        verify_val(layout, H5D_CONTIGUOUS, "DSetCreatPropList::getLayout", __LINE__, __FILE__);
+
+        // Create fixed mapping
+        hsize_t dims[RANK];
+        dims[0] = 6;
+        dims[1] = 6;
+        DataSpace vspace(RANK, dims, mdims);
+
+        hsize_t start[RANK];       // Hyperslab start
+        hsize_t count[RANK];       // Hyperslab count
+        start[0] = start[1] = 3;
+        count[0] = count[1] = 3;
+        vspace.selectHyperslab(H5S_SELECT_SET, count, start);
+
+        DataSpace srcspace(RANK, count);
+
+        H5std_string src_file = "src_file_map.h5";
+        H5std_string src_dset = "src_dset_fixed";
+        dcpl.setVirtual(vspace, src_file, src_dset, srcspace);
+
+        // Get and verify the new layout
+        layout = dcpl.getLayout();
+        verify_val(layout, H5D_VIRTUAL, "DSetCreatPropList::getLayout", __LINE__, __FILE__);
+
+        PASSED();
+        return 0;
+    } // end top try block
+
+    catch (Exception& E)
+    {
+        return -1;
+    }
+} // test_virtual
+
 
 /*-------------------------------------------------------------------------
  * Function:    test_dset
  *
- * Purpose:     Tests the dataset interface (H5D)
+ * Purpose      Tests the dataset interface (H5D)
  *
- * Return:      Success: 0
+ * Return       Success: 0
  *
  *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler (using C version)
+ * Programmer   Binh-Minh Ribler (using C version)
  *              Friday, January 5, 2001
  *
  * Modifications:
- *      Nov 12, 01:
+ *        Nov 12, 01:
  *              - moved h5_cleanup to outside of try block because
  *                dataset.h5 cannot be removed until "file" is out of
  *                scope and dataset.h5 is closed.
- *      Feb 20, 05:
+ *        Feb 20, 05:
  *              - cleanup_dsets took care of the cleanup now.
  *
  *-------------------------------------------------------------------------
@@ -1152,10 +1219,9 @@ test_types(H5File& file)
 extern "C"
 void test_dset()
 {
-    hid_t       fapl_id;
+    hid_t fapl_id;
     fapl_id = h5_fileaccess(); // in h5test.c, returns a file access template
-
-    int         nerrors=0;      // keep track of number of failures occurr
+    int nerrors=0; // keep track of number of failures occur
 
     try
     {
@@ -1176,6 +1242,7 @@ void test_dset()
         nerrors += test_nbit_compression(file) < 0 ? 1:0;
         nerrors += test_multiopen (file) < 0 ? 1:0;
         nerrors += test_types(file) < 0 ? 1:0;
+        nerrors += test_virtual() < 0 ? 1:0;
 
         // Close group "emit diagnostics".
         grp.close();
@@ -1194,15 +1261,15 @@ void test_dset()
     cleanup_dsets();
 }   // test_dset
 
+
 /*-------------------------------------------------------------------------
  * Function:    cleanup_dsets
  *
- * Purpose:     Cleanup temporary test files
+ * Purpose      Cleanup temporary test files
  *
- * Return:      none
- *
- * Programmer:  (use C version)
+ * Return       None
  *
+ * Programmer   (use C version)
  *-------------------------------------------------------------------------
  */
 extern "C"
