@@ -17,7 +17,6 @@
         C attribute interface (H5A)
 
  ***************************************************************************/
-
 #ifdef OLD_HEADER_FILENAME
 #include <iostream.h>
 #else
@@ -30,16 +29,17 @@ using std::endl;
 #include "H5Cpp.h"      // C++ API header file
 using namespace H5;
 
+#include "h5test.h"
 #include "h5cpputil.h"  // C++ utilility header file
 
-const H5std_string      FILE_BASIC("tattr_basic.h5");
-const H5std_string      FILE_COMPOUND("tattr_compound.h5");
-const H5std_string      FILE_SCALAR("tattr_scalar.h5");
-const H5std_string      FILE_MULTI("tattr_multi.h5");
-const H5std_string      FILE_DTYPE("tattr_dtype.h5");
-const H5std_string      ATTR_TMP_NAME("temp_attr_name");
-const H5std_string      FATTR_TMP_NAME("temp_fattr_name");
-const size_t            ATTR_MAX_DIMS = 7;
+const H5std_string FILE_BASIC("tattr_basic.h5");
+const H5std_string FILE_COMPOUND("tattr_compound.h5");
+const H5std_string FILE_SCALAR("tattr_scalar.h5");
+const H5std_string FILE_MULTI("tattr_multi.h5");
+const H5std_string FILE_DTYPE("tattr_dtype.h5");
+const H5std_string ATTR_TMP_NAME("temp_attr_name");
+const H5std_string FATTR_TMP_NAME("temp_fattr_name");
+const size_t ATTR_MAX_DIMS = 7;
 
 /* 3-D dataset with fixed dimensions */
 const int SPACE1_RANK = 3;
@@ -95,18 +95,21 @@ struct attr4_struct {
 
 const H5std_string ATTR5_NAME("Attr5");
 const int ATTR5_RANK = 0;
-float attr_data5 = (float)-5.123;       // Test data for 5th attribute
+float attr_data5 = (float)-5.123;     // Test data for 5th attribute
 
 /* Info for another attribute */
 const H5std_string ATTR1A_NAME("Attr1_a");
 int attr_data1a[ATTR1_DIM1]={256,11945,-22107};
 
-/****************************************************************
-**
-**  test_attr_basic_write(): Test basic write attribute.
-**      Tests integer attributes on both datasets and groups
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_basic_write
+ *
+ * Purpose      Test basic write attribute on both datasets and groups.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_basic_write()
 {
     hsize_t dims1[] = {SPACE1_DIM1, SPACE1_DIM2, SPACE1_DIM3};
@@ -126,7 +129,7 @@ static void test_attr_basic_write()
         DataSpace ds_space (SPACE1_RANK, dims1);
 
         /*
-         *              Test attribute with dataset
+         *                Test attribute with dataset
          */
 
         // Create a dataset
@@ -189,7 +192,7 @@ static void test_attr_basic_write()
         ds_attr3.close();
 
         /*
-         *              Test attribute with group
+         *                Test attribute with group
          */
 
         // Create group in file fid1
@@ -204,14 +207,14 @@ static void test_attr_basic_write()
         // Check storage size for attribute
         hsize_t attr_size = gr_attr.getStorageSize();
         verify_val((long)attr_size, (long)(ATTR2_DIM1*ATTR2_DIM2*sizeof(int)),
-                        "Attribute::getStorageSize",__LINE__,__FILE__);
+                "Attribute::getStorageSize",__LINE__,__FILE__);
 
         // Try to create the same attribute again (should fail)
         try {
             Attribute invalid_attr = group.createAttribute (ATTR2_NAME, PredType::NATIVE_INT, sid3);
             // continuation here, that means no exception has been thrown
             throw InvalidActionException("H5Group::createAttribute",
-                        "Attempting to create an existing attribute");
+                "Attempting to create an existing attribute");
         }
         catch (AttributeIException& E) // catching invalid creating attribute
         {} // do nothing, exception expected
@@ -222,7 +225,7 @@ static void test_attr_basic_write()
         // Check storage size for attribute
         attr_size = gr_attr.getStorageSize();
         verify_val((long)attr_size, (long)(ATTR2_DIM1*ATTR2_DIM2*sizeof(int)),
-                        "Attribute::getStorageSize",  __LINE__, __FILE__);
+                "Attribute::getStorageSize",  __LINE__, __FILE__);
 
         PASSED();
     } // end try block
@@ -233,29 +236,34 @@ static void test_attr_basic_write()
     }
 }   // test_attr_basic_write()
 
-/****************************************************************
-**
-**  test_attr_getname(): Test getting attribute name functions.
-**
-**  Test these functions:
-**  A. ssize_t Attribute::getName(char* attr_name, size_t buf_size)
-**    1. With arbitrary buf_size that is larger than the name size
-**    2. With arbitrary buf_size that is smaller than the name's length.
-**    3. With a buf_size that equals the name's length.
-**
-**  B. ssize_t Attribute::getName(H5std_string& attr_name, size_t buf_size)
-**    1. With buffer smaller than the actual name
-**    2. Same test but with retiring overloaded function
-**      ssize_t Attribute::getName(size_t buf_size, H5std_string& attr_name)
-**
-**  C. H5std_string Attribute::getName()
-**
-**  D. H5std_string Attribute::getName(size_t len)
-**
-**  E. ssize_t Attribute::getName(H5std_string& attr_name, size_t buf_size)
-**      With buffer size equals the name's length, i.e., buf_size=0
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_getname
+ *
+ * Purpose      Test getting attribute name.
+ *
+ * Description
+ *  Test these functions:
+ *  A. ssize_t Attribute::getName(char* attr_name, size_t buf_size)
+ *    1. With arbitrary buf_size that is larger than the name size
+ *    2. With arbitrary buf_size that is smaller than the name's length.
+ *    3. With a buf_size that equals the name's length.
+ *
+ *  B. ssize_t Attribute::getName(H5std_string& attr_name, size_t buf_size)
+ *    1. With buffer smaller than the actual name
+ *    2. Same test but with retiring overloaded function
+ *        ssize_t Attribute::getName(size_t buf_size, H5std_string& attr_name)
+ *
+ *  C. H5std_string Attribute::getName()
+ *
+ *  D. H5std_string Attribute::getName(size_t len)
+ *
+ *  E. ssize_t Attribute::getName(H5std_string& attr_name, size_t buf_size)
+ *        With buffer size equals the name's length, i.e., buf_size=0
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_getname()
 {
     // Output message about test being performed
@@ -368,11 +376,15 @@ static void test_attr_getname()
     }
 }   // test_attr_getname()
 
-/****************************************************************
-**
-**  test_attr_rename(): Test renaming attribute function.
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_rename
+ *
+ * Purpose      Test renaming attribute function.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_rename()
 {
     int read_data1[ATTR1_DIM1]={0}; // Buffer for reading the attribute
@@ -437,7 +449,7 @@ static void test_attr_rename()
                 TestErrPrintf("%d: attribute data different: attr_data1[%d]=%d,read_data1[%d]=%d\n",__LINE__,i,attr_data1[i],i,read_data1[i]);
 
         // Close attribute
-        attr1.close();
+            attr1.close();
 
         // Check for existence of second attribute
         attr_exists = dataset.attrExists(ATTR2_NAME);
@@ -479,11 +491,15 @@ static void test_attr_rename()
     }
 }   // test_attr_rename()
 
-/********************************************************************
-**
-**  test_attr_basic_read(): Test basic read attribute.
-**
-********************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_basic_read
+ *
+ * Purpose      Test basic read attribute.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_basic_read()
 {
     hsize_t i, j;
@@ -515,7 +531,7 @@ static void test_attr_basic_read()
                 TestErrPrintf("%d: attribute data different: attr_data1[%d]=%d, read_data1[%d]=%d\n",__LINE__,i,attr_data1[i],i,read_data1[i]);
 
         /*
-         *              Test attribute with group
+         *                Test attribute with group
          */
         // Open the group
         Group group = fid1.openGroup(GROUP1_NAME);
@@ -548,11 +564,15 @@ static void test_attr_basic_read()
     }
 }   // test_attr_basic_read()
 
-/****************************************************************
-**
-**  test_attr_compound_write(): Tests compound datatype attributes
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_compound_write
+ *
+ * Purpose      Test writing attributes with compound datatype.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_compound_write()
 {
 
@@ -564,7 +584,7 @@ static void test_attr_compound_write()
         H5File fid1(FILE_COMPOUND.c_str(), H5F_ACC_TRUNC);
 
         // Create dataspace for dataset
-        hsize_t         dims1[] = {SPACE1_DIM1, SPACE1_DIM2, SPACE1_DIM3};
+        hsize_t dims1[] = {SPACE1_DIM1, SPACE1_DIM2, SPACE1_DIM3};
         DataSpace sid1(SPACE1_RANK, dims1);
 
         // Create a dataset
@@ -583,7 +603,7 @@ static void test_attr_compound_write()
         comp_type.insertMember(ATTR4_FIELDNAME3, attr4_field3_off, PredType::NATIVE_SCHAR);
 
         // Create dataspace for 1st attribute
-        hsize_t         dims2[] = {ATTR4_DIM1,ATTR4_DIM2};
+        hsize_t dims2[] = {ATTR4_DIM1,ATTR4_DIM2};
         DataSpace sid2(ATTR4_RANK, dims2);
 
         // Create complex attribute for the dataset
@@ -608,11 +628,15 @@ static void test_attr_compound_write()
     }
 }   // test_attr_compound_write()
 
-/****************************************************************
-**
-**  test_attr_compound_read(): Test basic H5A (attribute) code.
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_compound_read
+ *
+ * Purpose      Test reading attributes with compound datatype.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_compound_read()
 {
     hsize_t dims[ATTR_MAX_DIMS];        // Attribute dimensions
@@ -672,8 +696,8 @@ static void test_attr_compound_read()
         {
             H5std_string fieldname = datatype.getMemberName(j);
             if(!((fieldname == ATTR4_FIELDNAME1) ||
-                (fieldname == ATTR4_FIELDNAME2) ||
-                (fieldname == ATTR4_FIELDNAME3)))
+                 (fieldname == ATTR4_FIELDNAME2) ||
+                 (fieldname == ATTR4_FIELDNAME3)))
             TestErrPrintf("%d:invalid field name for field #%d: %s\n",__LINE__,j,fieldname.c_str());
         } /* end for */
 
@@ -760,11 +784,15 @@ static void test_attr_compound_read()
     }
 }   // test_attr_compound_read()
 
-/****************************************************************
-**
-**  test_attr_scalar_write(): Test scalar attribute writing functionality.
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_scalar_write
+ *
+ * Purpose      Test scalar attribute writing functionality.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_scalar_write()
 {
     // Output message about test being performed
@@ -815,11 +843,18 @@ static void test_attr_scalar_write()
     }
 }   // test_attr_scalar_write()
 
-/****************************************************************
-**
-**  test_attr_scalar_read(): Test scalar attribute reading functionality.
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_scalar_read
+ *
+ * Purpose      Test scalar attribute reading functionality.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
+/* Epsilon for floating-point comparisons */
+#define FP_EPSILON 0.000001F
+
 static void test_attr_scalar_read()
 {
     // Output message about test being performed
@@ -842,7 +877,8 @@ static void test_attr_scalar_read()
         // Read attribute information
         float read_data2=0.0;  // Buffer for reading 1st attribute
         ds_attr.read(PredType::NATIVE_FLOAT,&read_data2);
-        verify_val(read_data2, attr_data5, "Attribute::read", __LINE__, __FILE__);
+        if (HDfabs(read_data2 - attr_data5) > FP_EPSILON)
+            verify_val(read_data2, attr_data5, FP_EPSILON, "Attribute::read", __LINE__, __FILE__);
 
         // Get the dataspace of the attribute
         DataSpace att_space = ds_attr.getSpace();
@@ -860,11 +896,15 @@ static void test_attr_scalar_read()
     }
 }   // test_attr_scalar_read()
 
-/****************************************************************
-**
-**  test_attr_mult_write(): Test multiple attributes
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_mult_write
+ *
+ * Purpose      Test writing multiple attributes.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_mult_write()
 {
     // Output message about test being performed
@@ -933,11 +973,15 @@ static void test_attr_mult_write()
     }
 }   // test_attr_mult_write()
 
-/****************************************************************
-**
-**  test_attr_mult_read(): Test reading multiple attributes.
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_mult_read
+ *
+ * Purpose      Test reading multiple attributes.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_mult_read()
 {
     int     read_data1[ATTR1_DIM1]={0}; // Buffer for reading 1st attribute
@@ -985,7 +1029,7 @@ static void test_attr_mult_read()
         // Verify that the type is of integer datatype
         verify_val(type_class, H5T_INTEGER, "Attribute::getTypeClass", __LINE__, __FILE__);
 
-        // Get the integer datatype
+            // Get the integer datatype
         IntType i_type1 = attr.getIntType();
 
         // Get and verify the order of this type
@@ -1037,7 +1081,7 @@ static void test_attr_mult_read()
         // Verify that the type is of integer datatype
         verify_val(type_class, H5T_INTEGER, "Attribute::getTypeClass", __LINE__, __FILE__);
 
-        // Get the integer datatype
+            // Get the integer datatype
         IntType i_type2 = attr.getIntType();
 
         // Get and verify the order of this type
@@ -1090,7 +1134,7 @@ static void test_attr_mult_read()
         // Verify that the type is of compound datatype
         verify_val(type_class, H5T_FLOAT, "Attribute::getTypeClass", __LINE__, __FILE__);
 
-        // Get the double datatype
+            // Get the double datatype
         FloatType f_type = attr.getFloatType();
 
         // Get and verify the order of this type
@@ -1124,16 +1168,18 @@ static void test_attr_mult_read()
     }
 }   // test_attr_mult_read()
 
-/****************************************************************
-**
-**  test_attr_delete(): Test deleting attribute from different
-**                      hdf5 objects.
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_delete
+ *
+ * Purpose      Test deleting attribute from different hdf5 objects.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_delete()
 {
     H5std_string  attr_name; // Buffer for attribute names
-    int ii;
 
     // Output message about test being performed
     SUBTEST("Removing Attribute Function");
@@ -1242,20 +1288,23 @@ static void test_attr_delete()
     }
 }   // test_attr_delete()
 
-/****************************************************************
-**
-**  test_attr_dtype_shared(): Test code for using shared datatypes
-**                              in attributes.
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_dtype_shared
+ *
+ * Purpose      Test accessing attributes using shared datatypes.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_dtype_shared()
 {
-    int data=8;                 /* Data to write */
-    int rdata=0;                /* Read read in */
+    int data=8;                 // Data to write
+    int rdata=0;                // Data read in
 #ifndef H5_NO_DEPRECATED_SYMBOLS
-    H5G_stat_t statbuf;         /* Object's information */
-#endif /* H5_NO_DEPRECATED_SYMBOLS */
-    h5_stat_size_t filesize;             /* Size of file after modifications */
+    H5G_stat_t statbuf;         // Object's information
+#endif
+    h5_stat_size_t filesize;    // Size of file after modifications
 
     // Output message about test being performed
     SUBTEST("Shared Datatypes with Attributes");
@@ -1287,19 +1336,11 @@ static void test_attr_dtype_shared()
         // Commit datatype to file
         dtype.commit(fid1, TYPE1_NAME);
 
-        // Retrieve and verify information about the type
-        H5O_info_t oinfo;
-        fid1.getObjectInfo(TYPE1_NAME, &oinfo);
-        if (oinfo.type != H5O_TYPE_NAMED_DATATYPE)
-            TestErrPrintf("Line %d: object type wrong!\n", __LINE__);
-        verify_val(oinfo.num_attrs, 0, "DataType::getObjinfo reference count", __LINE__, __FILE__);
-        verify_val((int)oinfo.rc, 1, "DataType::getObjinfo reference count", __LINE__, __FILE__);
-
 #ifndef H5_NO_DEPRECATED_SYMBOLS
         // Check reference count on named datatype
         fid1.getObjinfo(TYPE1_NAME, statbuf);
         verify_val((int)statbuf.nlink, 1, "DataType::getObjinfo", __LINE__, __FILE__);
-#endif /* H5_NO_DEPRECATED_SYMBOLS */
+#endif
 
         // Create dataspace for dataset
         DataSpace dspace;
@@ -1310,7 +1351,7 @@ static void test_attr_dtype_shared()
         // Check reference count on named datatype
         fid1.getObjinfo(TYPE1_NAME, statbuf);
         verify_val((int)statbuf.nlink, 2, "H5File::getObjinfo", __LINE__, __FILE__);
-#endif /* H5_NO_DEPRECATED_SYMBOLS */
+#endif
 
         // Create attribute on dataset
         Attribute attr = dset.createAttribute(ATTR1_NAME,dtype,dspace);
@@ -1319,7 +1360,7 @@ static void test_attr_dtype_shared()
         // Check reference count on named datatype
         fid1.getObjinfo(TYPE1_NAME, statbuf);
         verify_val((int)statbuf.nlink, 3, "DataSet::getObjinfo", __LINE__, __FILE__);
-#endif /* H5_NO_DEPRECATED_SYMBOLS */
+#endif
 
         // Close attribute
         attr.close();
@@ -1331,7 +1372,7 @@ static void test_attr_dtype_shared()
         // Check reference count on named datatype
         fid1.getObjinfo(TYPE1_NAME, statbuf);
         verify_val((int)statbuf.nlink, 2, "DataSet::getObjinfo after DataSet::removeAttr", __LINE__, __FILE__);
-#endif /* H5_NO_DEPRECATED_SYMBOLS */
+#endif
 
         // Create attribute on dataset
         attr = dset.createAttribute(ATTR1_NAME,dtype,dspace);
@@ -1340,7 +1381,7 @@ static void test_attr_dtype_shared()
         // Check reference count on named datatype
         fid1.getObjinfo(TYPE1_NAME, statbuf);
         verify_val((int)statbuf.nlink, 3, "DataSet::createAttribute", __LINE__, __FILE__);
-#endif /* H5_NO_DEPRECATED_SYMBOLS */
+#endif
 
         // Write data into the attribute
         attr.write(PredType::NATIVE_INT,&data);
@@ -1377,7 +1418,7 @@ static void test_attr_dtype_shared()
         // Check reference count on named datatype
         fid1.getObjinfo(TYPE1_NAME, statbuf);
         verify_val((int)statbuf.nlink, 3, "DataSet::openAttribute", __LINE__, __FILE__);
-#endif /* H5_NO_DEPRECATED_SYMBOLS */
+#endif
         } // end of second enclosing
 
         // Unlink the dataset
@@ -1387,7 +1428,7 @@ static void test_attr_dtype_shared()
         // Check reference count on named datatype
         fid1.getObjinfo(TYPE1_NAME, statbuf);
         verify_val((int)statbuf.nlink, 1, "H5File::unlink", __LINE__, __FILE__);
-#endif /* H5_NO_DEPRECATED_SYMBOLS */
+#endif
 
         // Unlink the named datatype
         fid1.unlink(TYPE1_NAME);
@@ -1408,12 +1449,15 @@ static void test_attr_dtype_shared()
     }
 }   // test_attr_dtype_shared()
 
-/****************************************************************
-**
-**  test_string_attr(): Test read/write string attribute.
-**      Tests string attributes on groups.
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_string_attr
+ *
+ * Purpose      Test read/write string attribute.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 /* Info for a string attribute */
 const H5std_string ATTR1_FL_STR_NAME("String_attr 1");
 const H5std_string ATTR2_FL_STR_NAME("String_attr 2");
@@ -1533,12 +1577,18 @@ static void test_string_attr()
     }
 }   // test_string_attr()
 
-/****************************************************************
-**
-**  test_attr_exists(): Test checking for attribute existence.
-**      (additional attrExists tests are in test_attr_rename())
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_exists
+ *
+ * Purpose      Test checking for attribute existence.
+ *
+ * Return       None
+ *
+ * Note
+ *              Additional attrExists tests are in test_attr_rename().
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_exists()
 {
     // Output message about test being performed
@@ -1564,20 +1614,11 @@ static void test_attr_exists()
         // Open a group.
         Group group = fid1.openGroup(GROUP1_NAME);
 
-        // Check for existence of attribute, Group::attrExists
+        // Check for existence of attribute
         attr_exists = group.attrExists(ATTR2_NAME);
         if (attr_exists == false)
             throw InvalidActionException("H5File::attrExists", "group, ATTR2_NAMEAttribute should exist but does not");
 
-        // Open attribute
-        Attribute attr = group.openAttribute(ATTR2_NAME);
-
-        // Test the existence of a name using attribute as location,
-        // Attribute::nameExists
-        bool name_exists = attr.nameExists(GROUP1_NAME);
-        if (name_exists == false)
-            throw InvalidActionException("Attribute::nameExists", "group GROUP1_NAME should exist but does not");
-
         PASSED();
     } // end try block
 
@@ -1591,12 +1632,15 @@ static void test_attr_exists()
     }
 }   // test_attr_exists()
 
-/****************************************************************
-**
-**  test_attr_dense_create(): Test phase change properties
-**      Tests "dense" attribute storage creation
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_dense_create
+ *
+ * Purpose      Test phase change properties
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 const H5std_string FILE_CRTPROPS("tattr_crt_properties.h5");
 const int NAME_BUF_SIZE = 1024;
 const unsigned MAX_COMPACT_DEF = 8;
@@ -1700,12 +1744,15 @@ static void test_attr_dense_create(FileCreatPropList& fcpl,
     }
 }   // test_attr_dense_create()
 
-/****************************************************************
-**
-**  test_attr_corder_create_basic(): Test creation order properties
-**      Tests creating an object w/attribute creation order info
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr_corder_create_basic
+ *
+ * Purpose      Test creation order properties
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 static void test_attr_corder_create_basic(FileCreatPropList& fcpl,
                                    FileAccPropList& fapl)
 {
@@ -1722,7 +1769,7 @@ static void test_attr_corder_create_basic(FileCreatPropList& fcpl,
         // Get creation order indexing on object
         unsigned crt_order_flags = 0;
         crt_order_flags = dcpl.getAttrCrtOrder();
-        verify_val(crt_order_flags, 0, "DSetCreatPropList::getAttrCrtOrder",__LINE__,__FILE__);
+        verify_val(crt_order_flags, (unsigned)0, "DSetCreatPropList::getAttrCrtOrder",__LINE__,__FILE__);
 
         // Setting invalid combination of a attribute order creation order
         // indexing on should fail
@@ -1739,7 +1786,7 @@ static void test_attr_corder_create_basic(FileCreatPropList& fcpl,
         // verify them
         dcpl.setAttrCrtOrder(H5P_CRT_ORDER_TRACKED | H5P_CRT_ORDER_INDEXED);
         crt_order_flags = dcpl.getAttrCrtOrder();
-        verify_val(crt_order_flags, (H5P_CRT_ORDER_TRACKED | H5P_CRT_ORDER_INDEXED), "DSetCreatPropList::getAttrCrtOrder",__LINE__,__FILE__);
+        verify_val(crt_order_flags, (unsigned)(H5P_CRT_ORDER_TRACKED | H5P_CRT_ORDER_INDEXED), "DSetCreatPropList::getAttrCrtOrder",__LINE__,__FILE__);
 
         // Create dataspace for dataset
         DataSpace ds_space(H5S_SCALAR);
@@ -1774,7 +1821,7 @@ static void test_attr_corder_create_basic(FileCreatPropList& fcpl,
 
         // Query the attribute creation properties
         crt_order_flags = dcpl.getAttrCrtOrder();
-        verify_val(crt_order_flags, (H5P_CRT_ORDER_TRACKED | H5P_CRT_ORDER_INDEXED), "DSetCreatPropList::getAttrCrtOrder",__LINE__,__FILE__);
+        verify_val(crt_order_flags, (unsigned)(H5P_CRT_ORDER_TRACKED | H5P_CRT_ORDER_INDEXED), "DSetCreatPropList::getAttrCrtOrder",__LINE__,__FILE__);
 
         PASSED();
     } // end try block
@@ -1785,16 +1832,19 @@ static void test_attr_corder_create_basic(FileCreatPropList& fcpl,
     }
 }   // test_attr_corder_create_basic()
 
-/****************************************************************
-**
-**  test_attr(): Main attribute testing routine.
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_attr
+ *
+ * Purpose      Main attribute testing routine
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 extern "C"
 void test_attr()
 {
     // Output message about test being performed
-    //MESSAGE("Testing Attributes\n");
     MESSAGE(5, ("Testing Attributes\n"));
 
     try
@@ -1819,7 +1869,7 @@ void test_attr()
         // use_shared test
 
         // Loop over using new group format
-        hbool_t new_format;
+        unsigned new_format;
         for (new_format = FALSE; new_format <= TRUE; new_format++)
         {
             FileAccPropList curr_fapl;
@@ -1827,13 +1877,13 @@ void test_attr()
             // Set the file access proplist for the type of format
             if (new_format)
             {
-            MESSAGE(7, ("testing with new file format\n"));
-            curr_fapl = fapl_new;
+                MESSAGE(7, ("testing with new file format\n"));
+                curr_fapl = fapl_new;
             }
             else
             {
-            MESSAGE(7, ("testing with old file format\n"));
-            curr_fapl = fapl;
+                MESSAGE(7, ("testing with old file format\n"));
+                curr_fapl = fapl;
             }
 
             test_attr_basic_write();    // Test basic H5A writing code
@@ -1873,17 +1923,14 @@ void test_attr()
         issue_fail_msg("test_attr()", __LINE__, __FILE__, E.getCDetailMsg());
     }
 }   // test_attr()
+
 
 /*-------------------------------------------------------------------------
  * Function:    cleanup_attr
  *
- * Purpose:     Cleanup temporary test files
- *
- * Return:      none
- *
- * Programmer:  Albert Cheng
- *              July 2, 1998
+ * Purpose      Cleanup temporary test files
  *
+ * Return       None
  *-------------------------------------------------------------------------
  */
 extern "C"
