@@ -20,7 +20,6 @@
         h5_fileaccess() -- in h5test.c, returns a file access template
 
  ***************************************************************************/
-
 #ifdef OLD_HEADER_FILENAME
 #include <iostream.h>
 #else
@@ -33,52 +32,52 @@ using std::endl;
 #include "H5Cpp.h"      // C++ API header file
 using namespace H5;
 
+#include "h5test.h"
 #include "h5cpputil.h"  // C++ utilility header file
 
-const hsize_t   F1_USERBLOCK_SIZE = (hsize_t)0;
-const size_t    F1_OFFSET_SIZE = sizeof(haddr_t);
-const size_t    F1_LENGTH_SIZE = sizeof(hsize_t);
-const unsigned  F1_SYM_LEAF_K  = 4;
-const unsigned  F1_SYM_INTERN_K = 16;
-const H5std_string  FILE1("tfile1.h5");
+const hsize_t F1_USERBLOCK_SIZE = (hsize_t)0;
+const size_t F1_OFFSET_SIZE = sizeof(haddr_t);
+const size_t F1_LENGTH_SIZE = sizeof(hsize_t);
+const unsigned F1_SYM_LEAF_K  = 4;
+const unsigned F1_SYM_INTERN_K = 16;
+const H5std_string    FILE1("tfile1.h5");
 
-const hsize_t   F2_USERBLOCK_SIZE = (hsize_t)512;
-const size_t    F2_OFFSET_SIZE = 8;
-const size_t    F2_LENGTH_SIZE = 8;
-const unsigned  F2_SYM_LEAF_K  = 8;
-const unsigned  F2_SYM_INTERN_K = 32;
-const unsigned  F2_ISTORE = 64;
+const hsize_t F2_USERBLOCK_SIZE = (hsize_t)512;
+const size_t F2_OFFSET_SIZE = 8;
+const size_t F2_LENGTH_SIZE = 8;
+const unsigned F2_SYM_LEAF_K  = 8;
+const unsigned F2_SYM_INTERN_K = 32;
+const unsigned F2_ISTORE = 64;
 const H5std_string    FILE2("tfile2.h5");
 
-const hsize_t   F3_USERBLOCK_SIZE = (hsize_t)0;
-const size_t    F3_OFFSET_SIZE = F2_OFFSET_SIZE;
-const size_t    F3_LENGTH_SIZE = F2_LENGTH_SIZE;
-const unsigned  F3_SYM_LEAF_K  = F2_SYM_LEAF_K;
-const unsigned  F3_SYM_INTERN_K = F2_SYM_INTERN_K;
+const hsize_t F3_USERBLOCK_SIZE = (hsize_t)0;
+const size_t F3_OFFSET_SIZE = F2_OFFSET_SIZE;
+const size_t F3_LENGTH_SIZE = F2_LENGTH_SIZE;
+const unsigned F3_SYM_LEAF_K  = F2_SYM_LEAF_K;
+const unsigned F3_SYM_INTERN_K = F2_SYM_INTERN_K;
 const H5std_string    FILE3("tfile3.h5");
 
-const int       KB =  1024;
-const H5std_string  FILE4("tfile4.h5");
+const int KB =  1024;
+const H5std_string    FILE4("tfile4.h5");
 
 
 /*-------------------------------------------------------------------------
  * Function:    test_file_create
  *
- * Purpose:     Test file and template creations
+ * Purpose      Test file and template creations
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Binh-Minh Ribler (use C version)
+ * Programmer   Binh-Minh Ribler (use C version)
  *              January, 2001
  *
  * Modifications:
- *      January, 2005: C tests' macro VERIFY casts values to 'long' for all
- *              cases.  Since there are no operator<< for 'long long'
- *              or int64 in VS C++ ostream, I casted the hsize_t values
- *              passed to verify_val to 'long' as well.  If problems
- *              arises later, this will have to be specificly handled
- *              with a special routine.
- *
+ *        January, 2005: C tests' macro VERIFY casts values to 'long' for all
+ *                     cases.  Since there are no operator<< for 'long long'
+ *                     or int64 in VS C++ ostream, I casted the hsize_t values
+ *                     passed to verify_val to 'long' as well.  If problems
+ *                     arises later, this will have to be specificly handled
+ *                     with a special routine.
  *-------------------------------------------------------------------------
  */
 static void test_file_create()
@@ -125,12 +124,12 @@ static void test_file_create()
         catch (FileIException& E) // catching creating existing file
         {} // do nothing, FAIL expected
 
-        // Test create with H5F_ACC_TRUNC. This will truncate the existing file.
+            // Test create with H5F_ACC_TRUNC. This will truncate the existing file.
         file1 = new H5File (FILE1, H5F_ACC_TRUNC);
 
         // Try to create first file again. This should fail because file1
         // is the same file and is currently open.
-        try {
+            try {
             H5File file2 (FILE1, H5F_ACC_TRUNC);   // should throw E
 
             // Should FAIL but didn't, so throw an invalid action exception
@@ -139,29 +138,29 @@ static void test_file_create()
         catch (FileIException& E) // catching truncating opened file
         {} // do nothing, FAIL expected
 
-        // Try with H5F_ACC_EXCL. This should fail too because the file already
-        // exists.
-        try {
+             // Try with H5F_ACC_EXCL. This should fail too because the file already
+             // exists.
+            try {
             H5File file3 (FILE1, H5F_ACC_EXCL);  // should throw E
 
             // Should FAIL but didn't, so throw an invalid action exception
             throw InvalidActionException("H5File constructor", "H5F_ACC_EXCL attempt on an existing file.");
-        }
+            }
         catch (FileIException& E) // catching H5F_ACC_EXCL on existing file
         {} // do nothing, FAIL expected
 
-        // Get the file-creation template
+            // Get the file-creation template
         FileCreatPropList tmpl1 = file1->getCreatePlist();
 
         hsize_t ublock = tmpl1.getUserblock();
         verify_val((long)ublock, (long)F1_USERBLOCK_SIZE, "FileCreatPropList::getUserblock", __LINE__, __FILE__);
 
-        size_t  parm1, parm2;   // file-creation parameters
+            size_t  parm1, parm2;                // file-creation parameters
         tmpl1.getSizes( parm1, parm2);
         verify_val(parm1, F1_OFFSET_SIZE, "FileCreatPropList::getSizes", __LINE__, __FILE__);
         verify_val(parm2, F1_LENGTH_SIZE, "FileCreatPropList::getSizes", __LINE__, __FILE__);
 
-        unsigned  iparm1,iparm2;    // file-creation parameters
+        unsigned  iparm1,iparm2;        // file-creation parameters
         tmpl1.getSymk( iparm1, iparm2);
         verify_val(iparm1, F1_SYM_INTERN_K, "FileCreatPropList::getSymk", __LINE__, __FILE__);
         verify_val(iparm2, F1_SYM_LEAF_K, "FileCreatPropList::getSymk", __LINE__, __FILE__);
@@ -191,19 +190,19 @@ static void test_file_create()
     FileCreatPropList* tmpl1 = NULL;
     try
     {
-        // Create a new file with a non-standard file-creation template
+            // Create a new file with a non-standard file-creation template
         tmpl1 = new FileCreatPropList;
 
-        // Set the new file-creation parameters
+            // Set the new file-creation parameters
         tmpl1->setUserblock (F2_USERBLOCK_SIZE);
         tmpl1->setSizes( F2_OFFSET_SIZE, F2_LENGTH_SIZE );
         tmpl1->setSymk( F2_SYM_INTERN_K, F2_SYM_LEAF_K );
 
-        // Try to create second file, with non-standard file-creation template
-        // params.
+             // Try to create second file, with non-standard file-creation template
+             // params.
         H5File file2( FILE2, H5F_ACC_TRUNC, *tmpl1 );
 
-        // Release file-creation template
+            // Release file-creation template
         delete tmpl1;
         tmpl1 = NULL;
 
@@ -214,12 +213,12 @@ static void test_file_create()
         hsize_t ublock = tmpl1->getUserblock();
         verify_val((long)ublock, (long)F2_USERBLOCK_SIZE, "FileCreatPropList::getUserblock", __LINE__, __FILE__);
 
-        size_t parm1, parm2;    // file-creation parameters
+        size_t parm1, parm2; // file-creation parameters
         tmpl1->getSizes( parm1, parm2);
         verify_val(parm1, F2_OFFSET_SIZE, "FileCreatPropList::getSizes", __LINE__, __FILE__);
         verify_val(parm2, F2_LENGTH_SIZE, "FileCreatPropList::getSizes", __LINE__, __FILE__);
 
-        unsigned iparm1,iparm2; // file-creation parameters
+        unsigned  iparm1,iparm2;        // file-creation parameters
         tmpl1->getSymk( iparm1, iparm2);
         verify_val(iparm1, F2_SYM_INTERN_K, "FileCreatPropList::getSymk", __LINE__, __FILE__);
         verify_val(iparm2, F2_SYM_LEAF_K, "FileCreatPropList::getSymk", __LINE__, __FILE__);
@@ -250,7 +249,7 @@ static void test_file_create()
         verify_val(parm1, F3_OFFSET_SIZE, "FileCreatPropList::getSizes", __LINE__, __FILE__);
         verify_val(parm2, F3_LENGTH_SIZE, "FileCreatPropList::getSizes", __LINE__, __FILE__);
 
-        tmpl1->getSymk( iparm1, iparm2);
+            tmpl1->getSymk( iparm1, iparm2);
         verify_val(iparm1, F3_SYM_INTERN_K, "FileCreatPropList::getSymk", __LINE__, __FILE__);
         verify_val(iparm2, F3_SYM_LEAF_K, "FileCreatPropList::getSymk", __LINE__, __FILE__);
 
@@ -271,21 +270,20 @@ static void test_file_create()
 /*-------------------------------------------------------------------------
  * Function:    test_file_open
  *
- * Purpose:     Test file accesses
+ * Purpose      Test file accesses
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Binh-Minh Ribler (use C version)
+ * Programmer   Binh-Minh Ribler (use C version)
  *              January, 2001
  *
  * Modifications:
- *      January, 2005: C tests' macro VERIFY casts values to 'long' for all
- *              cases.  Since there are no operator<< for 'long long'
- *              or int64 in VS C++ ostream, I casted the hsize_t values
- *              passed to verify_val to 'long' as well.  If problems
- *              arises later, this will have to be specificly handled
- *              with a special routine.
- *
+ *        January, 2005: C tests' macro VERIFY casts values to 'long' for all
+ *                     cases.  Since there are no operator<< for 'long long'
+ *                     or int64 in VS C++ ostream, I casted the hsize_t values
+ *                     passed to verify_val to 'long' as well.  If problems
+ *                     arises later, this will have to be specificly handled
+ *                     with a special routine.
  *-------------------------------------------------------------------------
  */
 static void test_file_open()
@@ -305,13 +303,13 @@ static void test_file_open()
         hsize_t ublock = tmpl1.getUserblock();
         verify_val((long)ublock, (long)F2_USERBLOCK_SIZE, "FileCreatPropList::getUserblock", __LINE__, __FILE__);
 
-        size_t parm1, parm2;    // file-creation parameters
-        tmpl1.getSizes(parm1, parm2);
+        size_t  parm1, parm2;  // file-creation parameters
+        tmpl1.getSizes( parm1, parm2);
         verify_val(parm1, F2_OFFSET_SIZE, "FileCreatPropList::getSizes", __LINE__, __FILE__);
         verify_val(parm2, F2_LENGTH_SIZE, "FileCreatPropList::getSizes", __LINE__, __FILE__);
 
-        unsigned iparm1,iparm2; // file-creation parameters
-        tmpl1.getSymk(iparm1, iparm2);
+        unsigned  iparm1,iparm2;       // file-creation parameters
+        tmpl1.getSymk( iparm1, iparm2);
         verify_val(iparm1, F2_SYM_INTERN_K, "FileCreatPropList::getSymk", __LINE__, __FILE__);
         verify_val(iparm2, F2_SYM_LEAF_K, "FileCreatPropList::getSymk", __LINE__, __FILE__);
 
@@ -321,12 +319,12 @@ static void test_file_open()
 
         // Try truncating the file, and it should fail because the file is
         // still opened with file2.
-        try {
+            try {
             H5File file3 (FILE2, H5F_ACC_TRUNC);  // should throw E
 
             // Should FAIL but didn't, so throw an invalid action exception
             throw InvalidActionException("H5File constructor", "Attempt truncating an opened file.");
-        }
+            }
         catch (FileIException& E) // catching H5F_ACC_TRUNC on opened file
         {} // do nothing, FAIL expected
 
@@ -354,13 +352,12 @@ static void test_file_open()
 /*-------------------------------------------------------------------------
  * Function:    test_file_size
  *
- * Purpose:     Test file size.
+ * Purpose      Test file size.
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Raymond Lu
+ * Programmer   Raymond Lu
  *              June, 2004
- *
  *-------------------------------------------------------------------------
  */
 static void test_file_size()
@@ -368,7 +365,7 @@ static void test_file_size()
     // Output message about test being performed
     SUBTEST("File Size");
 
-    hid_t fapl_id;
+    hid_t        fapl_id;
     fapl_id = h5_fileaccess(); // in h5test.c, returns a file access template
 
     try {
@@ -376,7 +373,7 @@ static void test_file_size()
         // list object to pass in H5File::H5File
         FileAccPropList fapl(fapl_id);
 
-        // Set to sec2 driver.  Do we want to test other file drivers?
+            // Set to sec2 driver.  Do we want to test other file drivers?
         // They're not tested in C++.
         // File drivers seem not implemented.
         // fapl.setSec2();
@@ -406,7 +403,7 @@ static void test_file_size()
         issue_fail_msg("test_file_size()", __LINE__, __FILE__, E.getCDetailMsg());
     }
 
-    // This fapl_id was returned from h5_fileaccess.
+    // use C test utility routine to close property list.
     herr_t ret = H5Pclose(fapl_id);
     if (ret < 0)
         issue_fail_msg("test_file_size()", __LINE__, __FILE__, "H5Pclose failed");
@@ -417,23 +414,22 @@ static void test_file_size()
 /*-------------------------------------------------------------------------
  * Function:    test_file_name
  *
- * Purpose:     Test getting file's name.
+ * Purpose      Test getting file's name.
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Binh-Minh Ribler
+ * Programmer   Binh-Minh Ribler
  *              July, 2004
- *
  *-------------------------------------------------------------------------
  */
-const int   RANK = 2;
-const int   NX = 4;
-const int   NY = 5;
-const H5std_string  GROUPNAME ("group");
-const H5std_string  DSETNAME ("dataset");
-const H5std_string  DATTRNAME ("dataset attribute");
-const H5std_string  FATTRNAME ("file attribute");
-const H5std_string  DTYPENAME ("compound");
+const int        RANK = 2;
+const int        NX = 4;
+const int        NY = 5;
+const H5std_string        GROUPNAME ("group");
+const H5std_string        DSETNAME ("dataset");
+const H5std_string        DATTRNAME ("dataset attribute");
+const H5std_string        FATTRNAME ("file attribute");
+const H5std_string        DTYPENAME ("compound");
 
 // Compound datatype
 typedef struct s1_t {
@@ -503,11 +499,20 @@ static void test_file_name()
 }   // test_file_name()
 
 
-const int   RANK1 = 1;
-const int   ATTR1_DIM1 = 3;
-const H5std_string  FILE5("tfattrs.h5");
-const H5std_string  FATTR1_NAME ("file attribute 1");
-const H5std_string  FATTR2_NAME ("file attribute 2");
+/*-------------------------------------------------------------------------
+ *
+ * Function:    test_file_attribute
+ *
+ * Purpose      Test file attributes
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
+const int        RANK1 = 1;
+const int        ATTR1_DIM1 = 3;
+const H5std_string        FILE5("tfattrs.h5");
+const H5std_string        FATTR1_NAME ("file attribute 1");
+const H5std_string        FATTR2_NAME ("file attribute 2");
 int fattr_data[ATTR1_DIM1]={512,-234,98123}; // Test data for file attribute
 int dattr_data[ATTR1_DIM1]={256,-123,1000}; // Test data for dataset attribute
 
@@ -592,18 +597,6 @@ static void test_file_attribute()
         n_attrs = dataset.getNumAttrs();
         verify_val(n_attrs, 1, "DataSet::getNumAttrs()", __LINE__, __FILE__);
 
-        // Get and verify the number of attributes at the location specified
-        // by a dataset's attribute
-        n_attrs = 0;
-        n_attrs = dattr.getNumAttrs();
-        verify_val(n_attrs, 1, "Attribute::getNumAttrs()", __LINE__, __FILE__);
-
-        // Get and verify the number of attributes at the location specified
-        // by a file's attribute
-        n_attrs = 0;
-        n_attrs = fattr1.getNumAttrs();
-        verify_val(n_attrs, 2, "Attribute::getNumAttrs()", __LINE__, __FILE__);
-
         // Read back attribute's data
         HDmemset(rdata, 0, sizeof(rdata));
         dattr.read(PredType::NATIVE_INT, rdata);
@@ -626,25 +619,24 @@ static void test_file_attribute()
 }   // test_file_attribute()
 
 
-const H5std_string  FILE6("tfile5.h5");
-const H5std_string  ROOTGROUP("/");
-const H5std_string  GROUP1("/G1");
-const H5std_string  SUBGROUP3("/G1/G3");
-
 /*-------------------------------------------------------------------------
  * Function:    test_libver_bounds_real
  *
- * Purpose:     Verify that a file created and modified with the
+ * Purpose      Verify that a file created and modified with the
  *              specified libver bounds has the specified object header
  *              versions for the right objects.
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Binh-Minh Ribler (use C version)
+ * Programmer   Binh-Minh Ribler (use C version)
  *              March, 2015
- *
  *-------------------------------------------------------------------------
  */
+const H5std_string        FILE6("tfile5.h5");
+const H5std_string        ROOTGROUP("/");
+const H5std_string        GROUP1("/G1");
+const H5std_string        SUBGROUP3("/G1/G3");
+
 static void test_libver_bounds_real(
                 H5F_libver_t libver_create, unsigned oh_vers_create,
                 H5F_libver_t libver_mod, unsigned oh_vers_mod)
@@ -715,18 +707,18 @@ static void test_libver_bounds_real(
 
 } /* end test_libver_bounds_real() */
 
+
 /*-------------------------------------------------------------------------
  *
  * Function:    test_libver_bounds
  *
- * Purpose:     Verify that a file created and modified with various
+ * Purpose      Verify that a file created and modified with various
  *              libver bounds is handled correctly.
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Binh-Minh Ribler (use C version)
+ * Programmer   Binh-Minh Ribler (use C version)
  *              March 2015
- *
  *-------------------------------------------------------------------------
  */
 static void test_libver_bounds()
@@ -736,7 +728,7 @@ static void test_libver_bounds()
 
     /* Run the tests */
     test_libver_bounds_real(H5F_LIBVER_EARLIEST, H5O_VERSION_1, H5F_LIBVER_LATEST, H5O_VERSION_2);
-    test_libver_bounds_real(H5F_LIBVER_LATEST, H5O_VERSION_2, H5F_LIBVER_EARLIEST, H5O_VERSION_1);
+    test_libver_bounds_real(H5F_LIBVER_LATEST, H5O_VERSION_2, H5F_LIBVER_EARLIEST, H5O_VERSION_2);
     PASSED();
 } /* end test_libver_bounds() */
 
@@ -744,13 +736,12 @@ static void test_libver_bounds()
 /*-------------------------------------------------------------------------
  * Function:    test_commonfg
  *
- * Purpose:     Verify that H5File works as a root group.
+ * Purpose      Verify that H5File works as a root group.
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Binh-Minh Ribler (use C version)
+ * Programmer   Binh-Minh Ribler (use C version)
  *              March, 2015
- *
  *-------------------------------------------------------------------------
  */
 static void test_commonfg()
@@ -804,24 +795,23 @@ static void test_commonfg()
 } /* end test_commonfg() */
 
 
-const H5std_string        FILE7("tfile7.h5");
-
 /*-------------------------------------------------------------------------
  * Function:    test_file_info
  *
- * Purpose:     Verify that various properties in a file creation property
+ * Purpose      Verify that various properties in a file creation property
  *              lists are stored correctly in the file and can be retrieved
  *              when the file is re-opened.
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Binh-Minh Ribler
+ * Programmer   Binh-Minh Ribler
  *              February, 2017
- *
  *-------------------------------------------------------------------------
  */
+const H5std_string FILE7("tfile7.h5");
 const hsize_t FSP_SIZE_DEF = 4096;
 const hsize_t FSP_SIZE512 = 512;
+
 static void test_file_info()
 {
     // Output message about test being performed
@@ -829,14 +819,19 @@ static void test_file_info()
 
     hsize_t out_threshold = 0;  // Free space section threshold to get
     hbool_t out_persist = FALSE;// Persist free-space read
+    // File space handling strategy
+    H5F_fspace_strategy_t out_strategy = H5F_FSPACE_STRATEGY_FSM_AGGR;
 
     try {
         // Create a file using default properties.
         H5File tempfile(FILE7, H5F_ACC_TRUNC);
 
         // Get the file's version information.
-        H5F_info_t finfo;
-        tempfile.getFileInfo(finfo); // there's no C test for H5Fget_info
+        H5F_info2_t finfo;
+        tempfile.getFileInfo(finfo);
+        verify_val(finfo.super.version, 0, "H5File::getFileInfo", __LINE__, __FILE__);
+        verify_val(finfo.free.version, 0, "H5File::getFileInfo", __LINE__, __FILE__);
+        verify_val(finfo.sohm.version, 0, "H5File::getFileInfo", __LINE__, __FILE__);
 
         // Close the file.
         tempfile.close();
@@ -844,12 +839,31 @@ static void test_file_info()
         // Create file creation property list.
         FileCreatPropList fcpl;
 
+        // Retrieve file space information.
+        fcpl.getFileSpaceStrategy(out_strategy, out_persist, out_threshold);
+
+        // Verify file space information.
+        verify_val(out_strategy, H5F_FSPACE_STRATEGY_FSM_AGGR, "H5File::getFileInfo", __LINE__, __FILE__);
+        verify_val(out_persist, FALSE, "H5File::getFileInfo", __LINE__, __FILE__);
+        verify_val(out_threshold, 1, "H5File::getFileInfo", __LINE__, __FILE__);
+
+        /* Retrieve file space page size */
+        hsize_t out_fsp_psize = fcpl.getFileSpacePagesize();
+        verify_val(out_fsp_psize, FSP_SIZE_DEF, "FileCreatPropList::getFileSpacePagesize", __LINE__, __FILE__);
+
         // Set various file information.
         fcpl.setUserblock(F2_USERBLOCK_SIZE);
         fcpl.setSizes(F2_OFFSET_SIZE, F2_LENGTH_SIZE);
         fcpl.setSymk(F2_SYM_INTERN_K, F2_SYM_LEAF_K);
         fcpl.setIstorek(F2_ISTORE);
 
+        hsize_t threshold = 5;      // Free space section threshold to set
+        hbool_t persist = TRUE;     // Persist free-space to set
+        H5F_fspace_strategy_t strategy = H5F_FSPACE_STRATEGY_PAGE;
+
+        fcpl.setFileSpaceStrategy(strategy, persist, threshold);
+        fcpl.setFileSpacePagesize(FSP_SIZE512);
+
         // Creating a file with the non-default file creation property list
         // should create a version 1 superblock
 
@@ -860,7 +874,10 @@ static void test_file_info()
         fcpl.close();
 
         // Get the file's version information.
-        file7.getFileInfo(finfo);   // there's no C test for H5Fget_info
+        file7.getFileInfo(finfo);
+        verify_val(finfo.super.version, 2, "H5File::getFileInfo", __LINE__, __FILE__);
+        verify_val(finfo.free.version, 0, "H5File::getFileInfo", __LINE__, __FILE__);
+        verify_val(finfo.sohm.version, 0, "H5File::getFileInfo", __LINE__, __FILE__);
 
         // Close the file.
         file7.close();
@@ -872,7 +889,10 @@ static void test_file_info()
         FileCreatPropList fcpl2 = file7.getCreatePlist();
 
         // Get the file's version information.
-        file7.getFileInfo(finfo);   // there's no C test for H5Fget_info
+        file7.getFileInfo(finfo);
+        verify_val(finfo.super.version, 2, "H5File::getFileInfo", __LINE__, __FILE__);
+        verify_val(finfo.free.version, 0, "H5File::getFileInfo", __LINE__, __FILE__);
+        verify_val(finfo.sohm.version, 0, "H5File::getFileInfo", __LINE__, __FILE__);
 
         // Retrieve the property values & check them.
         hsize_t userblock = fcpl2.getUserblock();
@@ -891,6 +911,20 @@ static void test_file_info()
         unsigned istore_ik = fcpl2.getIstorek();
         verify_val(istore_ik, F2_ISTORE, "FileCreatPropList::getIstorek", __LINE__, __FILE__);
 
+    /*  ret=H5Pget_shared_mesg_nindexes(fcpl2,&nindexes);
+    CHECK(ret, FAIL, "H5Pget_shared_mesg_nindexes");
+    VERIFY(nindexes, MISC11_NINDEXES, "H5Pget_shared_mesg_nindexes");
+ */ 
+
+        // Get and verify the file space info from the creation property list */
+        fcpl2.getFileSpaceStrategy(out_strategy, out_persist, out_threshold);
+        verify_val(out_strategy, strategy, "FileCreatPropList::getFileSpaceStrategy", __LINE__, __FILE__);
+        verify_val(out_persist, persist, "FileCreatPropList::getFileSpaceStrategy", __LINE__, __FILE__);
+        verify_val(out_threshold, threshold, "FileCreatPropList::getFileSpaceStrategy", __LINE__, __FILE__);
+
+        out_fsp_psize = fcpl2.getFileSpacePagesize();
+        verify_val(out_fsp_psize, FSP_SIZE512, "FileCreatPropList::getFileSpacePagesize", __LINE__, __FILE__);
+
         PASSED();
     }   // end of try block
     catch (Exception& E)
@@ -903,13 +937,12 @@ static void test_file_info()
 /*-------------------------------------------------------------------------
  * Function:    test_file
  *
- * Purpose:     Main file testing routine
+ * Purpose      Main file testing routine
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Binh-Minh Ribler (use C version)
+ * Programmer   Binh-Minh Ribler (use C version)
  *              January 2001
- *
  *-------------------------------------------------------------------------
  */
 extern "C"
@@ -932,12 +965,9 @@ void test_file()
 /*-------------------------------------------------------------------------
  * Function:    cleanup_file
  *
- * Purpose:     Cleanup temporary test files
- *
- * Return:      none
- *
- * Programmer:  (use C version)
+ * Purpose      Cleanup temporary test files
  *
+ * Return       none
  *-------------------------------------------------------------------------
  */
 #ifdef __cplusplus
