@@ -16,7 +16,6 @@
         C link interface (H5L)
 
  ***************************************************************************/
-
 #ifdef OLD_HEADER_FILENAME
 #include <iostream.h>
 #else
@@ -29,7 +28,8 @@ using std::endl;
 #include "H5Cpp.h"      // C++ API header file
 using namespace H5;
 
-#include "h5cpputil.h"  // C++ test utilility header file
+#include "h5test.h"
+#include "h5cpputil.h"  // C++ utilility header file
 
 // A lot of the definition inherited from C test links.c is left here until
 // the H5L API is implemented and tests are completed - BMR 10/19/2009
@@ -37,13 +37,13 @@ using namespace H5;
  * This file needs to access private information from the H5G package.
  * This file also needs to access the group testing code.
  */
-//#define H5G_PACKAGE
+//#define H5G_FRIEND
 //#define H5G_TESTING
 
 //#include "h5test.h"
-//#include "H5Gpkg.h"       // Groups
-//#include "H5Iprivate.h"   // IDs
-//#include "H5Lprivate.h"   // Links
+//#include "H5Gpkg.h"     /* Groups */
+//#include "H5Iprivate.h" /* IDs */
+//#include "H5Lprivate.h" /* Links */
 
 /* File for external link test.  Created with gen_udlinks.c */
 #define LINKED_FILE  "be_extlink2.h5"
@@ -114,22 +114,22 @@ const char *FILENAME[] = {
 #define MAX_NAME_LEN    ((64*1024)+1024)
 
 /* Link type IDs */
-#define UD_HARD_TYPE    201
-#define UD_CB_TYPE      H5L_TYPE_MAX
-#define UD_PLIST_TYPE   128
-#define UD_CBFAIL_TYPE  UD_PLIST_TYPE
-#define UD_ERROR_TYPE   189
-#define UD_BAD_TYPE1    H5L_TYPE_HARD
-#define UD_BAD_TYPE2    (H5L_TYPE_UD_MIN - 5)
-#define UD_BAD_VERS     (H5L_LINK_CLASS_T_VERS + 1)
+#define UD_HARD_TYPE 201
+#define UD_CB_TYPE H5L_TYPE_MAX
+#define UD_PLIST_TYPE 128
+#define UD_CBFAIL_TYPE UD_PLIST_TYPE
+#define UD_ERROR_TYPE 189
+#define UD_BAD_TYPE1 H5L_TYPE_HARD
+#define UD_BAD_TYPE2 (H5L_TYPE_UD_MIN - 5)
+#define UD_BAD_VERS (H5L_LINK_CLASS_T_VERS + 1)
 
-#define DEST_PROP_NAME      "destination_group"
-#define REREG_TARGET_NAME   "rereg_target"
+#define DEST_PROP_NAME "destination_group"
+#define REREG_TARGET_NAME "rereg_target"
 
-#define UD_CB_LINK_NAME     "ud_callback_link"
+#define UD_CB_LINK_NAME "ud_callback_link"
 #define NEW_UD_CB_LINK_NAME "ud_callback_link2"
-#define UD_CB_TARGET        "ud_target"
-#define UD_CB_TARGET_LEN    10
+#define UD_CB_TARGET "ud_target"
+#define UD_CB_TARGET_LEN 10
 
 #define LE_FILENAME "le_extlink1.h5"
 #define BE_FILENAME "be_extlink1.h5"
@@ -140,11 +140,8 @@ const char *FILENAME[] = {
 #define H5L_DIM2 100
 
 /* Creation order macros */
-#define CORDER_GROUP_NAME       "corder_group"
 #define CORDER_SOFT_GROUP_NAME  "corder_soft_group"
-#define CORDER_NLINKS           18
-#define CORDER_ITER_STOP        3
-#define CORDER_EST_ENTRY_LEN    9
+#define CORDER_NLINKS               18
 
 /* Timestamp macros */
 #define TIMESTAMP_GROUP_1       "timestamp1"
@@ -323,15 +320,12 @@ static const char *FILENAME[] = {
 /*-------------------------------------------------------------------------
  * Function:    test_basic_links
  *
- * Purpose:     Test building a file with assorted links.
- *
- * Return:      Success: 0
+ * Purpose      Test building a file with assorted links.
  *
+ * Return       Success: 0
  *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler
- *              October 16, 2009
- *
+ * October 16, 2009
  *-------------------------------------------------------------------------
  */
 static void test_basic_links(hid_t fapl_id, hbool_t new_format)
@@ -419,6 +413,11 @@ static void test_basic_links(hid_t fapl_id, hbool_t new_format)
             throw InvalidActionException("H5File::nameExists", "dset1 doesn't exist");
         if(file.nameExists("grp1/soft", LinkAccPropList::DEFAULT) != TRUE)
             throw InvalidActionException("H5File::nameExists", "grp1/soft doesn't exist");
+        // Deprecated
+        if(file.exists("dset1", LinkAccPropList::DEFAULT) != TRUE)
+            throw InvalidActionException("H5File::exists", "dset1 doesn't exist");
+        if(file.exists("grp1/soft", LinkAccPropList::DEFAULT) != TRUE)
+            throw InvalidActionException("H5File::exists", "grp1/soft doesn't exist");
 
         // Verify link values
         H5std_string softlink_val = file.getLinkval("grp1/soft");
@@ -433,20 +432,380 @@ static void test_basic_links(hid_t fapl_id, hbool_t new_format)
     {
         issue_fail_msg("test_basic_links()", __LINE__, __FILE__, E.getCDetailMsg());
     }
-}
+} // test_basic_links
+
+
+/*-------------------------------------------------------------------------
+ * Function:    test_lcpl
+ *
+ * Purpose:     Tests link creation property lists, specifically, the
+ *              character encoding property.
+ *
+ * Return:      Success:        0
+ *              Failure:        number of errors
+ * March, 2018
+ *-------------------------------------------------------------------------
+ */
+const H5std_string GROUP1NAME("First_group");
+const H5std_string GROUP2NAME("Second_group");
+static void
+test_lcpl(hid_t fapl_id, hbool_t new_format)
+{
+    H5L_info_t linfo;
+    char filename[1024];
+    hsize_t dims[2];
+
+    if(new_format)
+        SUBTEST("Link creation property lists (w/new group format)")
+    else
+        SUBTEST("Link creation property lists")
+
+    try
+    {
+        FileAccPropList fapl(fapl_id);
+
+        // Create a new file.
+        h5_fixname(FILENAME[0], fapl_id, filename, sizeof filename);
+        H5File file(filename, H5F_ACC_TRUNC, FileCreatPropList::DEFAULT, fapl);
+
+        // Create and link a group with the default LCPL.
+        Group grp_1(file.createGroup(GROUP1NAME));
+        grp_1.close();
+
+        // Check that its character encoding is the default.
+        linfo = file.getLinkInfo(GROUP1NAME);
+        if(linfo.cset != H5T_CSET_ASCII)
+            throw InvalidActionException("H5Lget_info", "Character encoding is not default");
+
+        // Create and commit a datatype with the default LCPL.
+        IntType dtype(PredType::NATIVE_INT);
+        dtype.commit(file, "/type");
+        dtype.close();
+
+        // Check that its character encoding is the default.
+        linfo = file.getLinkInfo("/type");
+        verify_val(linfo.cset, H5T_CSET_ASCII, "Character encoding is not default", __LINE__, __FILE__);
+
+        // Create a simple dataspace.
+        dims[0] = H5L_DIM1;
+        dims[1] = H5L_DIM2;
+        DataSpace dspace(2 ,dims);
+
+        // Create a dataset using the default LCPL.
+        DataSet dset(file.createDataSet("/dataset", PredType::NATIVE_INT, dspace));
+        dset.close();
+
+        // Check that its character encoding is the default.
+        linfo = file.getLinkInfo("/dataset");
+        verify_val(linfo.cset, H5T_CSET_ASCII, "Character encoding is not default", __LINE__, __FILE__);
+
+        // Create a link creation property list with the UTF-8 character encoding.
+        LinkCreatPropList lcpl;
+        lcpl.setCharEncoding(H5T_CSET_UTF8);
+
+        // Create and link a group with the new LCPL.
+        Group grp_2(file.createGroup(GROUP2NAME, lcpl));
+        grp_2.close();
+
+        // Check that its character encoding is UTF-8.
+        linfo = file.getLinkInfo(GROUP2NAME);
+        verify_val(linfo.cset, H5T_CSET_UTF8, "Character encoding is not UTF-8", __LINE__, __FILE__);
+
+        PASSED();
+    } // end of try block
+    catch (Exception& E)
+    {
+        issue_fail_msg("test_num_links()", __LINE__, __FILE__, E.getCDetailMsg());
+    }
+} // end test_lcpl()
+
+
+/*-------------------------------------------------------------------------
+ * Function:    test_move
+ *
+ * Purpose:     Tests wrappers of H5Lmove()
+ *
+ * Return:      Success: 0
+ *              Failure: number of errors
+ * March, 2018
+ *-------------------------------------------------------------------------
+ */
+static void
+test_move(hid_t fapl_id, hbool_t new_format)
+{
+    char   filename[1024];
+
+    if(new_format)
+        SUBTEST("Group::moveLink (w/new group format)")
+    else
+        SUBTEST("Group::moveLink")
+
+    try
+    {
+        FileAccPropList fapl(fapl_id);
+
+        // Create two new files
+        h5_fixname(FILENAME[0], fapl_id, filename, sizeof filename);
+        H5File file_a(filename, H5F_ACC_TRUNC, FileCreatPropList::DEFAULT, fapl);
+        h5_fixname(FILENAME[1], fapl_id, filename, sizeof filename);
+        H5File file_b(filename, H5F_ACC_TRUNC, FileCreatPropList::DEFAULT, fapl);
+
+        // Create groups in first file
+        Group grp_1(file_a.createGroup(GROUP1NAME));
+        Group grp_2(file_a.createGroup(GROUP2NAME));
+        Group grp_move(grp_1.createGroup("group_move"));
+
+        // Create hard and soft links
+        grp_1.link(H5L_TYPE_HARD, "group_move", "hard");
+        grp_2.link(H5L_TYPE_SOFT, "/First_group/group_copy", "soft");
+
+        // Move a group across files, should fail
+        try {
+            grp_1.moveLink("group_move", file_b, "group_new_name");
+
+            // Should throw an exception but didn't
+    	    H5_FAILED();
+	        cerr << "    Group group_move should not be moved across files" << endl;
+        } catch (Exception& E) {
+            // expected
+        }
+
+        // Move a soft link across files, should succeed
+        grp_2.moveLink("soft", file_b, "soft_new_name");
+        if(file_b.exists("soft_new_name") != TRUE)
+            throw InvalidActionException("H5File::exists", "grp1/soft doesn't exist");
+
+        // Move a group across groups in the same file while renaming it
+        grp_1.moveLink("group_move", grp_2, "group_new_name");
+
+        // Open the group just moved to the new location. */
+        Group moved_grp = grp_2.openGroup("group_new_name");
+        moved_grp.close();
+
+        // Verify that the group is no longer in the original location
+        try {
+            moved_grp = grp_1.openGroup("group_move");
+
+            // Should throw an exception but didn't
+    	    H5_FAILED();
+	        cerr << "    Group group_move should not be in original location" << endl;
+        } catch (Exception& E) {
+            // expected
+        }
+
+        // Use H5Lmove to rename a group without moving it
+        H5std_string new_name("group_new_name");
+        H5std_string newer_name("group_newer_name");
+        grp_2.moveLink(new_name, newer_name);
+
+        // Open the group
+        moved_grp = grp_2.openGroup("group_newer_name");
+        moved_grp.close();
+
+        // Use H5Lmove to move a group without renaming it
+        grp_2.moveLink(newer_name, grp_1, newer_name);
+
+        // Open the group
+        moved_grp = grp_1.openGroup("group_newer_name");
+        moved_grp.close();
+
+        // Move the group while giving long paths
+        file_a.moveLink("/First_group/group_newer_name", grp_2, "/Second_group/group_newest_name");
+
+        // Open the group just moved to the new location
+        moved_grp = grp_2.openGroup("group_newest_name");
+        moved_grp.close();
+
+        // Verify that the groups are not in previous locations
+        try {
+            moved_grp = grp_1.openGroup("group_newer_name");
+            moved_grp.close();
+
+    	    H5_FAILED(); // Should throw an exception but didn't
+	        cerr << "    Group group_newer_name should not be in GROUP1NAME" << endl;
+        } catch (Exception& E) {
+            // expected
+        }
+        try {
+            moved_grp = grp_2.openGroup("group_newer_name");
+            moved_grp.close();
+
+    	    H5_FAILED(); // Should throw an exception but didn't
+	        cerr << "    Group group_newer_name should not be in GROUP2NAME" << endl;
+        } catch (Exception& E) {
+            // expected
+        }
+        try {
+            moved_grp = grp_2.openGroup("group_new_name");
+            moved_grp.close();
+
+    	    H5_FAILED(); // Should throw an exception but didn't
+	        cerr << "    Group group_new_name should not be in GROUP2NAME" << endl;
+        } catch (Exception& E) {
+            // expected
+        }
+        try {
+            moved_grp = grp_1.openGroup("group_copy");
+            moved_grp.close();
+
+    	    H5_FAILED(); // Should throw an exception but didn't
+	        cerr << "    Group group_copy should not be in GROUP1NAME" << endl;
+        } catch (Exception& E) {
+            // expected
+        }
+        PASSED();
+    } // end of try block
+    catch (Exception& E)
+    {
+        issue_fail_msg("test_num_links()", __LINE__, __FILE__, E.getCDetailMsg());
+    }
+} // test_move
+
+/*-------------------------------------------------------------------------
+ * Function:    test_copy
+ *
+ * Purpose:     Tests wrappers of H5Lcopy()
+ *
+ * Return:      Success: 0
+ *              Failure: number of errors
+ * March, 2018
+ *-------------------------------------------------------------------------
+ */
+static void test_copy(hid_t fapl_id, hbool_t new_format)
+{
+    char filename[1024];
+
+    if(new_format)
+        SUBTEST("Group::copyLink (w/new group format)")
+    else
+        SUBTEST("Group::copyLink")
+
+    try
+    {
+        // Create two new files
+        h5_fixname(FILENAME[0], fapl_id, filename, sizeof filename);
+        H5File file_a(filename, H5F_ACC_TRUNC, FileCreatPropList::DEFAULT, fapl_id);
+        h5_fixname(FILENAME[1], fapl_id, filename, sizeof filename);
+        H5File file_b(filename, H5F_ACC_TRUNC, FileCreatPropList::DEFAULT, fapl_id);
+
+        // Create groups in first file
+        Group grp_1(file_a.createGroup(GROUP1NAME));
+        Group grp_2(file_a.createGroup(GROUP2NAME));
+        Group grp_move(grp_1.createGroup("group_copy"));
+
+        // Create hard and soft links
+        grp_1.link("group_copy", H5L_SAME_LOC, "hard");
+        grp_2.link("/First_group/group_copy", "soft");
+
+        // Copy a group across files, should fail
+        try {
+            grp_1.copyLink("group_copy", file_b, "group_new_name");
+        } catch (Exception& E) {
+            // expected
+        }
+
+        // Copy a soft link across files, should succeed
+        grp_2.copyLink("soft", file_b, "soft_new_name");
+        if (file_b.exists("soft_new_name") != TRUE)
+            throw InvalidActionException("H5File::exists", "soft_new_name doesn't exist");
+
+        // Move a group across groups in the same file while renaming it
+        H5std_string copy_name("group_copy");
+        H5std_string new_name("group_new_name");
+        grp_1.copyLink(copy_name, grp_2, new_name);
+
+        // Open the group just moved to the new location.
+        Group moved_grp(grp_2.openGroup("group_new_name"));
+        moved_grp.close();
+
+        // Verify that the group is also in the original location
+        moved_grp = grp_1.openGroup("group_copy");
+        moved_grp.close();
+
+        // Create a group in the same location with a different name
+        grp_2.copyLink("group_new_name", "group_newer_name");
+
+        // Open the group
+        moved_grp = grp_2.openGroup("group_newer_name");
+        moved_grp.close();
+
+        // Verify that the group is also in the original location
+        moved_grp = grp_2.openGroup("group_new_name");
+        moved_grp.close();
+
+        // Use H5Lcopy to copy to a different location with the same name
+        grp_2.copyLink("group_newer_name", grp_1, "group_newer_name");
+
+        // Open the group
+        moved_grp = grp_1.openGroup("group_newer_name");
+        moved_grp.close();
+
+        // Verify that the group is still in the previous location
+        moved_grp = grp_2.openGroup("group_new_name");
+        moved_grp.close();
+
+        // Copy the group while giving long paths
+        file_a.copyLink("/First_group/group_newer_name", grp_2, "/Second_group/group_newest_name");
+
+        // Open the newest group just moved to the new location
+        moved_grp = grp_2.openGroup("group_newest_name");
+        moved_grp.close();
+
+        // Verify that the group is still in all previous original locations
+        moved_grp = grp_1.openGroup("group_newer_name");
+        moved_grp.close();
+
+        moved_grp = grp_2.openGroup("group_newer_name");
+        moved_grp.close();
+
+        moved_grp = grp_2.openGroup("group_new_name");
+        moved_grp.close();
+
+        moved_grp = grp_1.openGroup("group_copy");
+        moved_grp.close();
+
+        // Delete "group_newer_name" from group 2, then try to open it.
+        grp_2.unlink("group_newer_name");
+        try {
+            moved_grp = grp_2.openGroup("group_newer_name");
+            moved_grp.close();
+
+    	    H5_FAILED(); // Should throw an exception but didn't
+	        cerr << "    Group group_newer_name should not be in GROUP2NAME" << endl;
+        } catch (Exception& E) {
+            // expected
+        }
+
+        // Delete "group_copy" from group 1, then try to open it.
+        grp_1.unlink("group_copy");
+        try {
+            moved_grp = grp_1.openGroup("group_copy");
+            moved_grp.close();
+
+    	    H5_FAILED(); // Should throw an exception but didn't
+	        cerr << "    Group group_copy should not be in GROUP1NAME" << endl;
+        } catch (Exception& E) {
+            // expected
+        }
+
+        PASSED();
+    } // end of try block
+    catch (Exception& E)
+    {
+        issue_fail_msg("test_num_links()", __LINE__, __FILE__, E.getCDetailMsg());
+    }
+} // test_copy
+
 
 /*-------------------------------------------------------------------------
  * Function:    test_num_links
  *
- * Purpose:     Test setting and getting limit of number of links
- *
- * Return:      Success: 0
+ * Purpose      Test setting and getting limit of number of links
  *
+ * Return       Success: 0
  *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler
- *              Mar, 2017
- *
+ * October 16, 2009
  *-------------------------------------------------------------------------
  */
 static void test_num_links(hid_t fapl_id, hbool_t new_format)
@@ -481,25 +840,24 @@ static void test_num_links(hid_t fapl_id, hbool_t new_format)
         issue_fail_msg("test_num_links()", __LINE__, __FILE__, E.getCDetailMsg());
     }
 } // test_num_links
+
 
 /*-------------------------------------------------------------------------
  * Function:    test_links
  *
- * Purpose:     Test links
+ * Purpose      Test links
  *
- * Return:      None
- *
- * Programmer:  Binh-Minh Ribler
- *              October 16, 2009
+ * Return       None
  *
+ * October 16, 2009
  *-------------------------------------------------------------------------
  */
 extern "C"
 void test_links()
 {
-    hid_t fapl_id, fapl2_id;    // File access property lists
-    hbool_t new_format;         // Whether to use the new format or not
-    const char *envval;
+    hid_t        fapl_id, fapl2_id;    /* File access property lists */
+    unsigned new_format;     /* Whether to use the new format or not */
+    const char  *envval;
 
     envval = HDgetenv("HDF5_DRIVER");
     if(envval == NULL)
@@ -515,9 +873,10 @@ void test_links()
         if((fapl2_id = H5Pcopy(fapl_id)) < 0)
             throw Exception("test_links", "H5Pcopy failed");
 
-        /* Set the "use the latest version of the format" bounds for creating objects in the file */
+        /* Set the "use the latest version of the format" bounds for creating
+           objects in the file */
         if(H5Pset_libver_bounds(fapl2_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0)
-        throw Exception("test_links", "H5Pset_libver_bounds failed");
+            throw Exception("test_links", "H5Pset_libver_bounds failed");
 
         /* Loop over using new group format */
         for(new_format = FALSE; new_format <= TRUE; new_format++)
@@ -533,125 +892,11 @@ void test_links()
             /* General tests... (on both old & new format groups */
             // FileAccPropList may be passed in instead of fapl id
             test_basic_links(my_fapl_id, new_format);
-            test_num_links(my_fapl_id, new_format);
-#if 0
-// these tests are from the C test links.c and left here for future
-// implementation of H5L API
-        nerrors += test_basic_links(fapl_id, new_format) < 0 ? 1 : 0;
-        nerrors += cklinks(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += new_links(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += ck_new_links(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += long_links(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += toomany(my_fapl, new_format) < 0 ? 1 : 0;
-
-        /* Test new H5L link creation routine */
-        nerrors += test_lcpl(my_fapl, new_format);
-        nerrors += test_move(my_fapl, new_format);
-        nerrors += test_copy(my_fapl, new_format);
-        nerrors += test_move_preserves(my_fapl, new_format);
-#ifndef H5_NO_DEPRECATED_SYMBOLS
-        nerrors += test_deprec(my_fapl, new_format);
-#endif /* H5_NO_DEPRECATED_SYMBOLS */
-        nerrors += external_link_root(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_path(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_mult(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_self(envval, my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_pingpong(envval, my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_toomany(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_dangling(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_recursive(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_query(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_unlink_compact(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_unlink_dense(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_move(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_ride(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_closing(envval, my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_endian(new_format) < 0 ? 1 : 0;
-        nerrors += external_link_strong(my_fapl, new_format) < 0 ? 1 : 0;
-
-        /* tests for external link */
-        nerrors += external_link_env(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_prefix(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_abs_mainpath(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_rel_mainpath(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_cwd(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_abstar(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_abstar_cur(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_reltar(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_chdir(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_set_elink_fapl1(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_set_elink_fapl2(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_set_elink_fapl3(new_format) < 0 ? 1 : 0;
-        nerrors += external_set_elink_acc_flags(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_set_elink_cb(my_fapl, new_format) < 0 ? 1 : 0;
-
-#ifdef H5_HAVE_WINDOW_PATH
-        nerrors += external_link_win1(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_win2(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_win3(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_win4(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_win5(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_win6(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_win7(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_win8(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += external_link_win9(my_fapl, new_format) < 0 ? 1 : 0;
-#endif
-        /* These tests assume that external links are a form of UD links,
-         * so assume that everything that passed for external links
-         * above has already been tested for UD links.
-         */
-        if(new_format == TRUE) {
-            nerrors += ud_hard_links(fapl2) < 0 ? 1 : 0;     /* requires new format groups */
-            nerrors += ud_link_reregister(fapl2) < 0 ? 1 : 0;        /* requires new format groups */
-        } /* end if */
-
-        nerrors += ud_callbacks(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += ud_link_errors(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += lapl_udata(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += lapl_nlinks(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += linkinfo(my_fapl, new_format) < 0 ? 1 : 0;
-
-        /* Misc. extra tests, useful for both new & old format files */
-        nerrors += link_visit(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += link_visit_by_name(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += obj_visit(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += obj_visit_by_name(my_fapl, new_format) < 0 ? 1 : 0;
-        nerrors += obj_visit_stop(my_fapl, new_format) < 0 ? 1 : 0;
-
-        /* Keep this test last, it's testing files that are used above */
-        /* do not do this for files used by external link tests */
-        nerrors += check_all_closed(my_fapl, new_format, EXTSTOP) < 0 ? 1 : 0;
-#endif // 0
+            test_move(my_fapl_id, new_format);
+            test_copy(my_fapl_id, new_format);
+            test_lcpl(my_fapl_id, new_format);
         } /* end for */
 
-#if 0
-    /* New group revision feature tests */
-    nerrors += corder_create_empty(fapl2) < 0 ? 1 : 0;
-/* XXX: when creation order indexing is fully working, go back and add checks
-*      to these tests to make certain that the creation order values are
-*      correct.
-*/
-    nerrors += corder_create_compact(fapl2) < 0 ? 1 : 0;
-    nerrors += corder_create_dense(fapl2) < 0 ? 1 : 0;
-    nerrors += corder_transition(fapl2) < 0 ? 1 : 0;
-    nerrors += corder_delete(fapl2) < 0 ? 1 : 0;
-    nerrors += link_info_by_idx(fapl2) < 0 ? 1 : 0;
-    nerrors += delete_by_idx(fapl2) < 0 ? 1 : 0;
-    nerrors += link_iterate(fapl2) < 0 ? 1 : 0;
-    nerrors += open_by_idx(fapl2) < 0 ? 1 : 0;
-    nerrors += object_info(fapl2) < 0 ? 1 : 0;
-    nerrors += group_info(fapl2) < 0 ? 1 : 0;
-    nerrors += timestamps(fapl2) < 0 ? 1 : 0;
-
-    /* Test new API calls on old-style groups */
-    nerrors += link_info_by_idx_old(fapl) < 0 ? 1 : 0;
-    nerrors += delete_by_idx_old(fapl) < 0 ? 1 : 0;
-    nerrors += link_iterate_old(fapl) < 0 ? 1 : 0;
-    nerrors += open_by_idx_old(fapl) < 0 ? 1 : 0;
-    nerrors += object_info_old(fapl) < 0 ? 1 : 0;
-    nerrors += group_info_old(fapl) < 0 ? 1 : 0;
-
-#endif
         /* Close 2nd FAPL */
         H5Pclose(fapl2_id);
 
@@ -670,21 +915,22 @@ void test_links()
 
 }
 
+
 /*-------------------------------------------------------------------------
  * Function:    cleanup_links
  *
- * Purpose:     Cleanup temporary test files
- *
- * Return:      none
- *
- * Programmer:  Binh-Minh Ribler
- *              October 16, 2009
+ * Purpose      Cleanup temporary test files
  *
+ * Return       none
  *-------------------------------------------------------------------------
  */
 extern "C"
 void cleanup_links()
 {
     HDremove(FILENAME[0]);
+    HDremove(FILENAME[1]);
 }
 
+
+
+
