@@ -16,48 +16,56 @@
    tobject.cpp - HDF5 C++ testing object related functionality
 
  ***************************************************************************/
-
 #ifdef OLD_HEADER_FILENAME
 #include <iostream.h>
 #else
 #include <iostream>
 #endif
-using std::cerr;
-using std::endl;
 
 #include <string>
 #include "H5Cpp.h"      // C++ API header file
 using namespace H5;
 
+#include "h5test.h"
 #include "h5cpputil.h"  // C++ utilility header file
 
-const H5std_string      FILE_OBJECTS("tobjects.h5");
-const H5std_string      FILE_OBJHDR("tobject_header.h5");
-const H5std_string      GROUP1("Top Group");
-const H5std_string      GROUP1_PATH("/Top Group");
-const H5std_string      GROUP1_1("Sub-Group 1.1");
-const H5std_string      GROUP1_1_PATH("/Top Group/Sub-Group 1.1");
-const H5std_string      GROUP1_2("Sub-Group 1.2");
-const H5std_string      GROUP1_2_PATH("/Top Group/Sub-Group 1.2");
-const H5std_string      DSET_DEFAULT_NAME("default");
-const H5std_string      DSET_IN_FILE("Dataset in File");
-const H5std_string      DSET_IN_FILE_PATH("/Dataset in File");
-const H5std_string      DSET_IN_GRP1("Dataset in Group 1");
-const H5std_string      DSET_IN_GRP1_PATH("/Top Group/Dataset in Group 1");
-const H5std_string      DSET_IN_GRP1_2("Dataset in Group 1.2");
-const H5std_string      DSET_IN_GRP1_2_PATH("/Top Group/Sub-Group 1.2/Dataset in Group 1.2");
+const H5std_string        FILE_OBJECTS("tobjects.h5");
+const H5std_string        FILE_OBJHDR("tobject_header.h5");
+const H5std_string        GROUP1("Top Group");
+const H5std_string        GROUP1_PATH("/Top Group");
+const H5std_string        GROUP1_1("Sub-Group 1.1");
+const H5std_string        GROUP1_1_PATH("/Top Group/Sub-Group 1.1");
+const H5std_string        GROUP1_2("Sub-Group 1.2");
+const H5std_string        GROUP1_2_PATH("/Top Group/Sub-Group 1.2");
+const H5std_string        DSET_DEFAULT_NAME("default");
+const H5std_string        DSET_IN_FILE("Dataset in File");
+const H5std_string        DSET_IN_FILE_PATH("/Dataset in File");
+const H5std_string        DSET_IN_GRP1("Dataset_in_Group_1");
+const H5std_string        DSET_IN_GRP1_PATH("/Top Group/Dataset_in_Group_1");
+const H5std_string        DSET_IN_GRP1_2("Dataset_in_Group_1.2");
+const H5std_string        DSET_IN_GRP1_2_PATH("/Top Group/Sub-Group 1.2/Dataset_in_Group_1.2");
 
+
 /*-------------------------------------------------------------------------
  * Function:    test_get_objname
  *
- * Purpose:     Tests getting object name of groups and datasets.
+ * Purpose      Tests getting object name of groups and datasets.
  *
- * Return:      Success:        0
- *              Failure:        -1
+ * Description:
+ *        File structure:
+ *              GROUP1
+ *                  GROUP1_1
+ *                  GROUP1_2
+ *                      DSET_IN_GRP1_2
+ *                  DSET_IN_GRP1
+ *              DSET_IN_FILE
  *
- * Programmer:  Binh-Minh Ribler
+ *
+ * Return       Success: 0
+ *              Failure: -1
+ *
+ * Programmer   Binh-Minh Ribler
  *              Friday, March 4, 2014
- *
  *-------------------------------------------------------------------------
  */
 static void test_get_objname()
@@ -96,11 +104,11 @@ static void test_get_objname()
 
         // Create a dataset in the file
         DataSet dsinfile = file.createDataSet(DSET_IN_FILE,
-                         PredType::NATIVE_DOUBLE, space);
+                           PredType::NATIVE_DOUBLE, space);
 
         // Create a dataset in the group
         DataSet dsingrp = grp1.createDataSet(DSET_IN_GRP1,
-                         PredType::NATIVE_INT, space);
+                           PredType::NATIVE_INT, space);
 
         // Get and verify the name of each dataset, using
         // H5std_string getObjName() and
@@ -134,7 +142,7 @@ static void test_get_objname()
         // Everything will be closed as they go out of scope
 
         PASSED();
-    }   // try block
+    }        // try block
 
     // catch all other exceptions
     catch (Exception& E)
@@ -143,17 +151,111 @@ static void test_get_objname()
     }
 }   // test_get_objname
 
+
+/*-------------------------------------------------------------------------
+ * Function:    test_existance
+ *
+ * Purpose      Tests getting object name of groups and datasets.
+ *
+ * Description:
+ *        File structure:
+ *              GROUP1
+ *                  GROUP1_1
+ *                  GROUP1_2
+ *                      DSET_IN_GRP1_2
+ *                  DSET_IN_GRP1
+ *              DSET_IN_FILE
+ *
+ *
+ * Return       Success: 0
+ *              Failure: -1
+ *
+ * Programmer   Binh-Minh Ribler
+ *              Friday, March 4, 2014
+ *-------------------------------------------------------------------------
+ */
+static void test_existance()
+{
+    SUBTEST("H5File::exists and Group::exists");
+
+    try {
+        // Open file
+        H5File file(FILE_OBJECTS, H5F_ACC_RDONLY);
+
+        // Check if GROUP1 exists in the file
+        bool exists = file.nameExists(GROUP1);
+        verify_val(exists, TRUE, "Group::nameExists GROUP1_1", __LINE__, __FILE__);
+        // Deprecated
+        exists = file.exists(GROUP1);
+        verify_val(exists, TRUE, "Group::exists GROUP1_1", __LINE__, __FILE__);
+
+        // Open GROUP1
+        Group grp1 = file.openGroup(GROUP1);
+
+        // Check if GROUP1_1 and GROUP1_2 exist in GROUP1
+        exists = grp1.nameExists(GROUP1_1);
+        verify_val(exists, TRUE, "Group::nameExists GROUP1_1", __LINE__, __FILE__);
+        exists = grp1.nameExists(GROUP1_2);
+        verify_val(exists, TRUE, "Group::nameExists GROUP1_2", __LINE__, __FILE__);
+        // Deprecated
+        exists = grp1.exists(GROUP1_1);
+        verify_val(exists, TRUE, "Group::exists GROUP1_1", __LINE__, __FILE__);
+        exists = grp1.exists(GROUP1_2);
+        verify_val(exists, TRUE, "Group::exists GROUP1_2", __LINE__, __FILE__);
+
+        // Check if DSET_IN_GRP1 exists in GROUP1
+        exists = grp1.nameExists(DSET_IN_GRP1);
+        verify_val(exists, TRUE, "Group::nameExists DSET_IN_GRP1", __LINE__, __FILE__);
+        // Deprecated
+        exists = grp1.exists(DSET_IN_GRP1);
+        verify_val(exists, TRUE, "Group::exists DSET_IN_GRP1", __LINE__, __FILE__);
+
+        // Open GROUP1_2
+        Group grp1_2 = grp1.openGroup(GROUP1_2);
+
+        // Check if DSET_IN_GRP1_2 exists in GROUP1_2
+        exists = grp1_2.nameExists(DSET_IN_GRP1_2);
+        verify_val(exists, TRUE, "Group::nameExists DSET_IN_GRP1_2", __LINE__, __FILE__);
+        // Deprecated
+        exists = grp1_2.exists(DSET_IN_GRP1_2);
+        verify_val(exists, TRUE, "Group::exists DSET_IN_GRP1_2", __LINE__, __FILE__);
+
+        // Check if a dataset exists given dataset as location with full path name
+        DataSet dset1 = file.openDataSet(DSET_IN_FILE);
+        exists = dset1.nameExists("/Top Group/Dataset_in_Group_1");
+        verify_val(exists, TRUE, "Group::nameExists given dataset with full path name", __LINE__, __FILE__);
+
+        exists = grp1_2.nameExists(DSET_IN_GRP1);
+        verify_val(exists, FALSE, "Group::nameExists DSET_IN_GRP1", __LINE__, __FILE__);
+        // Deprecated
+        exists = dset1.exists("/Top Group/Dataset_in_Group_1");
+        verify_val(exists, TRUE, "Group::exists given dataset with full path name", __LINE__, __FILE__);
+        exists = grp1_2.exists(DSET_IN_GRP1);
+        verify_val(exists, FALSE, "Group::exists DSET_IN_GRP1", __LINE__, __FILE__);
+
+        // Everything will be closed as they go out of scope
+
+        PASSED();
+    }        // try block
+
+    // catch all other exceptions
+    catch (Exception& E)
+    {
+        issue_fail_msg("test_existance", __LINE__, __FILE__);
+    }
+}   // test_existance
+
+
 /*-------------------------------------------------------------------------
  * Function:    test_get_objname_ontypes
  *
- * Purpose:     Test getting object name from various committed types.
+ * Purpose      Test getting object name from various committed types.
  *
- * Return:      Success:        0
- *              Failure:        -1
+ * Return       Success: 0
+ *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler
+ * Programmer   Binh-Minh Ribler
  *              March 4, 2014
- *
  *-------------------------------------------------------------------------
  */
 static void test_get_objname_ontypes()
@@ -173,12 +275,21 @@ static void test_get_objname_ontypes()
 
         // Close the type then open it again to test getting its name
         inttype.close();
-        inttype = file.openIntType("INT type of STD_B8LE");
+        inttype = file.openIntType("INT type of STD_B8LE"); // deprecated
 
         // Get and verify its name
         H5std_string inttype_name = inttype.getObjName();
         verify_val(inttype_name, "/INT type of STD_B8LE", "DataType::getObjName", __LINE__, __FILE__);
 
+        // Close the type then open it again to test getting its name, but
+        // with the constructor this time
+        inttype.close();
+        IntType std_b8le(file, "INT type of STD_B8LE");
+
+        // Get and verify its name
+        H5std_string std_b8le_name = std_b8le.getObjName();
+        verify_val(std_b8le_name, "/INT type of STD_B8LE", "DataType::getObjName", __LINE__, __FILE__);
+
         // Make copy of a predefined type and save it
         DataType dtype(PredType::STD_B8LE);
         dtype.commit(file, "STD_B8LE");
@@ -189,15 +300,22 @@ static void test_get_objname_ontypes()
 
         // Re-open the file and the data type to test getting its name
         file.openFile(FILE_OBJECTS, H5F_ACC_RDWR);
-        dtype = file.openDataType("STD_B8LE");
+        dtype = file.openDataType("STD_B8LE"); // deprecated
 
         // Get and verify its name
         H5std_string type_name = dtype.getObjName();
         verify_val(type_name, "/STD_B8LE", "DataType::getObjName", __LINE__, __FILE__);
 
+        // Close the type and open it again with the constructor then test
+        // getting its name
+        dtype.close();
+        DataType dtype2(file, "STD_B8LE");
+        type_name = dtype2.getObjName();
+        verify_val(type_name, "/STD_B8LE", "DataType::getObjName", __LINE__, __FILE__);
+
         // Test getting type's name from copied type
         DataType copied_type;
-        copied_type.copy(dtype);
+        copied_type.copy(dtype2);
         copied_type.commit(file, "copy of STD_B8LE");
         type_name = copied_type.getObjName();
         verify_val(type_name, "/copy of STD_B8LE", "DataType::getObjName", __LINE__, __FILE__);
@@ -212,7 +330,7 @@ static void test_get_objname_ontypes()
         verify_val(type_name, "/typetests/IntType NATIVE_INT", "DataType::getObjName", __LINE__, __FILE__);
 
         // Close everything or they can be closed when objects go out of scope
-        dtype.close();
+        dtype2.close();
         copied_type.close();
         new_int_type.close();
         grp.close();
@@ -226,17 +344,17 @@ static void test_get_objname_ontypes()
     }
 }   // test_get_objname_ontypes
 
+
 /*-------------------------------------------------------------------------
  * Function:    test_get_objtype
  *
- * Purpose:     Tests getting object type
+ * Purpose      Tests getting object type
  *
- * Return:      Success:        0
- *              Failure:        -1
+ * Return       Success: 0
+ *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler
+ * Programmer   Binh-Minh Ribler
  *              Friday, March 4, 2014
- *
  *-------------------------------------------------------------------------
  */
 static void test_get_objtype()
@@ -257,30 +375,30 @@ static void test_get_objtype()
         // Get and verify object type with
         // H5O_type_t childObjType(const H5std_string& objname)
         H5O_type_t objtype = file.childObjType(DSET_IN_FILE);
-        verify_val(objtype, H5O_TYPE_DATASET, "H5File::childObjType", __LINE__, __FILE__);
+        verify_val(objtype, H5O_TYPE_DATASET, "DataSet::childObjType", __LINE__, __FILE__);
 
         // Get and verify object type with
         // H5O_type_t childObjType(const char* objname)
         objtype = grp1.childObjType(GROUP1_1.c_str());
-        verify_val(objtype, H5O_TYPE_GROUP, "Group::childObjType", __LINE__, __FILE__);
+        verify_val(objtype, H5O_TYPE_GROUP, "DataSet::childObjType", __LINE__, __FILE__);
 
         // Get and verify object type with
         // H5O_type_t childObjType(hsize_t index, H5_index_t index_type,
         // H5_iter_order_t order, const char* objname=".")
         objtype = grp1.childObjType((hsize_t)1, H5_INDEX_NAME, H5_ITER_INC);
-        verify_val(objtype, H5O_TYPE_NAMED_DATATYPE, "Group::childObjType", __LINE__, __FILE__);
+        verify_val(objtype, H5O_TYPE_NAMED_DATATYPE, "DataSet::childObjType", __LINE__, __FILE__);
 
         // Get and verify object type with
         // H5O_type_t childObjType(hsize_t index,
         // H5_index_t index_type=H5_INDEX_NAME,
         // H5_iter_order_t order=H5_ITER_INC, const char* objname=".")
         objtype = grp1.childObjType((hsize_t)2);
-        verify_val(objtype, H5O_TYPE_GROUP, "Group::childObjType", __LINE__, __FILE__);
+        verify_val(objtype, H5O_TYPE_GROUP, "DataSet::childObjType", __LINE__, __FILE__);
 
         // Everything will be closed as they go out of scope
 
         PASSED();
-    }   // try block
+    }        // try block
 
     // catch all other exceptions
     catch (Exception& E)
@@ -288,17 +406,17 @@ static void test_get_objtype()
         issue_fail_msg("test_get_objtype", __LINE__, __FILE__);
     }
 }   // test_get_objtype
+
 
 /*-------------------------------------------------------------------------
  * Function:    test_open_object_header
  *
- * Purpose:     Test H5Location::openObjId function.
+ * Purpose      Test Group::getObjId function.
  *
- * Return:      None
- *
- * Programmer:  Binh-Minh Ribler (use C version)
- *              May 15, 2017
+ * Return       None
  *
+ * Programmer   Binh-Minh Ribler (use C version)
+ *              March, 2017
  *-------------------------------------------------------------------------
  */
 const H5std_string GROUPNAME("group");
@@ -308,13 +426,14 @@ const H5std_string DSETNAME("dataset");
 #define RANK 2
 #define DIM0 5
 #define DIM1 10
+
 static void test_open_object_header()
 {
     hsize_t     dims[2];
     H5G_info_t  ginfo;                      /* Group info struct */
 
     // Output message about test being performed
-    SUBTEST("H5Location::openObjId and H5Location::closeObjId");
+    SUBTEST("Group::getObjId");
 
     try {
         // Create file FILE1
@@ -340,10 +459,10 @@ static void test_open_object_header()
         dset.close();
         dspace.close();
 
-        // Now make sure that openObjId can open all three types of objects
-        hid_t obj_grp = file1.openObjId(GROUPNAME);
-        hid_t obj_dtype = file1.openObjId(DTYPENAME);
-        hid_t obj_dset = file1.openObjId(DSETNAME);
+        // Now make sure that getObjId can open all three types of objects
+        hid_t obj_grp = file1.getObjId(GROUPNAME);
+        hid_t obj_dtype = file1.getObjId(DTYPENAME);
+        hid_t obj_dset = file1.getObjId(DSETNAME);
 
         // Make sure that each is the right kind of ID
         H5I_type_t id_type = IdComponent::getHDFObjType(obj_grp);
@@ -369,7 +488,7 @@ static void test_open_object_header()
         dspace.close();
 
         // Open datatype object from the group
-        obj_dtype = grp2.openObjId(DTYPENAME_INGRP);
+        obj_dtype = grp2.getObjId(DTYPENAME_INGRP);
 
         dtype.setId(obj_dtype);
         H5T_class_t type_class = dtype.getClass();
@@ -405,71 +524,18 @@ static void test_open_object_header()
         issue_fail_msg("test_file_name()", __LINE__, __FILE__, E.getCDetailMsg());
     }
 } /* test_open_object_header() */
+
 
-/*-------------------------------------------------------------------------
- * Function:    test_is_valid
- *
- * Purpose:     Tests validating IDs.
- *
- * Return:      Success:        0
- *              Failure:        -1
- *
- * Programmer:  Binh-Minh Ribler
- *              May 15, 2017
- *
- *-------------------------------------------------------------------------
- */
-static void test_is_valid()
-{
-    SUBTEST("IdComponent::isValid");
-
-    try {
-        // Create a datatype
-        IntType int1(PredType::NATIVE_INT);
-
-        // Check that the ID is valid
-        hid_t int1_id = int1.getId();
-        bool is_valid = IdComponent::isValid(int1_id);
-        verify_val(is_valid, true, "IdComponent::isValid", __LINE__, __FILE__);
-
-        // Create another datatype
-        FloatType float1(PredType::NATIVE_FLOAT);
-
-        // Check that the ID is valid
-        is_valid = IdComponent::isValid(float1.getId());
-        verify_val(is_valid, true, "IdComponent::isValid", __LINE__, __FILE__);
-
-        // Close the integer type, then check the id, it should no longer be valid
-        int1.close();
-        is_valid = IdComponent::isValid(int1_id);
-        verify_val(is_valid, false, "IdComponent::isValid", __LINE__, __FILE__);
-
-        // Check that an id of -1 is invalid
-        is_valid = IdComponent::isValid((hid_t)-1);
-        verify_val(is_valid, false, "IdComponent::isValid", __LINE__, __FILE__);
-
-        PASSED();
-    }   // try block
-
-    // catch all other exceptions
-    catch (Exception& E)
-    {
-  cerr << " in catch " << endl;
-        issue_fail_msg("test_get_objtype", __LINE__, __FILE__, E.getCDetailMsg());
-    }
-}   // test_is_valid
-
 /*-------------------------------------------------------------------------
  * Function:    test_objects
  *
- * Purpose:     Tests HDF5 object related functionality
+ * Purpose      Tests HDF5 object related functionality
  *
- * Return:      Success: 0
+ * Return       Success: 0
  *              Failure: -1
  *
- * Programmer:  Binh-Minh Ribler
+ * Programmer   Binh-Minh Ribler
  *              Friday, Mar 4, 2014
- *
  *-------------------------------------------------------------------------
  */
 extern "C"
@@ -479,22 +545,20 @@ void test_object()
     MESSAGE(5, ("Testing Object Functions\n"));
 
     test_get_objname();    // Test get object name from groups/datasets
-    test_get_objname_ontypes(); // Test get object name from types
+    test_existance();      // Test check for object existance
+    test_get_objname_ontypes();        // Test get object name from types
     test_get_objtype();    // Test get object type
-    test_is_valid();       // Test validating IDs
     test_open_object_header();    // Test object header functions (H5O)
 
 }   // test_objects
 
+
 /*-------------------------------------------------------------------------
  * Function:    cleanup_objects
  *
- * Purpose:     Cleanup temporary test files
- *
- * Return:      none
- *
- * Programmer:  (use C version)
+ * Purpose      Cleanup temporary test files
  *
+ * Return       None
  *-------------------------------------------------------------------------
  */
 extern "C"
