@@ -17,30 +17,28 @@
                 Reference interface (H5R)
 
  ***************************************************************************/
-
 #ifdef OLD_HEADER_FILENAME
 #include <iostream.h>
 #else
 #include <iostream>
 #endif
+using std::cerr;
+using std::endl;
 
 #include <string>
 #include "H5Cpp.h"      // C++ API header file
 using namespace H5;
 
+#include "h5test.h"
 #include "h5cpputil.h"  // C++ utilility header file
 
-// File names
 const H5std_string      FILE1("trefer1.h5");
 const H5std_string      FILE2("trefer2.h5");
 
-// Dataset and datatype names
+// Dataset 1
 const H5std_string      DSET1_NAME("Dataset1");
 const H5std_string      DSET2_NAME("Dataset2");
-const H5std_string      DSET3_NAME("Dataset3");
-const H5std_string      DTYPE_NAME("Datatype1");
 
-// Compound type member names
 const H5std_string MEMBER1( "a_name" );
 const H5std_string MEMBER2( "b_name" );
 const H5std_string MEMBER3( "c_name" );
@@ -63,14 +61,16 @@ typedef struct s1_t {
     float c;
 } s1_t;
 
-/****************************************************************
-**
-**  test_reference_params(): Test basic H5R (reference) parameters
-**                           for correct processing
-**
-****************************************************************/
-static void
-test_reference_params(void)
+
+/*-------------------------------------------------------------------------
+ * Function:    test_reference_params
+ *
+ * Purpose      Test basic H5R (reference) parameters for correct processing
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
+static void test_reference_params()
 {
     const char *write_comment = "Foo!"; /* Comments for group */
 
@@ -117,7 +117,7 @@ test_reference_params(void)
         dataset.close();
 
         // Create another dataset (inside /Group1)
-        dataset = group.createDataSet(DSET2_NAME, PredType::NATIVE_UCHAR, sid1);
+        dataset = group.createDataSet("Dataset2", PredType::NATIVE_UCHAR, sid1);
 
         // Close Dataset
         dataset.close();
@@ -131,14 +131,14 @@ test_reference_params(void)
         dtype1.insertMember(MEMBER3, HOFFSET(s1_t, c), PredType::NATIVE_FLOAT);
 
         // Save datatype for later
-        dtype1.commit(group, DTYPE_NAME);
+        dtype1.commit(group, "Datatype1");
 
         // Close datatype and group
         dtype1.close();
         group.close();
 
         // Create a dataset
-        dataset = file1->createDataSet(DSET3_NAME, PredType::STD_REF_OBJ, sid1);
+        dataset = file1->createDataSet("Dataset3", PredType::STD_REF_OBJ, sid1);
 
         /* Test parameters to H5Location::reference */
         try {
@@ -179,13 +179,17 @@ test_reference_params(void)
         delete file1;
 }   /* test_reference_param() */
 
-/****************************************************************
-**
-**  test_reference_obj(): Test basic object reference functions
-**                        to various kinds of objects
-**
-****************************************************************/
-static void test_reference_obj(void)
+
+/*-------------------------------------------------------------------------
+ * Function:    test_reference_obj
+ *
+ * Purpose      Test basic object reference functions to various kinds
+ *              of objects
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
+static void test_reference_obj()
 {
     int    i;          // counting variables
     const  H5std_string write_comment="Foo!"; // Comments for group
@@ -212,6 +216,9 @@ static void test_reference_obj(void)
         hsize_t dims1[] = {SPACE1_DIM1};
         DataSpace sid1(SPACE1_RANK, dims1);
 
+        // Create dataset access property list
+        PropList dapl(H5P_DATASET_ACCESS);
+
         // Create a group
         Group group = file1->createGroup("Group1");
 
@@ -232,7 +239,7 @@ static void test_reference_obj(void)
         dataset.close();
 
         // Create another dataset (inside /Group1)
-        dataset = group.createDataSet(DSET2_NAME, PredType::NATIVE_UCHAR, sid1);
+        dataset = group.createDataSet("Dataset2", PredType::NATIVE_UCHAR, sid1);
 
         // Close Dataset
         dataset.close();
@@ -246,14 +253,14 @@ static void test_reference_obj(void)
         dtype1.insertMember(MEMBER3, HOFFSET(s1_t, c), PredType::NATIVE_FLOAT);
 
         // Save datatype for later
-        dtype1.commit(group, DTYPE_NAME);
+        dtype1.commit(group, "Datatype1");
 
         // Close datatype and group
         dtype1.close();
         group.close();
 
         // Create a dataset
-        dataset = file1->createDataSet(DSET3_NAME, PredType::STD_REF_OBJ, sid1);
+        dataset = file1->createDataSet("Dataset3", PredType::STD_REF_OBJ, sid1);
 
         // Create reference to dataset and test getRefObjType
         file1->reference(&wbuf[0], "/Group1/Dataset1");
@@ -287,14 +294,14 @@ static void test_reference_obj(void)
         file1 = new H5File(FILE1, H5F_ACC_RDWR);
 
         // Open the dataset
-        dataset = file1->openDataSet(DSET3_NAME);
+        dataset = file1->openDataSet("/Dataset3");
 
         // Read selection from disk
         dataset.read(rbuf, PredType::STD_REF_OBJ);
 
         // Dereference dataset object by ctor, from the location where
         // 'dataset' is located
-        DataSet dset2(dataset, &rbuf[0], H5R_OBJECT);
+        DataSet dset2(dataset, &rbuf[0], H5R_OBJECT, dapl);
 
         // Check information in the referenced dataset
         sid1 = dset2.getSpace();
@@ -305,7 +312,7 @@ static void test_reference_obj(void)
         dset2.read(tbuf, PredType::NATIVE_UINT);
 
         for(tu32 = (unsigned *)tbuf, i = 0; i < SPACE1_DIM1; i++, tu32++)
-           verify_val(*tu32, (uint32_t)(i*3), "DataSpace::getSimpleExtentNpoints",__LINE__,__FILE__);
+            verify_val(*tu32, (uint32_t)(i*3), "DataSpace::getSimpleExtentNpoints",__LINE__,__FILE__);
 
         // Close dereferenced dataset
         dset2.close();
@@ -368,27 +375,26 @@ static void test_reference_obj(void)
         delete file1;
 }   // test_reference_obj()
 
-
-/****************************************************************
-**
-**  test_reference_group(): Test object reference functionality
-**      Tests for correct behavior of various routines on
-**      dereferenced group
-**
-****************************************************************/
+
+/*-------------------------------------------------------------------------
+ * Function:    test_reference_group
+ *
+ * Purpose      Test object reference functionality on group.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 #define GROUPNAME       "/group"
 #define GROUPNAME2      "group2"
 #define GROUPNAME3      "group3"
 #define DSETNAME        "/dset"
 #define DSETNAME2       "dset2"
-#define ATTRNAME        "some attribute"
 #define NAME_SIZE       16
 
-static void
-test_reference_group(void)
+static void test_reference_group()
 {
-    hobj_ref_t wref;    /* Reference to write */
-    hobj_ref_t rref;    /* Reference to read */
+    hobj_ref_t wref;        /* Reference to write */
+    hobj_ref_t rref;        /* Reference to read */
     const  H5std_string write_comment="Foo!"; // Comments for group
 
     // Output message about test being performed
@@ -433,6 +439,7 @@ test_reference_group(void)
 
         // Close resources
         dset2.close();
+        sid1.close();
         file1->close();
 
         /*
@@ -448,15 +455,9 @@ test_reference_group(void)
         // Read in the reference
         dset1.read(&rref, PredType::STD_REF_OBJ);
 
-        // Create an attribute for the dataset
-        Attribute ds_attr1 = dset1.createAttribute(ATTRNAME, PredType::NATIVE_INT, sid1);
-
-        // Dereference to get the group using constructor, with dataset as loc
+        // Dereference to get the group
         Group refgroup(dset1, &rref);
 
-        // Dereference to get the group using constructor, with attribute as loc
-        Group attrrefgroup(ds_attr1, &rref);
-
         // Dereference group object the other way
         group.dereference(dset1, &rref);
 
@@ -468,22 +469,14 @@ test_reference_group(void)
         hsize_t nobjs = refgroup.getNumObjs();
         verify_val(nobjs, (hsize_t)3, "H5Group::getNumObjs",__LINE__,__FILE__);
 
-        // Check getting file name given the group dereferenced via constructor
-        H5std_string fname = refgroup.getFileName();
-        verify_val(fname, FILE1, "H5Group::getFileName",__LINE__,__FILE__);
-    
-        // Check number of objects in the group dereferenced by constructor
-        nobjs = attrrefgroup.getNumObjs();
-        verify_val(nobjs, (hsize_t)3, "H5Group::getNumObjs",__LINE__,__FILE__);
-
-        // Check getting file name given the group dereferenced via constructor
-        fname = attrrefgroup.getFileName();
-        verify_val(fname, FILE1, "H5Group::getFileName",__LINE__,__FILE__);
-    
         // Check number of objects in the group dereferenced by ::reference
         nobjs = group.getNumObjs();
         verify_val(nobjs, (hsize_t)3, "H5Group::getNumObjs",__LINE__,__FILE__);
 
+        // Check getting file name given the group dereferenced via constructor
+        H5std_string fname = refgroup.getFileName();
+        verify_val(fname, FILE1, "H5Group::getFileName",__LINE__,__FILE__);
+    
         // Check getting file name given the group dereferenced by ::reference
         fname = group.getFileName();
         verify_val(fname, FILE1, "H5Group::getFileName",__LINE__,__FILE__);
@@ -496,9 +489,7 @@ test_reference_group(void)
         // Close resources
         group.close();
         refgroup.close();
-        attrrefgroup.close();
         dset1.close();
-        sid1.close();
         file1->close();
 
         PASSED();
@@ -513,33 +504,35 @@ test_reference_group(void)
         delete file1;
 }   /* test_reference_group() */
 
-/****************************************************************
-**
-**  test_reference_region_1D(): Test 1-D reference functionality
-**      Tests 1-D references to various kinds of objects
-**
-****************************************************************/
-static void
-test_reference_region_1D(void)
+
+/*-------------------------------------------------------------------------
+ * Function:    test_reference_region_1D
+ *
+ * Purpose      Test 1-D reference functionality on various kinds of objects.
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
+static void test_reference_region_1D()
 {
-    hsize_t     start[SPACE3_RANK];     /* Starting location of hyperslab */
-    hsize_t     stride[SPACE3_RANK];    /* Stride of hyperslab */
-    hsize_t     count[SPACE3_RANK];     /* Element count of hyperslab */
-    hsize_t     block[SPACE3_RANK];     /* Block size of hyperslab */
-    hsize_t     coord1[POINT1_NPOINTS][SPACE3_RANK]; /* Coordinates for point selection */
+    hsize_t        start[SPACE3_RANK];     /* Starting location of hyperslab */
+    hsize_t        stride[SPACE3_RANK];    /* Stride of hyperslab */
+    hsize_t        count[SPACE3_RANK];     /* Element count of hyperslab */
+    hsize_t        block[SPACE3_RANK];     /* Block size of hyperslab */
+    hsize_t        coord1[POINT1_NPOINTS][SPACE3_RANK]; /* Coordinates for point selection */
     hsize_t *   coords;                 /* Coordinate buffer */
-    hsize_t     low[SPACE3_RANK];       /* Selection bounds */
-    hsize_t     high[SPACE3_RANK];      /* Selection bounds */
+    hsize_t        low[SPACE3_RANK];       /* Selection bounds */
+    hsize_t        high[SPACE3_RANK];      /* Selection bounds */
     int         i;      /* counting variables */
 
     // Output message about test being performed
     SUBTEST("1-D Dataset Region Reference Functions");
 
     try {
-        hdset_reg_ref_t *wbuf,  // buffer to write to disk
-                   *rbuf;       // buffer read from disk
-        uint8_t    *dwbuf,      // Buffer for writing numeric data to disk
-                   *drbuf;      // Buffer for reading numeric data from disk
+        hdset_reg_ref_t *wbuf,   // buffer to write to disk
+                   *rbuf;        // buffer read from disk
+        uint8_t    *dwbuf,       // Buffer for writing numeric data to disk
+                   *drbuf;       // Buffer for reading numeric data from disk
 
         // Allocate write & read buffers
         wbuf = (hdset_reg_ref_t *)HDcalloc(sizeof(hdset_reg_ref_t), (size_t)SPACE1_DIM1);
@@ -554,6 +547,9 @@ test_reference_region_1D(void)
         hsize_t dims3[] = {SPACE3_DIM1};
         DataSpace sid3(SPACE3_RANK, dims3);
 
+        // Create dataset access property list
+        PropList dapl(H5P_DATASET_ACCESS);
+
         // Create a dataset
         DataSet dset3 = file1.createDataSet(DSET2_NAME, PredType::STD_U8LE, sid3);
 
@@ -643,7 +639,7 @@ test_reference_region_1D(void)
         dset1.read(rbuf, PredType::STD_REF_DSETREG);
 
         { // Test DataSet::dereference
-            dset3.dereference(dset1, &rbuf[0], H5R_DATASET_REGION);
+            dset3.dereference(dset1, &rbuf[0], H5R_DATASET_REGION, dapl);
 
             // Get and verify object type
             obj_type = dset1.getRefObjType(&rbuf[0], H5R_DATASET_REGION);
@@ -658,7 +654,7 @@ test_reference_region_1D(void)
         { // Test DataSet constructor -by dereference
             // Dereference dataset object by ctor, from the location where
             // 'dset1' is located
-            DataSet newds(dset1, &rbuf[0], H5R_DATASET_REGION);
+            DataSet newds(dset1, &rbuf[0], H5R_DATASET_REGION, dapl);
 
             // Get dataspace of newds then verify number of elements
             sid1 = newds.getSpace();
@@ -674,7 +670,7 @@ test_reference_region_1D(void)
         dset3.read(drbuf, PredType::STD_U8LE);
 
         for(tu8 = (uint8_t *)drbuf, i = 0; i < SPACE3_DIM1; i++, tu8++)
-           verify_val(*tu8, (uint8_t)(i * 3), "DataSpace::getSimpleExtentNpoints",__LINE__,__FILE__);
+            verify_val(*tu8, (uint8_t)(i * 3), "DataSpace::getSimpleExtentNpoints",__LINE__,__FILE__);
 
         /*
          * Test getting the referenced region
@@ -799,47 +795,36 @@ test_reference_region_1D(void)
     }
 }   /* test_reference_region_1D() */
 
-
-/****************************************************************
-**
-**  test_reference_compat(): Test basic object reference functionality.
-**      Tests references to various kinds of objects using deprecated API.
-**
-****************************************************************/
-static void test_reference_compat(void)
-{
-   // Not yet
-}   // test_reference_compat()
-
-
-/****************************************************************
-**
-**  test_reference(): Main reference testing routine.
-**
-****************************************************************/
+/*-------------------------------------------------------------------------
+ *
+ *  test_reference(): Main reference testing routine.
+ *
+ *-------------------------------------------------------------------------
+ */
 extern "C"
-void test_reference(void)
+void test_reference()
 {
     // Output message about test being performed
-    //MESSAGE("Testing References\n");
     MESSAGE(5, ("Testing References\n"));
 
     test_reference_params();    // Test basic parameters of reference functionality
     test_reference_obj();       // Test basic object reference functionality
     test_reference_group();     // Test group reference functionality
     test_reference_region_1D(); // Test 1-D reference functionality
-    test_reference_compat();    // Tests deprecated reference routines (not yet)
 
 }   // test_reference()
 
 
-/****************************************************************
-** Function:    cleanup_reference
-** Purpose:     Cleanup temporary test files
-** Return:      none
-****************************************************************/
+/*-------------------------------------------------------------------------
+ * Function:    cleanup_reference
+ *
+ * Purpose      Cleanup temporary test files
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 extern "C"
-void cleanup_reference(void)
+void cleanup_reference()
 {
     HDremove(FILE1.c_str());
     HDremove(FILE2.c_str());
