@@ -16,7 +16,6 @@
    ttypes.cpp - HDF5 C++ testing the general datatype functionality
 
  ***************************************************************************/
-
 #ifdef OLD_HEADER_FILENAME
 #include <iostream.h>
 #else
@@ -29,13 +28,14 @@ using std::endl;
 #include "H5Cpp.h"      // C++ API header file
 using namespace H5;
 
+#include "h5test.h"
 #include "h5cpputil.h"  // C++ utilility header file
 
 /*
  * Offset from alinged memory returned by malloc().  This can be used to test
  * that type conversions handle non-aligned buffers correctly.
  */
-#define ALIGNMENT       1
+#define ALIGNMENT        1
 
 /*
  * Define if you want to test alignment code on a machine that doesn't
@@ -47,17 +47,20 @@ using namespace H5;
 
 /* Alignment test stuff */
 #ifdef TEST_ALIGNMENT
-#define H5T_PACKAGE
+#define H5T_FRIEND
 #include "H5Tpkg.h"
 #endif
 #define SET_ALIGNMENT(TYPE,VAL) \
     H5T_NATIVE_##TYPE##_ALIGN_g=MAX(H5T_NATIVE_##TYPE##_ALIGN_g, VAL)
 #endif
+ /* #include "H5Tpkg.h"
+ */ 
 
 const char *FILENAME[] = {
     "dtypes1.h5",
     "dtypes2.h5",
     "dtypes3.h5",
+    "dtypes4.h5",
     NULL
 };
 
@@ -66,7 +69,7 @@ const char *FILENAME[] = {
  * endian.  If local variable `endian' is H5T_ORDER_BE then the result will
  * be I, otherwise the result will be Z-(I+1).
  */
-#define ENDIAN(Z,I)     (H5T_ORDER_BE==endian?(I):(Z)-((I)+1))
+#define ENDIAN(Z,I)        (H5T_ORDER_BE==endian?(I):(Z)-((I)+1))
 
 
 typedef enum flt_t {
@@ -84,17 +87,17 @@ typedef struct {
     long   c;
     double d;
 } src_typ_t;
+
 
 /*-------------------------------------------------------------------------
  * Function:    test_classes
  *
- * Purpose:     Test type classes
+ * Purpose      Test type classes
  *
- * Return:      None
+ * Return       None.
  *
- * Programmer:  Binh-Minh Ribler (using C version)
+ * Programmer   Binh-Minh Ribler (using C version)
  *              January, 2007
- *
  *-------------------------------------------------------------------------
  */
 static void test_classes()
@@ -121,21 +124,22 @@ static void test_classes()
         issue_fail_msg("test_classes", __LINE__, __FILE__, E.getCDetailMsg());
     }
 }
+
 
 /*-------------------------------------------------------------------------
  * Function:    test_copy
  *
- * Purpose:     Test datatype copy functionality
+ * Purpose      Test datatype copy functionality
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Binh-Minh Ribler (using C version)
+ * Programmer   Binh-Minh Ribler (using C version)
  *              January, 2007
- *
  *-------------------------------------------------------------------------
  */
 static void test_copy()
 {
+
     SUBTEST("DataType::copy() and DataType::operator=");
     try {
         // Test copying from a predefined datatype using DataType::operator=
@@ -176,24 +180,289 @@ static void test_copy()
 
 
 /*-------------------------------------------------------------------------
- * Function:    test_query
+ * Function:    test_detect_type_class
  *
- * Purpose:     Tests query functions of compound and enumeration types.
+ * Purpose      Test DataType::detectClass()
  *
- * Return:      None
- *
- * Programmer:  Binh-Minh Ribler (use C version)
- *              January, 2007
+ * Return       None
  *
+ * Programmer   Binh-Minh Ribler (using C version)
+ *              August, 2017
  *-------------------------------------------------------------------------
  */
+typedef struct {     /* Struct with atomic fields */
+        int i;
+        float f;
+        char c;
+        double d;
+        short s;
+} atomic_typ_t;
 
+typedef struct {    /* Struct with complex fields */
+        hobj_ref_t arr_r[3][3];
+        int i;
+        hvl_t vl_f;
+        hvl_t vl_s;
+        char c;
+        short s;
+} complex_typ_t;
+
+static void test_detect_type_class()
+{
+
+    SUBTEST("DataType::detectClass()");
+    try {
+        bool in_class = false;      // indicates whether a datatype is in a class
+
+        /*
+         * Test class of some atomic types.
+         */
+
+        // Native integers should be in the integer class
+        in_class = DataType::detectClass(PredType::NATIVE_INT, H5T_INTEGER);
+        verify_val(in_class, true, "DataType::detectClass() with H5T_INTEGER", __LINE__, __FILE__);
+
+        // Native integers should _not_ be in other classes
+        in_class = DataType::detectClass(PredType::NATIVE_INT, H5T_FLOAT);
+        verify_val(in_class, false, "DataType::detectClass() with H5T_FLOAT", __LINE__, __FILE__);
+        in_class = DataType::detectClass(PredType::NATIVE_INT, H5T_ARRAY);
+        verify_val(in_class, false, "DataType::detectClass() with H5T_ARRAY", __LINE__, __FILE__);
+        in_class = DataType::detectClass(PredType::NATIVE_INT, H5T_ENUM);
+        verify_val(in_class, false, "DataType::detectClass() with H5T_ENUM", __LINE__, __FILE__);
+
+        /*
+         *  Test class of a compound type with some atomic types as fields.
+         */
+
+        // Create a compound datatype and insert some atomic types
+        CompType atom_cmpd(sizeof(atomic_typ_t));
+        atom_cmpd.insertMember("i", HOFFSET(atomic_typ_t, i), PredType::NATIVE_INT);
+        atom_cmpd.insertMember("f", HOFFSET(atomic_typ_t, f), PredType::NATIVE_FLOAT);
+        atom_cmpd.insertMember("c", HOFFSET(atomic_typ_t, c), PredType::NATIVE_CHAR);
+        atom_cmpd.insertMember("d", HOFFSET(atomic_typ_t, d), PredType::NATIVE_DOUBLE);
+        atom_cmpd.insertMember("s", HOFFSET(atomic_typ_t, s), PredType::NATIVE_SHORT);
+
+        // Make certain that atom_cmpd is a compound type,
+        in_class = atom_cmpd.detectClass(H5T_COMPOUND);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_COMPOUND", __LINE__, __FILE__);
+        // and that it contains a field of type integer
+        in_class = atom_cmpd.detectClass(H5T_INTEGER);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_INTEGER", __LINE__, __FILE__);
+        // and a field of type float,
+        in_class = atom_cmpd.detectClass(H5T_FLOAT);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_FLOAT", __LINE__, __FILE__);
+        // and that it doesn't contain any field of variable-length
+        in_class = atom_cmpd.detectClass(H5T_VLEN);
+        verify_val(in_class, false, "CompType::detectClass() with H5T_VLEN", __LINE__, __FILE__);
+
+        /*
+         *  Test class of array datatype
+         */
+
+        // Create an array datatype with an atomic base type
+        unsigned rank = 2;          // Rank for array datatype
+        hsize_t dims[2] = {3,3};    // Dimensions for array datatype
+        ArrayType atom_arr(PredType::STD_REF_OBJ, rank, dims);
+
+        // Make certain that the correct classes can be detected
+        in_class = atom_arr.detectClass(H5T_ARRAY);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_ARRAY", __LINE__, __FILE__);
+        in_class = atom_arr.detectClass(H5T_REFERENCE);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_REFERENCE", __LINE__, __FILE__);
+
+        // Make certain that an incorrect class is not detected
+        in_class = atom_arr.detectClass(H5T_VLEN);
+        verify_val(in_class, false, "CompType::detectClass() with H5T_VLEN", __LINE__, __FILE__);
+        in_class = atom_arr.detectClass(H5T_FLOAT);
+        verify_val(in_class, false, "CompType::detectClass() with H5T_FLOAT", __LINE__, __FILE__);
+        in_class = atom_arr.detectClass(H5T_INTEGER);
+        verify_val(in_class, false, "CompType::detectClass() with H5T_INTEGER", __LINE__, __FILE__);
+
+        /*
+         *  Test class of VL datatype
+         */
+
+        // Create a VL datatype with an atomic base type of float
+        VarLenType atom_vlf(PredType::NATIVE_FLOAT);
+
+        // Make certain that the correct classes can be detected
+        in_class = atom_vlf.detectClass(H5T_VLEN);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_VLEN", __LINE__, __FILE__);
+        in_class = atom_vlf.detectClass(H5T_FLOAT);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_FLOAT", __LINE__, __FILE__);
+
+        // Make certain that an incorrect class is not detected
+        in_class = atom_vlf.detectClass(H5T_COMPOUND);
+        verify_val(in_class, false, "CompType::detectClass() with H5T_COMPOUND", __LINE__, __FILE__);
+        in_class = atom_vlf.detectClass(H5T_INTEGER);
+        verify_val(in_class, false, "CompType::detectClass() with H5T_INTEGER", __LINE__, __FILE__);
+
+        /*
+         *  Test class of VL datatype
+         */
+
+        // Create a VL datatype with an atomic base type of char.  It should be a VL
+        // but not a string class.
+        VarLenType atom_vlc(PredType::NATIVE_CHAR);
+
+        // Make certain that the correct classes can be detected
+        in_class = atom_vlc.detectClass(H5T_VLEN);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_VLEN", __LINE__, __FILE__);
+        in_class = atom_vlc.detectClass(H5T_INTEGER);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_INTEGER", __LINE__, __FILE__);
+
+        // Make certain that an incorrect class is not detected
+        in_class = atom_vlc.detectClass(H5T_STRING);
+        verify_val(in_class, false, "CompType::detectClass() with H5T_STRING", __LINE__, __FILE__);
+
+        /*
+         *  Test class of VL string datatype
+         */
+
+        // Create a VL string.  It should be a string, not a VL class.
+        StrType atom_vls(0, H5T_VARIABLE);
+
+        // Make certain that the correct classes can be detected
+        in_class = atom_vls.detectClass(H5T_STRING);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_STRING", __LINE__, __FILE__);
+
+        // Make certain that an incorrect class is not detected
+        in_class = atom_vls.detectClass(H5T_VLEN);
+        verify_val(in_class, false, "CompType::detectClass() with H5T_VLEN", __LINE__, __FILE__);
+
+        /*
+         *  Test class of a compound type with some complex types as fields.
+         */
+
+        // Create a compound datatype with complex type fields
+        CompType cplx_cmpd(sizeof(complex_typ_t));
+        cplx_cmpd.insertMember("arr_r", HOFFSET(complex_typ_t, arr_r), atom_arr);
+        cplx_cmpd.insertMember("i", HOFFSET(complex_typ_t, i), PredType::NATIVE_INT);
+        cplx_cmpd.insertMember("vl_f", HOFFSET(complex_typ_t, vl_f), atom_vlf);
+        cplx_cmpd.insertMember("vl_s", HOFFSET(complex_typ_t, vl_s), atom_vls);
+        cplx_cmpd.insertMember("c", HOFFSET(complex_typ_t, c), PredType::NATIVE_CHAR);
+        cplx_cmpd.insertMember("s", HOFFSET(complex_typ_t, s), PredType::NATIVE_SHORT);
+
+        // Make certain that the correct classes can be detected
+        in_class = cplx_cmpd.detectClass(H5T_COMPOUND);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_COMPOUND", __LINE__, __FILE__);
+        in_class = cplx_cmpd.detectClass(H5T_ARRAY);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_ARRAY", __LINE__, __FILE__);
+        in_class = cplx_cmpd.detectClass(H5T_REFERENCE);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_REFERENCE", __LINE__, __FILE__);
+        in_class = cplx_cmpd.detectClass(H5T_INTEGER);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_INTEGER", __LINE__, __FILE__);
+        in_class = cplx_cmpd.detectClass(H5T_FLOAT);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_FLOAT", __LINE__, __FILE__);
+        in_class = cplx_cmpd.detectClass(H5T_STRING);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_STRING", __LINE__, __FILE__);
+        in_class = cplx_cmpd.detectClass(H5T_VLEN);
+        verify_val(in_class, true, "CompType::detectClass() with H5T_VLEN", __LINE__, __FILE__);
+
+        // Make certain that an incorrect class is not detected
+        in_class = cplx_cmpd.detectClass(H5T_TIME);
+        verify_val(in_class, false, "CompType::detectClass() with H5T_TIME", __LINE__, __FILE__);
+        in_class = cplx_cmpd.detectClass(H5T_ENUM);
+        verify_val(in_class, false, "CompType::detectClass() with H5T_ENUM", __LINE__, __FILE__);
+
+        PASSED();
+    }
+    catch (Exception& E)
+    {
+        issue_fail_msg("test_detect_type_class", __LINE__, __FILE__, E.getCDetailMsg());
+    }
+}
+
+
+/*-------------------------------------------------------------------------
+ * Function:    test_vltype
+ *
+ * Purpose      Tests VarLenType class
+ *
+ * Return       None
+ *
+ * Programmer   Binh-Minh Ribler (use C version)
+ *              August, 2017
+ *-------------------------------------------------------------------------
+ */
+static void test_vltype()
+{
+    // Output message about test being performed
+    SUBTEST("VarLenType functions");
+    try
+    {
+        VarLenType vltype(PredType::NATIVE_INT);
+        
+        bool in_class = vltype.detectClass(H5T_VLEN);
+        verify_val(in_class, true, "VarLenType::detectClass() with H5T_VLEN", __LINE__, __FILE__);
+        in_class = vltype.detectClass(H5T_INTEGER);
+        verify_val(in_class, true, "VarLenType::detectClass() with H5T_INTEGER", __LINE__, __FILE__);
+
+        // Test copy constructor
+        VarLenType vltype2(vltype);
+
+        // Verify that the copied type has a valid id
+        bool is_valid = IdComponent::isValid(vltype2.getId());
+        verify_val(in_class, true, "isValid on vltype2", __LINE__, __FILE__);
+
+        in_class = vltype2.detectClass(H5T_VLEN);
+        verify_val(in_class, true, "VarLenType::detectClass() with H5T_VLEN for vltype2", __LINE__, __FILE__);
+        in_class = vltype2.detectClass(H5T_INTEGER);
+        verify_val(in_class, true, "VarLenType::detectClass() with H5T_INTEGER for vltype2", __LINE__, __FILE__);
+        in_class = vltype2.detectClass(H5T_FLOAT);
+        verify_val(in_class, false, "VarLenType::detectClass() with H5T_FLOAT for vltype2", __LINE__, __FILE__);
+
+        // Create a new file to use in this test
+        H5File file(FILENAME[3], H5F_ACC_TRUNC);
+
+        // Create a group in the file, to hold some varlentype
+        Group top_group(file.createGroup("top group"));
+
+        // Create a variable-length type
+        VarLenType first_vlt(PredType::NATIVE_FLOAT);
+
+        // Commit the type to the group
+        first_vlt.commit(top_group, "first variable-length type");
+
+        // Close it
+        first_vlt.close();
+
+        // Reopen it
+        VarLenType first_vlt_again(top_group, "first variable-length type");
+
+        // Trying to detect H5T_VLEN and H5T_FLOAT classes on this type,
+        // should both be true
+        in_class = vltype2.detectClass(H5T_VLEN);
+        verify_val(in_class, true, "VarLenType::detectClass() with H5T_VLEN for vltype2", __LINE__, __FILE__);
+        in_class = first_vlt_again.detectClass(H5T_FLOAT);
+        verify_val(in_class, true, "VarLenType::detectClass() with H5T_FLOAT for first_vlt_again", __LINE__, __FILE__);
+
+        PASSED();
+    }   // end of try block
+    catch (Exception& E)
+    {
+        issue_fail_msg("test_vltype", __LINE__, __FILE__, E.getCDetailMsg());
+    }
+}   // test_vltype
+
+
+/*-------------------------------------------------------------------------
+ * Function:    test_query
+ *
+ * Purpose      Tests query functions of compound and enumeration types.
+ *
+ * Return       None
+ *
+ * Programmer   Binh-Minh Ribler (use C version)
+ *              January, 2007
+ *-------------------------------------------------------------------------
+ */
 const H5std_string CompT_NAME("Compound_type");
 const H5std_string EnumT_NAME("Enum_type");
 
 static void test_query()
 {
-    short       enum_val;
+    short        enum_val;
 
     // Output message about test being performed
     SUBTEST("Query functions of compound and enumeration types");
@@ -240,7 +509,7 @@ static void test_query()
         if (!IdComponent::isValid(tcpl.getId()))
         {
             // Throw an invalid action exception
-            throw InvalidActionException("IdComponent::isValid", "Datatype creation property list is not valid");
+            throw InvalidActionException("H5Object::createAttribute", "Datatype creation property list is not valid");
         }
         tcpl.close();
         tid1.close();
@@ -252,34 +521,40 @@ static void test_query()
         if (!IdComponent::isValid(tcpl.getId()))
         {
             // Throw an invalid action exception
-            throw InvalidActionException("IdComponent::isValid", "Datatype creation property list is not valid");
+            throw InvalidActionException("H5Object::createAttribute", "Datatype creation property list is not valid");
         }
         tcpl.close();
         tid2.close();
 
-        // Open the datatypes for query
+        // Open the datatypes for query.  Testing both ways
+
         tid1 = file.openCompType(CompT_NAME);
+        tid1.close();
         tid2 = file.openEnumType(EnumT_NAME);
+        tid2.close();
+
+        CompType comptype(file, CompT_NAME);
+        EnumType enumtype(file, EnumT_NAME);
 
         // Query member number and member index by name, for compound type
-        nmembs = tid1.getNmembers();
+        nmembs = comptype.getNmembers();
         verify_val(nmembs, 4, "CompType::getNmembers()", __LINE__, __FILE__);
-        index = tid1.getMemberIndex("c");
+        index = comptype.getMemberIndex("c");
         verify_val(index, 2, "CompType::getMemberIndex()", __LINE__, __FILE__);
 
         // Query member number and member index by name, for enumeration type
-        nmembs = tid2.getNmembers();
+        nmembs = enumtype.getNmembers();
         verify_val(nmembs, 5, "EnumType::getNmembers()", __LINE__, __FILE__);
-        index = tid2.getMemberIndex("ORANGE");
+        index = enumtype.getMemberIndex("ORANGE");
         verify_val(index, 3, "EnumType::getMemberIndex()", __LINE__, __FILE__);
 
         // Close datatypes and file
-        tid1.close();
-        tid2.close();
+        comptype.close();
+        enumtype.close();
         file.close();
 
         // Try truncating the file to make sure reference counting is good.
-        // If any references to ids of tid1 and tid2 are left unterminated,
+        // If any references to ids of the accessed types are left unterminated,
         // the truncating will fail, because the file will not be closed in
         // the file.close() above.
         H5File file1(FILENAME[2], H5F_ACC_TRUNC);
@@ -296,19 +571,19 @@ static void test_query()
 /*-------------------------------------------------------------------------
  * Function:    test_transient
  *
- * Purpose:     Tests transient datatypes.
+ * Purpose      Tests transient datatypes.
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Binh-Minh Ribler (use C version)
+ * Programmer   Binh-Minh Ribler (use C version)
  *              January, 2007
- *
  *-------------------------------------------------------------------------
  */
 const char* filename1 = "dtypes1.h5";
+
 static void test_transient ()
 {
-    static hsize_t      ds_size[2] = {10, 20};
+    static hsize_t        ds_size[2] = {10, 20};
 
     SUBTEST("Transient datatypes");
     try {
@@ -371,22 +646,22 @@ static void test_transient ()
 /*-------------------------------------------------------------------------
  * Function:    test_named
  *
- * Purpose:     Tests named datatypes.
+ * Purpose      Tests named datatypes.
  *
- * Return:      None
+ * Return       None
  *
- * Programmer:  Binh-Minh Ribler (use C version)
+ * Programmer   Binh-Minh Ribler (use C version)
  *              January, 2007
- *
  *-------------------------------------------------------------------------
  */
 const H5std_string filename2("dtypes2.h5");
+
 static void test_named ()
 {
-    static hsize_t      ds_size[2] = {10, 20};
-    hsize_t             i;
-    unsigned            attr_data[10][20];
-    DataType           *ds_type = NULL;
+    static hsize_t ds_size[2] = {10, 20};
+    hsize_t i;
+    unsigned attr_data[10][20];
+    DataType *ds_type = NULL;
 
     SUBTEST("Named datatypes");
     try {
@@ -433,7 +708,7 @@ static void test_named ()
 
         // We should not be able to modify a type after it has been committed.
         try {
-            itype.setPrecision(256);    // attempt an invalid action...
+            itype.setPrecision(256);        // attempt an invalid action...
 
             // Should FAIL but didn't, so throw an invalid action exception
             throw InvalidActionException("IntType::setPrecision", "Attempted to modify a committed datatype.");
@@ -532,18 +807,18 @@ static void test_named ()
 /*-------------------------------------------------------------------------
  * Function:    test_encode_decode
  *
- * Purpose:     Test datatype encode/decode functionality.
+ * Purpose      Test datatype encode/decode functionality.
  *
- * Return:      None
- *
- * Programmer:  Binh-Minh Ribler (using C version)
- *              October, 2017
+ * Return       None
  *
+ * Programmer   Binh-Minh Ribler (using C version)
+ *              August, 2017
  *-------------------------------------------------------------------------
  */
 const H5std_string filename3("encode_decode.h5");
 const int ARRAY1_RANK = 1;
 const int ARRAY1_DIM = 10;
+
 static void test_encode_decode()
 {
     short        enum_val;
@@ -750,11 +1025,14 @@ static void test_encode_decode()
 }
 
 
-/****************************************************************
-**
-**  test_types(): Main datatypes testing routine.
-**
-****************************************************************/
+/*-------------------------------------------------------------------------
+ * Function:    test_types
+ *
+ * Purpose      Main datatypes testing routine
+ *
+ * Return       None
+ *-------------------------------------------------------------------------
+ */
 extern "C"
 void test_types()
 {
@@ -764,6 +1042,8 @@ void test_types()
     // Test basic datatypes
     test_classes();
     test_copy();
+    test_detect_type_class();
+    test_vltype();
     test_query();
     test_transient();
     test_named();
@@ -775,13 +1055,9 @@ void test_types()
 /*-------------------------------------------------------------------------
  * Function:    cleanup_types
  *
- * Purpose:     Cleanup temporary test files
- *
- * Return:      none
- *
- * Programmer:  Quincey Koziol
- *              September 10, 1999
+ * Purpose      Cleanup temporary test files
  *
+ * Return       None
  *-------------------------------------------------------------------------
  */
 extern "C"
