@@ -320,7 +320,7 @@ herr_t H5DSattach_scale(hid_t did,
             ref_j = ((hobj_ref_t *)buf[idx].p)[i];
 
             /* get the scale id for this REF */
-            if((dsid_j = H5Rdereference(did,H5R_OBJECT,&ref_j)) < 0)
+            if((dsid_j = H5Rdereference2(did, H5P_DEFAULT, H5R_OBJECT, &ref_j)) < 0)
                 goto out;
 
             /* get info for DS in the parameter list */
@@ -692,7 +692,7 @@ herr_t H5DSdetach_scale(hid_t did,
             ref = ((hobj_ref_t *)buf[idx].p)[j];
 
             /* get the DS id */
-            if ((dsid_j = H5Rdereference(did,H5R_OBJECT,&ref)) < 0)
+            if ((dsid_j = H5Rdereference2(did,H5P_DEFAULT,H5R_OBJECT,&ref)) < 0)
                 goto out;
 
             /* get info for this DS */
@@ -802,7 +802,7 @@ herr_t H5DSdetach_scale(hid_t did,
             ref = dsbuf[ii].ref;
 
             /* get the dataset id */
-            if ((did_i = H5Rdereference(did,H5R_OBJECT,&ref)) < 0)
+            if ((did_i = H5Rdereference2(did,H5P_DEFAULT,H5R_OBJECT,&ref)) < 0)
                 goto out;
 
             /* get info for this dataset */
@@ -1050,7 +1050,7 @@ htri_t H5DSis_attached(hid_t did,
             ref = ((hobj_ref_t *)buf[idx].p)[i];
 
             /* get the scale id for this REF */
-            if ((dsid_j = H5Rdereference(did,H5R_OBJECT,&ref)) < 0)
+            if ((dsid_j = H5Rdereference2(did,H5P_DEFAULT,H5R_OBJECT,&ref)) < 0)
                 goto out;
 
             /* get info for DS in the parameter list */
@@ -1140,7 +1140,7 @@ htri_t H5DSis_attached(hid_t did,
             if (ref)
             {
                 /* get the dataset id */
-                if ((did_i = H5Rdereference(did,H5R_OBJECT,&ref)) < 0)
+                if ((did_i = H5Rdereference2(did,H5P_DEFAULT,H5R_OBJECT,&ref)) < 0)
                     goto out;
 
                 /* get info for dataset in the parameter list */
@@ -1344,7 +1344,7 @@ herr_t H5DSiterate_scales(hid_t did,
                 /* disable error reporting, the ID might refer to a deleted dataset */
                 H5E_BEGIN_TRY {
                     /* get the DS id */
-                    if ((scale_id = H5Rdereference(did,H5R_OBJECT,&ref)) < 0)
+                    if ((scale_id = H5Rdereference2(did,H5P_DEFAULT,H5R_OBJECT,&ref)) < 0)
                         goto out;
                 } H5E_END_TRY;
 
@@ -1883,7 +1883,7 @@ ssize_t H5DSget_scale_name(hid_t did,
     if (buf)
         HDfree(buf);
 
-    return (ssize_t) MAX(0,nbytes-1);
+    return (ssize_t)(nbytes - 1);
 
     /* error zone */
 out:
