@@ -14,11 +14,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include "h5hltest.h"
-#include "H5srcdir.h"
 #include "H5DOpublic.h"
 #include <math.h>
 
-#if defined(H5_HAVE_ZLIB_H) && !defined(H5_ZLIB_HEADER)
+#if defined(H5_HAVE_ZLIB_H) && !defined(H5_ZLIB_HEADER) 
 # define H5_ZLIB_HEADER "zlib.h"
 #endif
 #if defined(H5_ZLIB_HEADER)
@@ -35,7 +34,6 @@
 #define DATASETNAME5        "contiguous_dset"
 #define DATASETNAME6        "invalid_argue"
 #define DATASETNAME7        "overwrite_chunk"
-
 /* Datasets for Direct Read tests */
 #define DATASETNAME8        "disabled_chunk_cache"
 #define DATASETNAME9        "flush_chunk_cache"
@@ -49,13 +47,13 @@
 #define CHUNK_NX     4
 #define CHUNK_NY     4
 
-#define DEFLATE_SIZE_ADJUST(s) (ceil(((double)(s))*1.001)+12)
+#define DEFLATE_SIZE_ADJUST(s) (HDceil(((double)(s))*H5_DOUBLE(1.001))+H5_DOUBLE(12.0))
 
 /* Temporary filter IDs used for testing */
-#define H5Z_FILTER_BOGUS1    305
-#define H5Z_FILTER_BOGUS2    306
-#define ADD_ON            7
-#define FACTOR            3
+#define H5Z_FILTER_BOGUS1	305
+#define H5Z_FILTER_BOGUS2	306
+#define ADD_ON			7
+#define FACTOR			3
 
 /* Constants for the overwrite test */
 #define OVERWRITE_NDIMS         3
@@ -64,6 +62,14 @@
 #define OVERWRITE_CHUNK_NY      2
 #define OVERWRITE_VALUE         42
 
+/* Defines used in test_single_chunk_latest() */
+#define FILE            "single_latest.h5"
+#define DATASET         "dataset"
+#define DIM0            4
+#define DIM1            32
+#define CHUNK0          DIM0
+#define CHUNK1          DIM1
+
 /* Local prototypes for filter functions */
 static size_t filter_bogus1(unsigned int flags, size_t cd_nelmts,
     const unsigned int *cd_values, size_t nbytes, size_t *buf_size, void **buf);
@@ -73,33 +79,33 @@ static size_t filter_bogus2(unsigned int flags, size_t cd_nelmts,
 /* This message derives from H5Z */
 const H5Z_class2_t H5Z_BOGUS1[1] = {{
     H5Z_CLASS_T_VERS,       /* H5Z_class_t version */
-    H5Z_FILTER_BOGUS1,        /* Filter id number        */
+    H5Z_FILTER_BOGUS1,		/* Filter id number		*/
     1, 1,               /* Encoding and decoding enabled */
-    "bogus1",            /* Filter name for debugging    */
+    "bogus1",			/* Filter name for debugging	*/
     NULL,                       /* The "can apply" callback     */
     NULL,                       /* The "set local" callback     */
-    filter_bogus1,        /* The actual filter function    */
+    filter_bogus1,		/* The actual filter function	*/
 }};
 
 const H5Z_class2_t H5Z_BOGUS2[1] = {{
     H5Z_CLASS_T_VERS,       /* H5Z_class_t version */
-    H5Z_FILTER_BOGUS2,        /* Filter id number        */
+    H5Z_FILTER_BOGUS2,		/* Filter id number		*/
     1, 1,               /* Encoding and decoding enabled */
-    "bogus2",            /* Filter name for debugging    */
+    "bogus2",			/* Filter name for debugging	*/
     NULL,                       /* The "can apply" callback     */
     NULL,                       /* The "set local" callback     */
-    filter_bogus2,        /* The actual filter function    */
+    filter_bogus2,		/* The actual filter function	*/
 }};
 
 /*-------------------------------------------------------------------------
- * Function:    test_direct_chunk_write
+ * Function:	test_direct_chunk_write
  *
- * Purpose:        Test the basic functionality of H5DOwrite_chunk
+ * Purpose:	    Test the basic functionality of H5DOwrite_chunk
  *
- * Return:        Success:    0
- *                Failure:    1
+ * Return:	    Success:	0
+ *		        Failure:	1
  *
- * Programmer:  Raymond Lu
+ * Programmer:  Raymond Lu	
  *              30 November 2012
  *
  *-------------------------------------------------------------------------
@@ -111,7 +117,7 @@ test_direct_chunk_write (hid_t file)
     hid_t       dataspace = -1, dataset = -1;
     hid_t       mem_space = -1;
     hid_t       cparms = -1, dxpl = -1;
-    hsize_t     dims[2]  = {NX, NY};
+    hsize_t     dims[2]  = {NX, NY};        
     hsize_t     maxdims[2] = {H5S_UNLIMITED, H5S_UNLIMITED};
     hsize_t     chunk_dims[2] ={CHUNK_NX, CHUNK_NY};
     herr_t      status;
@@ -126,11 +132,11 @@ test_direct_chunk_write (hid_t file)
     size_t      buf_size = CHUNK_NX*CHUNK_NY*sizeof(int);
 
     const Bytef *z_src = (const Bytef*)(direct_buf);
-    Bytef        *z_dst = NULL;            /*destination buffer        */
-    uLongf         z_dst_nbytes = (uLongf)DEFLATE_SIZE_ADJUST(buf_size);
-    uLong         z_src_nbytes = (uLong)buf_size;
+    Bytef	    *z_dst = NULL;		    /*destination buffer		*/
+    uLongf	     z_dst_nbytes = (uLongf)DEFLATE_SIZE_ADJUST(buf_size);
+    uLong	     z_src_nbytes = (uLong)buf_size;
     int          aggression = 9;        /* Compression aggression setting */
-    void        *outbuf = NULL;         /* Pointer to new buffer */
+    void	    *outbuf = NULL;         /* Pointer to new buffer */
 
     hsize_t start[2];  /* Start of hyperslab */
     hsize_t stride[2]; /* Stride of hyperslab */
@@ -165,29 +171,29 @@ test_direct_chunk_write (hid_t file)
      * creation properties.
      */
     if((dataset = H5Dcreate2(file, DATASETNAME1, H5T_NATIVE_INT, dataspace, H5P_DEFAULT,
-            cparms, H5P_DEFAULT)) < 0)
+			cparms, H5P_DEFAULT)) < 0)
         goto error;
 
     /* Initialize the dataset */
     for(i = n = 0; i < NX; i++)
         for(j = 0; j < NY; j++)
-        data[i][j] = n++;
+	    data[i][j] = n++;
 
     if((dxpl = H5Pcreate(H5P_DATASET_XFER)) < 0)
         goto error;
 
     /*
      * Write the data for the dataset.  It should stay in the chunk cache.
-     * It will be evicted from the cache by the H5DOwrite_chunk calls.
+     * It will be evicted from the cache by the H5DOwrite_chunk calls. 
      */
     if((status = H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL,
-            dxpl, data)) < 0)
+		      dxpl, data)) < 0)
         goto error;
 
     /* Initialize data for one chunk */
     for(i = n = 0; i < CHUNK_NX; i++)
         for(j = 0; j < CHUNK_NY; j++)
-        direct_buf[i][j] = n++;
+	    direct_buf[i][j] = n++;
 
     /* Allocate output (compressed) buffer */
     outbuf = HDmalloc(z_dst_nbytes);
@@ -208,8 +214,8 @@ test_direct_chunk_write (hid_t file)
         goto error;
     }
 
-    /* Write the compressed chunk data repeatedly to cover all the chunks in the
-     * dataset, using the direct writing function.     */
+    /* Write the compressed chunk data repeatedly to cover all the chunks in the 
+     * dataset, using the direct writing function.     */ 
     for(i=0; i<NX/CHUNK_NX; i++) {
         for(j=0; j<NY/CHUNK_NY; j++) {
             status = H5DOwrite_chunk(dataset, dxpl, filter_mask, offset, z_dst_nbytes, outbuf);
@@ -222,7 +228,7 @@ test_direct_chunk_write (hid_t file)
     if(outbuf)
         HDfree(outbuf);
 
-    if(H5Fflush(dataset, H5F_SCOPE_LOCAL) < 0)
+    if(H5Fflush(dataset, H5F_SCOPE_LOCAL) < 0) 
         goto error;
 
     if(H5Dclose(dataset) < 0)
@@ -251,7 +257,7 @@ test_direct_chunk_write (hid_t file)
             if(direct_buf[i][j] != check_chunk[i][j]) {
                 HDprintf("    1. Read different values than written.");
                 HDprintf("    At index %d,%d\n", i, j);
-                HDprintf("    direct_buf=%d, check_chunk=%d\n", direct_buf[i][j], check_chunk[i][j]);
+                HDprintf("    direct_buf=%d, check_chunk=%d\n", direct_buf[i][j], check_chunk[i][j]); 
                 goto error;
             }
         }
@@ -260,7 +266,7 @@ test_direct_chunk_write (hid_t file)
     /* Reinitialize different data for one chunk */
     for(i = 0; i < CHUNK_NX; i++)
         for(j = 0; j < CHUNK_NY; j++)
-        direct_buf[i][j] = i + j;
+	    direct_buf[i][j] = i + j;
 
     /* Allocate output (compressed) buffer */
     outbuf = HDmalloc(z_dst_nbytes);
@@ -281,8 +287,8 @@ test_direct_chunk_write (hid_t file)
         goto error;
     }
 
-    /* Rewrite the compressed chunk data repeatedly to cover all the chunks in the
-     * dataset, using the direct writing function.     */
+    /* Rewrite the compressed chunk data repeatedly to cover all the chunks in the 
+     * dataset, using the direct writing function.     */ 
     offset[0] = offset[1] = 0;
     for(i=0; i<NX/CHUNK_NX; i++) {
         for(j=0; j<NY/CHUNK_NY; j++) {
@@ -296,7 +302,7 @@ test_direct_chunk_write (hid_t file)
     if(outbuf)
         HDfree(outbuf);
 
-    if(H5Fflush(dataset, H5F_SCOPE_LOCAL) < 0)
+    if(H5Fflush(dataset, H5F_SCOPE_LOCAL) < 0) 
         goto error;
 
     if(H5Dclose(dataset) < 0)
@@ -315,7 +321,7 @@ test_direct_chunk_write (hid_t file)
             if(direct_buf[i][j] != check_chunk[i][j]) {
                 HDprintf("    2. Read different values than written.");
                 HDprintf("    At index %d,%d\n", i, j);
-                HDprintf("    direct_buf=%d, check_chunk=%d\n", direct_buf[i][j], check_chunk[i][j]);
+                HDprintf("    direct_buf=%d, check_chunk=%d\n", direct_buf[i][j], check_chunk[i][j]); 
                 goto error;
             }
         }
@@ -329,7 +335,7 @@ test_direct_chunk_write (hid_t file)
     H5Sclose(dataspace);
     H5Pclose(cparms);
     H5Pclose(dxpl);
-
+    
     PASSED();
     return 0;
 
@@ -347,7 +353,7 @@ error:
 
     H5_FAILED();
     return 1;
-}
+} /* test_direct_chunk_write() */
 #endif /* H5_HAVE_FILTER_DEFLATE */
 
 /*-------------------------------------------------------------------------
@@ -459,13 +465,13 @@ error:
 /*-------------------------------------------------------------------------
  * Function:	test_skip_compress_write1
  *
- * Purpose:    Test skipping compression filter when it is the only filter
+ * Purpose:	Test skipping compression filter when it is the only filter
  *              for the dataset
  *
- * Return:    Success:    0
- *            Failure:    1
+ * Return:	Success:	0
+ *		    Failure:	1
  *
- * Programmer:  Raymond Lu
+ * Programmer:  Raymond Lu	
  *              30 November 2012
  *
  *-------------------------------------------------------------------------
@@ -526,7 +532,7 @@ test_skip_compress_write1(hid_t file)
      * creation properties.
      */
     if((dataset = H5Dcreate2(file, DATASETNAME2, H5T_NATIVE_INT, dataspace, H5P_DEFAULT,
-            cparms, H5P_DEFAULT)) < 0)
+			cparms, H5P_DEFAULT)) < 0)
         goto error;
 
     if((dxpl = H5Pcreate(H5P_DATASET_XFER)) < 0)
@@ -535,7 +541,7 @@ test_skip_compress_write1(hid_t file)
     /* Initialize data for one chunk */
     for(i = n = 0; i < CHUNK_NX; i++)
         for(j = 0; j < CHUNK_NY; j++) {
-        direct_buf[i][j] = n++;
+            direct_buf[i][j] = n++;
     }
 
     /* write the uncompressed chunk data repeatedly to dataset, using the direct writing function.
@@ -634,13 +640,13 @@ error:
 } /* test_skip_compress_write1() */
 
 /*-------------------------------------------------------------------------
- * Function:    filter_bogus1
+ * Function:	filter_bogus1
  *
- * Purpose:     A bogus filter that adds ADD_ON to the original value
+ * Purpose:	    A bogus filter that adds ADD_ON to the original value
  *
- * Return:      Success:    Data chunk size
+ * Return:	    Success:	Data chunk size
  *
- * Programmer:  Raymond Lu
+ * Programmer:  Raymond Lu	
  *              30 November 2012
  *
  *-------------------------------------------------------------------------
@@ -672,13 +678,13 @@ filter_bogus1(unsigned int flags, size_t H5_ATTR_UNUSED cd_nelmts,
 } /* filter_bogus1() */
 
 /*-------------------------------------------------------------------------
- * Function:    filter_bogus2
+ * Function:	filter_bogus2
  *
- * Purpose:    A bogus filter that multiplies the original value by FACTOR.
+ * Purpose:	A bogus filter that multiplies the original value by FACTOR.
  *
- * Return:    Success:    Data chunk size
+ * Return:	Success:	Data chunk size
  *
- * Programmer:  Raymond Lu
+ * Programmer:  Raymond Lu	
  *              30 November 2012
  *-------------------------------------------------------------------------
  */
@@ -709,15 +715,15 @@ filter_bogus2(unsigned int flags, size_t H5_ATTR_UNUSED cd_nelmts,
 } /* filter_bogus2() */
 
 /*-------------------------------------------------------------------------
- * Function:    test_skip_compress_write2
+ * Function:	test_skip_compress_write2
  *
- * Purpose:    Test skipping compression filter when there are three filters
+ * Purpose:	Test skipping compression filter when there are three filters
  *              for the dataset
  *
- * Return:    Success:    0
- *            Failure:    1
+ * Return:	Success:	0
+ *		    Failure:	1
  *
- * Programmer:  Raymond Lu
+ * Programmer:  Raymond Lu	
  *              30 November 2012
  *
  *-------------------------------------------------------------------------
@@ -795,7 +801,7 @@ test_skip_compress_write2(hid_t file)
      * creation properties.
      */
     if((dataset = H5Dcreate2(file, DATASETNAME3, H5T_NATIVE_INT, dataspace, H5P_DEFAULT,
-            cparms, H5P_DEFAULT)) < 0)
+			cparms, H5P_DEFAULT)) < 0)
         goto error;
 
     if((dxpl = H5Pcreate(H5P_DATASET_XFER)) < 0)
@@ -905,14 +911,14 @@ error:
 } /* test_skip_compress_write2() */
 
 /*-------------------------------------------------------------------------
- * Function:    test_data_conv
+ * Function:	test_data_conv
  *
- * Purpose:    Test data conversion
+ * Purpose:	Test data conversion
  *
- * Return:    Success:    0
- *            Failure:    1
+ * Return:	Success:	0
+ *		    Failure:	1
  *
- * Programmer:  Raymond Lu
+ * Programmer:  Raymond Lu	
  *              30 November 2012
  *
  *-------------------------------------------------------------------------
@@ -930,13 +936,13 @@ test_data_conv(hid_t file)
     hid_t       dataspace = -1, dataset = -1;
     hid_t       mem_space = -1;
     hid_t       cparms = -1, dxpl = -1;
-    hsize_t     dims[2]  = {NX, NY};
+    hsize_t     dims[2]  = {NX, NY};        
     hsize_t     maxdims[2] = {H5S_UNLIMITED, H5S_UNLIMITED};
     hsize_t     chunk_dims[2] ={CHUNK_NX, CHUNK_NY};
     herr_t      status;
     int         i, j, n;
-    const hsize_t    four = 4;
-    hid_t    st=-1, dt=-1;
+    const hsize_t	four = 4;
+    hid_t	st=-1, dt=-1;
     hid_t       array_dt;
 
     unsigned    filter_mask = 0;
@@ -1000,7 +1006,7 @@ test_data_conv(hid_t file)
      * creation properties.
      */
     if((dataset = H5Dcreate2(file, DATASETNAME4, st, dataspace, H5P_DEFAULT,
-            cparms, H5P_DEFAULT)) < 0)
+			cparms, H5P_DEFAULT)) < 0)
         goto error;
 
     if((dxpl = H5Pcreate(H5P_DATASET_XFER)) < 0)
@@ -1020,7 +1026,7 @@ test_data_conv(hid_t file)
         }
     }
 
-    /* write the chunk data to dataset, using the direct writing function.
+    /* write the chunk data to dataset, using the direct writing function. 
      * There should be no data conversion involved. */
     offset[0] = CHUNK_NX;
     offset[1] = CHUNK_NY;
@@ -1028,7 +1034,7 @@ test_data_conv(hid_t file)
     if((status = H5DOwrite_chunk(dataset, dxpl, filter_mask, offset, buf_size, direct_buf)) < 0)
         goto error;
 
-    if(H5Fflush(dataset, H5F_SCOPE_LOCAL) < 0)
+    if(H5Fflush(dataset, H5F_SCOPE_LOCAL) < 0) 
         goto error;
 
     if(H5Dclose(dataset) < 0)
@@ -1055,10 +1061,10 @@ test_data_conv(hid_t file)
                     HDprintf("    1. Read different values than written.");
                     HDprintf("    At index %d,%d\n", i, j);
                     HDprintf("    src={a=%d, b=%d, c=[%d,%d,%d,%d], d=%d, e=%d\n",
-                        (direct_buf[i][j]).a, (direct_buf[i][j]).b, (direct_buf[i][j]).c[0], (direct_buf[i][j]).c[1],
+                        (direct_buf[i][j]).a, (direct_buf[i][j]).b, (direct_buf[i][j]).c[0], (direct_buf[i][j]).c[1], 
                         (direct_buf[i][j]).c[2], (direct_buf[i][j]).c[3], (direct_buf[i][j]).d, (direct_buf[i][j]).e);
                     HDprintf("    dst={a=%d, b=%d, c=[%d,%d,%d,%d], d=%d, e=%d\n",
-                        (read_chunk[i][j]).a, (read_chunk[i][j]).b, (read_chunk[i][j]).c[0], (read_chunk[i][j]).c[1],
+                        (read_chunk[i][j]).a, (read_chunk[i][j]).b, (read_chunk[i][j]).c[0], (read_chunk[i][j]).c[1], 
                         (read_chunk[i][j]).c[2], (read_chunk[i][j]).c[3], (read_chunk[i][j]).d, (read_chunk[i][j]).e);
 
                     goto error;
@@ -1092,10 +1098,10 @@ test_data_conv(hid_t file)
                     HDprintf("    1. Read different values than written.");
                     HDprintf("    At index %d,%d\n", i, j);
                     HDprintf("    src={a=%d, b=%d, c=[%d,%d,%d,%d], d=%d, e=%d\n",
-                        (direct_buf[i][j]).a, (direct_buf[i][j]).b, (direct_buf[i][j]).c[0], (direct_buf[i][j]).c[1],
+                        (direct_buf[i][j]).a, (direct_buf[i][j]).b, (direct_buf[i][j]).c[0], (direct_buf[i][j]).c[1], 
                         (direct_buf[i][j]).c[2], (direct_buf[i][j]).c[3], (direct_buf[i][j]).d, (direct_buf[i][j]).e);
                     HDprintf("    dst={a=%d, c=[%d,%d,%d,%d], e=%d\n",
-                        (check_chunk[i][j]).a, (check_chunk[i][j]).c[0], (check_chunk[i][j]).c[1], (check_chunk[i][j]).c[2],
+                        (check_chunk[i][j]).a, (check_chunk[i][j]).c[0], (check_chunk[i][j]).c[1], (check_chunk[i][j]).c[2], 
                         (check_chunk[i][j]).c[3], (check_chunk[i][j]).e);
 
                     goto error;
@@ -1114,7 +1120,6 @@ test_data_conv(hid_t file)
     H5Tclose(st);
     H5Tclose(dt);
 
-
     PASSED();
     return 0;
 
@@ -1134,14 +1139,14 @@ error:
 } /* test_data_conv() */
 
 /*-------------------------------------------------------------------------
- * Function:    test_invalid_parameters
+ * Function:	test_invalid_parameters
  *
  * Purpose:     Test invalid parameters for H5DOwrite_chunk and H5DOread_chunk
  *
- * Return:      Success:    0
- *              Failure:    1
+ * Return:      Success:	0
+ *              Failure:	1
  *
- * Programmer:  Raymond Lu
+ * Programmer:  Raymond Lu	
  *              30 November 2012
  *
  *-------------------------------------------------------------------------
@@ -1152,7 +1157,7 @@ test_invalid_parameters(hid_t file)
     hid_t       dataspace = -1, dataset = -1;
     hid_t       mem_space = -1;
     hid_t       cparms = -1, dxpl = -1;
-    hsize_t     dims[2]  = {NX, NY};
+    hsize_t     dims[2]  = {NX, NY};        
     hsize_t     chunk_dims[2] ={CHUNK_NX, CHUNK_NY};
     herr_t      status;
     int         i, j, n;
@@ -1186,7 +1191,7 @@ test_invalid_parameters(hid_t file)
      * Create a new contiguous dataset to verify H5DOwrite_chunk/H5DOread_chunk doesn't work
      */
     if((dataset = H5Dcreate2(file, DATASETNAME5, H5T_NATIVE_INT, dataspace, H5P_DEFAULT,
-            cparms, H5P_DEFAULT)) < 0)
+			cparms, H5P_DEFAULT)) < 0)
         goto error;
 
     if((dxpl = H5Pcreate(H5P_DATASET_XFER)) < 0)
@@ -1235,10 +1240,10 @@ test_invalid_parameters(hid_t file)
      * creation properties.
      */
     if((dataset = H5Dcreate2(file, DATASETNAME6, H5T_NATIVE_INT, dataspace, H5P_DEFAULT,
-            cparms, H5P_DEFAULT)) < 0)
+			cparms, H5P_DEFAULT)) < 0)
         goto error;
 
-    /* Check invalid dataset ID for H5DOwrite_chunk and H5DOread_chunk */
+    /* Check invalid dataset ID for H5DOwrite_chunk and H5DOread_chunk */ 
     H5E_BEGIN_TRY {
         if((status = H5DOwrite_chunk((hid_t)-1, dxpl, filter_mask, offset, buf_size, direct_buf)) != FAIL)
             goto error;
@@ -1300,7 +1305,7 @@ test_invalid_parameters(hid_t file)
     /* Check invalid buffer size for H5DOwrite_chunk only */
     offset[0] = CHUNK_NX;
     offset[1] = CHUNK_NY;
-    buf_size = 0;
+    buf_size = 0; 
     H5E_BEGIN_TRY {
         if((status = H5DOwrite_chunk(dataset, dxpl, filter_mask, offset, buf_size, direct_buf)) != FAIL)
             goto error;
@@ -1328,7 +1333,7 @@ test_invalid_parameters(hid_t file)
     H5Sclose(dataspace);
     H5Pclose(cparms);
     H5Pclose(dxpl);
-
+    
     PASSED();
     return 0;
 
@@ -1634,7 +1639,7 @@ test_direct_chunk_read_cache (hid_t file, hbool_t flush)
                 goto error;
             if(read_buf_size == 0)
                 goto error;
-
+        
             /* Read the compressed chunk back using the direct read function. */
             if((status = H5DOread_chunk(dataset, dxpl, offset, &filter_mask, outbuf)) < 0)
                 goto error;
@@ -1972,15 +1977,148 @@ error:
 } /* test_read_unallocated_chunk() */
 
 /*-------------------------------------------------------------------------
- * Function:    Main function
+ * Function:    test_single_chunk_latest
  *
- * Purpose:    Test direct chunk write function H5DOwrite_chunk
+ * Purpose:     This is to verify the fix for jira issue HDFFV-10425.
+ *              The problem was due to a bug in the internal ilbrary routine
+ *              H5D__chunk_direct_write() which passed a null dataset
+ *              pointer to the insert callback for the chunk index type.
+ *              Currently, the single chunk index is the only one that
+ *              used the dataset pointer in the insert callback.
  *
- * Return:    Success:    0
+ *              This routine is based on the test program attached to
+ *              this jira issue:
+ *                  Create a file with the latest format and a chunked dataset
+ *                  with one single chunk.  The library will use single chunk
+ *                  index for the dataset.  
+ *                  Verify that the data read is the same as the written data.
  *
- *        Failure:    1
+ * Return:      Success:        0
+ *              Failure:        1
  *
- * Programmer:  Raymond Lu
+ *-------------------------------------------------------------------------
+ */
+static int
+test_single_chunk_latest(void)
+{
+    hid_t fid;              /* File ID */
+    hid_t fapl;             /* File access property list ID */
+    hid_t sid;              /* Dataspace ID */
+    hid_t did;              /* Dataset ID */
+    hid_t dcpl;             /* Dataset creation property list */
+    hsize_t dims[2] = {DIM0, DIM1};         /* Dimension sizes */
+    hsize_t chunk[2] = {CHUNK0, CHUNK1};    /* Chunk dimension sizes */
+    hsize_t offset[2] = {0,0};              /* Offset for writing */
+    int wdata[DIM0][DIM1];  /* Write buffer */
+    int rdata[DIM0][DIM1];  /* Read buffer */
+    int i, j;               /* Local index variable */
+
+    TESTING("H5DOwrite_chunk with single chunk and latest format");
+
+    /* Initialize data */
+    for (i=0; i<DIM0; i++) {
+      for (j=0; j< DIM1; j++)
+        wdata[i][j] = j/CHUNK0;
+    }
+
+    /* Create a new file with the latest format  */
+    if((fapl = H5Pcreate(H5P_FILE_ACCESS)) < 0)
+        goto error;
+    if(H5Pset_libver_bounds(fapl, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0)
+        goto error;
+    if((fid = H5Fcreate(FILE, H5F_ACC_TRUNC, H5P_DEFAULT, fapl)) < 0)
+        goto error;
+
+    /* Create dataspace */
+    if((sid = H5Screate_simple(2, dims, NULL)) < 0)
+        goto error;
+
+    /* Create the dataset creation property list and set the chunk size */
+    if((dcpl = H5Pcreate(H5P_DATASET_CREATE)) < 0)
+        goto error;
+    if(H5Pset_chunk(dcpl, 2, chunk) < 0)
+        goto error;
+
+    /* Create the dataset */
+    if((did = H5Dcreate2(fid, DATASET, H5T_NATIVE_INT, sid, H5P_DEFAULT, dcpl, H5P_DEFAULT)) < 0)
+        goto error;
+
+    /* Write the data directly to the dataset */
+    if(H5DOwrite_chunk(did, H5P_DEFAULT, 0, offset, CHUNK0*CHUNK1*4, (void *)wdata) < 0)
+        goto error;
+
+    /*
+     * Close and release resources.
+     */
+    if(H5Pclose(dcpl) < 0)
+        goto error;
+    if(H5Dclose(did) < 0)
+        goto error;
+    if(H5Sclose(sid) < 0)
+        goto error;
+    if(H5Pclose(fapl) < 0)
+        goto error;
+    if(H5Fclose(fid) < 0)
+        goto error;
+
+    /* Open the file and dataset with default properties  */
+    if((fid = H5Fopen(FILE, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0)
+        goto error;
+    if((did = H5Dopen2(fid, DATASET, H5P_DEFAULT)) < 0)
+        goto error;
+
+    /* Retrieve dataset creation property list */
+    if((dcpl = H5Dget_create_plist(did)) < 0)
+        goto error;
+
+    /* Read the data */
+    if(H5Dread(did, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, rdata) < 0)
+        goto error;
+
+    /* Verify that the data read was correct.  */
+    for (i = 0; i < DIM0; i++) {
+        for (j = 0; j < DIM1; j++) {
+            if(rdata[i][j] != wdata[i][j])
+                goto error;
+        }
+    }
+
+    /*
+     * Close and release resources
+     */
+    if(H5Pclose(dcpl) < 0)
+        goto error;
+    if(H5Dclose(did) < 0)
+        goto error;
+    if(H5Fclose(fid) < 0)
+        goto error;
+
+    PASSED();
+    return 0;
+
+error:
+    H5E_BEGIN_TRY {
+        H5Dclose(did);
+        H5Sclose(sid);
+        H5Pclose(dcpl);
+        H5Pclose(fapl);
+        H5Fclose(fid);
+    } H5E_END_TRY;
+
+    H5_FAILED();
+    return 1;
+} /* test_single_chunk_latest() */
+
+/*-------------------------------------------------------------------------
+ * Function:	Main function
+ *
+ * Purpose:	    Test direct chunk write function H5DOwrite_chunk and
+ *              chunk direct read function H5DOread_chunk
+ *
+ * Return:	    Success:	0
+ *		        Failure:	1
+ *
+ * Programmer:  Raymond Lu	
  *              30 November 2012
  *
  *-------------------------------------------------------------------------
@@ -2000,13 +2138,12 @@ int main( void )
 #ifdef H5_HAVE_FILTER_DEFLATE
     nerrors += test_direct_chunk_write(file_id);
 #endif /* H5_HAVE_FILTER_DEFLATE */
-    nerrors += test_direct_chunk_overwrite_data(file_id);
+	nerrors += test_direct_chunk_overwrite_data(file_id);
     nerrors += test_skip_compress_write1(file_id);
     nerrors += test_skip_compress_write2(file_id);
     nerrors += test_data_conv(file_id);
     nerrors += test_invalid_parameters(file_id);
 
-
     /* Test direct chunk read */
 #ifdef H5_HAVE_FILTER_DEFLATE
     nerrors += test_direct_chunk_read_no_cache(file_id);
@@ -2016,6 +2153,8 @@ int main( void )
     nerrors += test_read_unfiltered_dset(file_id);
     nerrors += test_read_unallocated_chunk(file_id);
 
+    nerrors += test_single_chunk_latest();
+
     if(H5Fclose(file_id) < 0)
         goto error;
 
