@@ -20,12 +20,8 @@
  *    buffer.  The main system support this feature is Linux.
  */
 
-/* Interface initialization */
-#define H5_INTERFACE_INIT_FUNC  H5FD_direct_init_interface
+#include "H5FDdrvr_module.h" /* This source code file is part of the H5FD driver module */
 
-/* For system function posix_memalign - Commented it out because copper isn't able to compile
- * this file. */
-/* #define _XOPEN_SOURCE 600 */
 
 #include "H5private.h"    /* Generic Functions      */
 #include "H5Eprivate.h"    /* Error handling        */
@@ -95,6 +91,7 @@ typedef struct H5FD_direct_t {
     DWORD fileindexlo;
     DWORD fileindexhi;
 #endif
+
 } H5FD_direct_t;
 
 /*
@@ -121,6 +118,7 @@ typedef struct H5FD_direct_t {
          (HDoff_t)((A)+(Z))<(HDoff_t)(A))
 
 /* Prototypes */
+static herr_t H5FD_direct_term(void);
 static void *H5FD_direct_fapl_get(H5FD_t *file);
 static void *H5FD_direct_fapl_copy(const void *_old_fa);
 static H5FD_t *H5FD_direct_open(const char *name, unsigned flags, hid_t fapl_id,
@@ -130,18 +128,22 @@ static int H5FD_direct_cmp(const H5FD_t *_f1, const H5FD_t *_f2);
 static herr_t H5FD_direct_query(const H5FD_t *_f1, unsigned long *flags);
 static haddr_t H5FD_direct_get_eoa(const H5FD_t *_file, H5FD_mem_t type);
 static herr_t H5FD_direct_set_eoa(H5FD_t *_file, H5FD_mem_t type, haddr_t addr);
-static haddr_t H5FD_direct_get_eof(const H5FD_t *_file);
+static haddr_t H5FD_direct_get_eof(const H5FD_t *_file, H5FD_mem_t type);
 static herr_t  H5FD_direct_get_handle(H5FD_t *_file, hid_t fapl, void** file_handle);
 static herr_t H5FD_direct_read(H5FD_t *_file, H5FD_mem_t type, hid_t fapl_id, haddr_t addr,
            size_t size, void *buf);
 static herr_t H5FD_direct_write(H5FD_t *_file, H5FD_mem_t type, hid_t fapl_id, haddr_t addr,
             size_t size, const void *buf);
 static herr_t H5FD_direct_truncate(H5FD_t *_file, hid_t dxpl_id, hbool_t closing);
+static herr_t H5FD_direct_lock(H5FD_t *_file, hbool_t rw);
+static herr_t H5FD_direct_unlock(H5FD_t *_file);
+
 
 static const H5FD_class_t H5FD_direct_g = {
     "direct",          /*name      */
     MAXADDR,          /*maxaddr    */
     H5F_CLOSE_WEAK,        /* fc_degree    */
+    H5FD_direct_term,                           /*terminate             */
     NULL,          /*sb_size    */
     NULL,          /*sb_encode    */
     NULL,          /*sb_decode    */
@@ -166,10 +168,10 @@ static const H5FD_class_t H5FD_direct_g = {
     H5FD_direct_read,        /*read      */
     H5FD_direct_write,        /*write      */
     NULL,          /*flush      */
-    H5FD_direct_truncate,      /*truncate    */
-    NULL,                                       /*lock                  */
-    NULL,                                       /*unlock                */
-    H5FD_FLMAP_DICHOTOMY                        /*fl_map                */
+    H5FD_direct_truncate,      	/*truncate    */
+    H5FD_direct_lock,          	/*lock                  */
+    H5FD_direct_unlock,        	/*unlock                */
+    H5FD_FLMAP_DICHOTOMY       	/*fl_map                */
 };
 
 /* Declare a free list to manage the H5FD_direct_t struct */
@@ -178,10 +180,9 @@ H5FL_DEFINE_STATIC(H5FD_direct_t);
 
 /*--------------------------------------------------------------------------
 NAME
-   H5FD_direct_init_interface -- Initialize interface-specific information
+   H5FD__init_package -- Initialize interface-specific information
 USAGE
-    herr_t H5FD_direct_init_interface()
-
+    herr_t H5FD__init_package()
 RETURNS
     Non-negative on success/Negative on failure
 DESCRIPTION
@@ -190,12 +191,18 @@ DESCRIPTION
 
 --------------------------------------------------------------------------*/
 static herr_t
-H5FD_direct_init_interface(void)
+H5FD__init_package(void)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOERR
+    herr_t ret_value = SUCCEED;
 
-    FUNC_LEAVE_NOAPI(H5FD_direct_init())
-} /* H5FD_direct_init_interface() */
+    FUNC_ENTER_STATIC
+
+    if(H5FD_direct_init() < 0)
+        HGOTO_ERROR(H5E_VFL, H5E_CANTINIT, FAIL, "unable to initialize direct VFD")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* H5FD__init_package() */
 
 
 /*-------------------------------------------------------------------------
@@ -226,7 +233,7 @@ H5FD_direct_init(void)
         H5FD_DIRECT_g = H5FD_register(&H5FD_direct_g,sizeof(H5FD_class_t),FALSE);
 
     /* Set return value */
-    ret_value=H5FD_DIRECT_g;
+    ret_value = H5FD_DIRECT_g;
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -238,16 +245,14 @@ done:
  *
  * Purpose:  Shut down the VFD
  *
- * Return:  <none>
+ * Returns:     Non-negative on success or negative on failure
  *
  * Programmer:  Raymond Lu
  *              Wednesday, 20 September 2006
  *
- * Modification:
- *
  *---------------------------------------------------------------------------
  */
-void
+static herr_t
 H5FD_direct_term(void)
 {
     FUNC_ENTER_NOAPI_NOINIT_NOERR
@@ -255,7 +260,7 @@ H5FD_direct_term(void)
     /* Reset VFL ID */
     H5FD_DIRECT_g=0;
 
-    FUNC_LEAVE_NOAPI_VOID
+    FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5FD_direct_term() */
 
 
@@ -271,8 +276,6 @@ H5FD_direct_term(void)
  * Programmer:  Raymond Lu
  *    Wednesday, 20 September 2006
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 herr_t
@@ -289,17 +292,17 @@ H5Pset_fapl_direct(hid_t fapl_id, size_t boundary, size_t block_size, size_t cbu
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a file access property list")
 
     if(boundary != 0)
-      fa.mboundary = boundary;
+        fa.mboundary = boundary;
     else
-  fa.mboundary = MBOUNDARY_DEF;
+        fa.mboundary = MBOUNDARY_DEF;
     if(block_size != 0)
-      fa.fbsize = block_size;
+        fa.fbsize = block_size;
     else
-  fa.fbsize = FBSIZE_DEF;
+        fa.fbsize = FBSIZE_DEF;
     if(cbuf_size != 0)
-      fa.cbsize = cbuf_size;
+        fa.cbsize = cbuf_size;
     else
-  fa.cbsize = CBSIZE_DEF;
+        fa.cbsize = CBSIZE_DEF;
 
     /* Set the default to be true for data alignment */
     fa.must_align = TRUE;
@@ -308,7 +311,7 @@ H5Pset_fapl_direct(hid_t fapl_id, size_t boundary, size_t block_size, size_t cbu
     if(fa.cbsize % fa.fbsize != 0)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "copy buffer size must be a multiple of block size")
 
-    ret_value= H5P_set_driver(plist, H5FD_DIRECT, &fa);
+    ret_value = H5P_set_driver(plist, H5FD_DIRECT, &fa);
 
 done:
     FUNC_LEAVE_API(ret_value)
@@ -328,37 +331,35 @@ done:
  * Programmer:  Raymond Lu
  *              Wednesday, October 18, 2006
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 herr_t
 H5Pget_fapl_direct(hid_t fapl_id, size_t *boundary/*out*/, size_t *block_size/*out*/,
     size_t *cbuf_size/*out*/)
 {
-    H5FD_direct_fapl_t  *fa;
     H5P_genplist_t *plist;      /* Property list pointer */
-    herr_t      ret_value=SUCCEED;       /* Return value */
+    const H5FD_direct_fapl_t  *fa;
+    herr_t      ret_value = SUCCEED;       /* Return value */
 
     FUNC_ENTER_API(FAIL)
     H5TRACE4("e", "ixxx", fapl_id, boundary, block_size, cbuf_size);
 
     if(NULL == (plist = H5P_object_verify(fapl_id,H5P_FILE_ACCESS)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a file access list")
-    if (H5FD_DIRECT!=H5P_get_driver(plist))
+    if(H5FD_DIRECT != H5P_peek_driver(plist))
         HGOTO_ERROR(H5E_PLIST, H5E_BADVALUE, FAIL, "incorrect VFL driver")
-    if (NULL==(fa=H5P_get_driver_info(plist)))
+    if(NULL == (fa = H5P_peek_driver_info(plist)))
         HGOTO_ERROR(H5E_PLIST, H5E_BADVALUE, FAIL, "bad VFL driver info")
-    if (boundary)
+    if(boundary)
         *boundary = fa->mboundary;
-    if (block_size)
-  *block_size = fa->fbsize;
+    if(block_size)
+        *block_size = fa->fbsize;
     if (cbuf_size)
-  *cbuf_size = fa->cbsize;
+        *cbuf_size = fa->cbsize;
 
 done:
     FUNC_LEAVE_API(ret_value)
-}
+} /* end H5Pget_fapl_direct() */
 
 
 /*-------------------------------------------------------------------------
@@ -486,7 +487,7 @@ H5FD_direct_open(const char *name, unsigned flags, hid_t fapl_id, haddr_t maxadd
     o_flags |= O_DIRECT;
 
     /* Open the file */
-    if ((fd=HDopen(name, o_flags, 0666))<0)
+    if ((fd = HDopen(name, o_flags, H5_POSIX_CREATE_MODE_RW))<0)
         HSYS_GOTO_ERROR(H5E_FILE, H5E_CANTOPENFILE, NULL, "unable to open file")
 
     if (HDfstat(fd, &sb)<0)
@@ -499,7 +500,7 @@ H5FD_direct_open(const char *name, unsigned flags, hid_t fapl_id, haddr_t maxadd
     /* Get the driver specific information */
     if(NULL == (plist = H5P_object_verify(fapl_id,H5P_FILE_ACCESS)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, NULL, "not a file access property list")
-    if(NULL == (fa = (H5FD_direct_fapl_t *)H5P_get_driver_info(plist)))
+    if(NULL == (fa = (H5FD_direct_fapl_t *)H5P_peek_driver_info(plist)))
         HGOTO_ERROR(H5E_PLIST, H5E_BADVALUE, NULL, "bad VFL driver info")
 
     file->fd = fd;
@@ -523,6 +524,9 @@ H5FD_direct_open(const char *name, unsigned flags, hid_t fapl_id, haddr_t maxadd
      * is to handle correctly the case that the file is in a different file system
      * than the one where the program is running.
      */
+    /* NOTE: Use HDmalloc and HDfree here to ensure compatibility with
+     *       HDposix_memalign.
+     */
     buf1 = (int *)HDmalloc(sizeof(int));
     if(HDposix_memalign(&buf2, file->fa.mboundary, file->fa.fbsize) != 0)
         HGOTO_ERROR(H5E_RESOURCE, H5E_CANTALLOC, NULL, "HDposix_memalign failed")
@@ -690,10 +694,11 @@ H5FD_direct_query(const H5FD_t H5_ATTR_UNUSED * _f, unsigned long *flags /* out
     /* Set the VFL feature flags that this driver supports */
     if(flags) {
         *flags = 0;
-        *flags|=H5FD_FEAT_AGGREGATE_METADATA; /* OK to aggregate metadata allocations */
-        *flags|=H5FD_FEAT_ACCUMULATE_METADATA; /* OK to accumulate metadata for faster writes */
-        *flags|=H5FD_FEAT_DATA_SIEVE;       /* OK to perform data sieving for faster raw data reads & writes */
-        *flags|=H5FD_FEAT_AGGREGATE_SMALLDATA; /* OK to aggregate "small" raw data allocations */
+        *flags |= H5FD_FEAT_AGGREGATE_METADATA;     /* OK to aggregate metadata allocations                             */
+        *flags |= H5FD_FEAT_ACCUMULATE_METADATA;    /* OK to accumulate metadata for faster writes                      */
+        *flags |= H5FD_FEAT_DATA_SIEVE;             /* OK to perform data sieving for faster raw data reads & writes    */
+        *flags |= H5FD_FEAT_AGGREGATE_SMALLDATA;    /* OK to aggregate "small" raw data allocations                     */
+        *flags |= H5FD_FEAT_DEFAULT_VFD_COMPATIBLE; /* VFD creates a file which can be opened with the default VFD      */
     }
 
     FUNC_LEAVE_NOAPI(SUCCEED)
@@ -787,13 +792,13 @@ H5FD_direct_set_eoa(H5FD_t *_file, H5FD_mem_t H5_ATTR_UNUSED type, haddr_t addr)
  *-------------------------------------------------------------------------
  */
 static haddr_t
-H5FD_direct_get_eof(const H5FD_t *_file)
+H5FD_direct_get_eof(const H5FD_t *_file, H5FD_mem_t H5_ATTR_UNUSED type)
 {
     const H5FD_direct_t  *file = (const H5FD_direct_t*)_file;
 
     FUNC_ENTER_NOAPI_NOINIT
 
-    FUNC_LEAVE_NOAPI(MAX(file->eof, file->eoa))
+    FUNC_LEAVE_NOAPI(file->eof)
 }
 
 
@@ -989,6 +994,7 @@ H5FD_direct_read(H5FD_t *_file, H5FD_mem_t H5_ATTR_UNUSED type, hid_t H5_ATTR_UN
       addr = (haddr_t)(((addr + size - 1) / _fbsize + 1) * _fbsize);
 
       if(copy_buf) {
+                /* Free with HDfree since it came from posix_memalign */
                 HDfree(copy_buf);
                 copy_buf = NULL;
             } /* end if */
@@ -1000,6 +1006,7 @@ H5FD_direct_read(H5FD_t *_file, H5FD_mem_t H5_ATTR_UNUSED type, hid_t H5_ATTR_UN
 
 done:
     if(ret_value<0) {
+        /* Free with HDfree since it came from posix_memalign */
         if(copy_buf)
             HDfree(copy_buf);
 
@@ -1220,6 +1227,7 @@ H5FD_direct_write(H5FD_t *_file, H5FD_mem_t H5_ATTR_UNUSED type, hid_t H5_ATTR_U
   buf = (const char*)buf + size;
 
   if(copy_buf) {
+    /* Free with HDfree since it came from posix_memalign */
       HDfree(copy_buf);
       copy_buf = NULL;
         } /* end if */
@@ -1233,6 +1241,7 @@ H5FD_direct_write(H5FD_t *_file, H5FD_mem_t H5_ATTR_UNUSED type, hid_t H5_ATTR_U
 
 done:
     if(ret_value<0) {
+        /* Free with HDfree since it came from posix_memalign */
         if(copy_buf)
             HDfree(copy_buf);
 
@@ -1308,5 +1317,72 @@ H5FD_direct_truncate(H5FD_t *_file, hid_t H5_ATTR_UNUSED dxpl_id, hbool_t H5_ATT
 done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5FD_direct_truncate() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    H5FD_direct_lock
+ *
+ * Purpose:     To place an advisory lock on a file.
+ *		The lock type to apply depends on the parameter "rw":
+ *			TRUE--opens for write: an exclusive lock
+ *			FALSE--opens for read: a shared lock
+ *
+ * Return:      SUCCEED/FAIL
+ *
+ * Programmer:  Vailin Choi; May 2013
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5FD_direct_lock(H5FD_t *_file, hbool_t rw)
+{
+    H5FD_direct_t  *file = (H5FD_direct_t*)_file;	/* VFD file struct */
+    int lock;						/* The type of lock */
+    herr_t ret_value = SUCCEED;                 	/* Return value */
+
+    FUNC_ENTER_NOAPI_NOINIT
+
+    HDassert(file);
+
+    /* Determine the type of lock */
+    int lock = rw ? LOCK_EX : LOCK_SH;
+    
+    /* Place the lock with non-blocking */
+    if(HDflock(file->fd, lock | LOCK_NB) < 0)
+        HSYS_GOTO_ERROR(H5E_FILE, H5E_BADFILE, FAIL, "unable to flock file")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5FD_direct_lock() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    H5FD_direct_unlock
+ *
+ * Purpose:     To remove the existing lock on the file
+ *
+ * Return:      SUCCEED/FAIL
+ *
+ * Programmer:  Vailin Choi; May 2013
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5FD_direct_unlock(H5FD_t *_file)
+{
+    H5FD_direct_t  *file = (H5FD_direct_t*)_file;	/* VFD file struct */
+    herr_t ret_value = SUCCEED;                 	/* Return value */
+
+    FUNC_ENTER_NOAPI_NOINIT
+
+    HDassert(file);
+
+    if(HDflock(file->fd, LOCK_UN) < 0)
+        HSYS_GOTO_ERROR(H5E_FILE, H5E_BADFILE, FAIL, "unable to flock (unlock) file")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5FD_direct_unlock() */
+
 #endif /* H5_HAVE_DIRECT */
 
