@@ -11,29 +11,30 @@
  * help@hdfgroup.org.                                                        *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-#define H5O_PACKAGE		/*suppress error about including H5Opkg	  */
-#define H5T_PACKAGE		/*prevent warning from including H5Tpkg   */
+#include "H5Omodule.h"          /* This source code file is part of the H5O module */
+#define H5T_FRIEND		/*prevent warning from including H5Tpkg   */
 
-#include "H5private.h"		/* Generic Functions			*/
-#include "H5Dprivate.h"		/* Datasets				*/
-#include "H5Eprivate.h"		/* Error handling		  	*/
-#include "H5Fprivate.h"		/* Files				*/
-#include "H5FLprivate.h"	/* Free Lists				*/
-#include "H5Gprivate.h"		/* Groups				*/
-#include "H5MMprivate.h"	/* Memory management			*/
-#include "H5Opkg.h"             /* Object headers			*/
-#include "H5Tpkg.h"		/* Datatypes				*/
-#include "H5VMprivate.h"		/* Vectors and arrays 			*/
+
+#include "H5private.h"		/* Generic Functions    */
+#include "H5Dprivate.h"		/* Datasets	            */
+#include "H5Eprivate.h"		/* Error handling       */
+#include "H5Fprivate.h"		/* Files                */
+#include "H5FLprivate.h"	/* Free Lists           */
+#include "H5Gprivate.h"		/* Groups               */
+#include "H5MMprivate.h"	/* Memory management    */
+#include "H5Opkg.h"         /* Object headers       */
+#include "H5Tpkg.h"         /* Datatypes            */
+#include "H5VMprivate.h"    /* Vectors and arrays   */
 
 
 /* PRIVATE PROTOTYPES */
 static herr_t H5O_dtype_encode(H5F_t *f, uint8_t *p, const void *mesg);
 static void *H5O_dtype_decode(H5F_t *f, hid_t dxpl_id, H5O_t *open_oh,
-    unsigned mesg_flags, unsigned *ioflags, const uint8_t *p);
+    unsigned mesg_flags, unsigned *ioflags, size_t p_size, const uint8_t *p);
 static void *H5O_dtype_copy(const void *_mesg, void *_dest);
 static size_t H5O_dtype_size(const H5F_t *f, const void *_mesg);
 static herr_t H5O_dtype_reset(void *_mesg);
-static herr_t H5O_dtype_free(void *_mesg);
+static herr_t H5O__dtype_free(void *_mesg);
 static herr_t H5O_dtype_set_share(void *_mesg, const H5O_shared_t *sh);
 static htri_t H5O_dtype_can_share(const void *_mesg);
 static herr_t H5O_dtype_pre_copy_file(H5F_t *file_src, const void *mesg_src,
@@ -99,7 +100,7 @@ const H5O_msg_class_t H5O_MSG_DTYPE[1] = {{
     H5O_dtype_copy,		/* copy the native value	*/
     H5O_dtype_shared_size,	/* size of raw message		*/
     H5O_dtype_reset,		/* reset method			*/
-    H5O_dtype_free,		/* free method			*/
+    H5O__dtype_free,		/* free method			*/
     H5O_dtype_shared_delete,	/* file delete method		*/
     H5O_dtype_shared_link,	/* link method			*/
     H5O_dtype_set_share,	/* set share method		*/
@@ -260,7 +261,7 @@ H5O_dtype_decode_helper(H5F_t *f, unsigned *ioflags/*in,out*/, const uint8_t **p
                 unsigned offset_nbytes;         /* Size needed to encode member offsets */
                 size_t max_memb_pos = 0;        /* Maximum member covered, so far */
                 unsigned max_version = 0;       /* Maximum member version */
-                hbool_t upgrade_to = 0;         /* Version number we can "soft" upgrade to */
+                unsigned upgrade_to = 0;         /* Version number we can "soft" upgrade to */
                 unsigned j;
 
                 /* Compute the # of bytes required to store a member offset */
@@ -1093,10 +1094,10 @@ done:
 --------------------------------------------------------------------------*/
 static void *
 H5O_dtype_decode(H5F_t *f, hid_t H5_ATTR_UNUSED dxpl_id, H5O_t H5_ATTR_UNUSED *open_oh, unsigned H5_ATTR_UNUSED mesg_flags,
-    unsigned *ioflags/*in,out*/, const uint8_t *p)
+    unsigned *ioflags/*in,out*/, size_t H5_ATTR_UNUSED p_size, const uint8_t *p)
 {
     H5T_t	*dt = NULL;
-    void        *ret_value;     /* Return value */
+    void        *ret_value = NULL;     /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT
 
@@ -1178,9 +1179,9 @@ done:
 static void *
 H5O_dtype_copy(const void *_src, void *_dst)
 {
-    const H5T_t		   *src = (const H5T_t *) _src;
-    H5T_t		   *dst;
-    void 		   *ret_value;  /* Return value */
+    const H5T_t	*src = (const H5T_t *) _src;
+    H5T_t	*dst;
+    void        *ret_value = NULL;      /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT
 
@@ -1227,7 +1228,7 @@ H5O_dtype_size(const H5F_t *f, const void *_mesg)
 {
     const H5T_t	*dt = (const H5T_t *)_mesg;
     unsigned	u;                      /* Local index variable */
-    size_t	ret_value;
+    size_t	ret_value = 0;          /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT_NOERR
 
@@ -1372,31 +1373,34 @@ H5O_dtype_reset(void *_mesg)
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5O_dtype_free
+ * Function:	H5O__dtype_free
  *
- * Purpose:	Free's the message
+ * Purpose:	Frees the message
  *
  * Return:	Non-negative on success/Negative on failure
  *
  * Programmer:	Quincey Koziol
  *              Thursday, March 30, 2000
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5O_dtype_free(void *mesg)
+H5O__dtype_free(void *mesg)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOERR
+    herr_t ret_value = SUCCEED;         /* Return value */
 
+    FUNC_ENTER_STATIC
+
+    /* Sanity check */
     HDassert(mesg);
 
-    ((H5T_t *) mesg)->shared = H5FL_FREE(H5T_shared_t, ((H5T_t *) mesg)->shared);
-    mesg = H5FL_FREE(H5T_t, mesg);
+    /* Release the datatype */
+    if(H5T_close((H5T_t *)mesg) < 0)
+        HGOTO_ERROR(H5E_OHDR, H5E_CANTFREE, FAIL, "unable to free datatype")
 
-    FUNC_LEAVE_NOAPI(SUCCEED)
-} /* end H5O_dtype_free() */
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5O__dtype_free() */
 
 
 /*-------------------------------------------------------------------------
@@ -1440,9 +1444,10 @@ H5O_dtype_set_share(void *_mesg/*in,out*/, const H5O_shared_t *sh)
         dt->shared->state = H5T_STATE_NAMED;
 
         /* Set up the object location for the datatype also */
+        if(H5O_loc_reset(&(dt->oloc)) < 0)
+            HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to reset location")
         dt->oloc.file = sh->file;
-        dt->oloc.addr = sh->u.loc.oh_addr;;
-        dt->oloc.holding_file = FALSE;
+        dt->oloc.addr = sh->u.loc.oh_addr;
     } /* end if */
 
 done:
@@ -1511,7 +1516,7 @@ done:
  */
 static herr_t
 H5O_dtype_pre_copy_file(H5F_t *file_src, const void *mesg_src,
-    hbool_t H5_ATTR_UNUSED *deleted, const H5O_copy_t H5_ATTR_UNUSED *cpy_info,
+    hbool_t H5_ATTR_UNUSED *deleted, const H5O_copy_t *cpy_info,
     void *_udata)
 {
     const H5T_t	*dt_src = (const H5T_t *)mesg_src;  /* Source datatype */
@@ -1523,6 +1528,13 @@ H5O_dtype_pre_copy_file(H5F_t *file_src, const void *mesg_src,
     /* check args */
     HDassert(file_src);
     HDassert(dt_src);
+    HDassert(cpy_info);
+    HDassert(cpy_info->file_dst);
+
+    /* Check to ensure that the version of the message to be copied does not exceed
+       the message version as indicated by the destination file's high bound */
+    if(dt_src->shared->version > H5O_dtype_ver_bounds[H5F_HIGH_BOUND(cpy_info->file_dst)])
+        HGOTO_ERROR(H5E_OHDR, H5E_BADRANGE, FAIL, "datatype message version out of bounds")
 
     /* If the user data is non-NULL, assume we are copying a dataset
      * and check if we need to make a copy of the datatype for later in
@@ -1565,7 +1577,7 @@ H5O_dtype_copy_file(H5F_t H5_ATTR_UNUSED *file_src, const H5O_msg_class_t *mesg_
     H5O_copy_t H5_ATTR_UNUSED *cpy_info, void H5_ATTR_UNUSED *udata, hid_t H5_ATTR_UNUSED dxpl_id)
 {
     H5T_t *dst_mesg;            /* Destination datatype */
-    void *ret_value;            /* Return value */
+    void *ret_value = NULL;     /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT
 
@@ -1605,18 +1617,22 @@ H5O_dtype_shared_post_copy_upd(const H5O_loc_t H5_ATTR_UNUSED *src_oloc,
     hid_t H5_ATTR_UNUSED dxpl_id, H5O_copy_t H5_ATTR_UNUSED *cpy_info)
 {
     H5T_t       *dt_dst = (H5T_t *)mesg_dst;    /* Destination datatype */
+    herr_t ret_value = SUCCEED;     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOERR
+    FUNC_ENTER_NOAPI_NOINIT
 
     if(dt_dst->sh_loc.type == H5O_SHARE_TYPE_COMMITTED) {
         HDassert(H5T_committed(dt_dst));
+        if(H5O_loc_reset(&(dt_dst->oloc)) < 0)
+            HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to reset location")
         dt_dst->oloc.file = dt_dst->sh_loc.file;
         dt_dst->oloc.addr = dt_dst->sh_loc.u.loc.oh_addr;
     } /* end if */
     else
         HDassert(!H5T_committed(dt_dst));
 
-    FUNC_LEAVE_NOAPI(SUCCEED)
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5O_dtype_shared_post_copy_upd */
 
 
@@ -1706,62 +1722,64 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
         case H5T_NO_CLASS:
         case H5T_NCLASSES:
         default:
-            sprintf(buf, "H5T_CLASS_%d", (int)(dt->shared->type));
+            HDsprintf(buf, "H5T_CLASS_%d", (int)(dt->shared->type));
             s = buf;
             break;
     } /* end switch */
-    fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+    HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
 	    "Type class:",
 	    s);
 
-    fprintf(stream, "%*s%-*s %lu byte%s\n", indent, "", fwidth,
+    HDfprintf(stream, "%*s%-*s %lu byte%s\n", indent, "", fwidth,
 	    "Size:",
 	    (unsigned long)(dt->shared->size), 1 == dt->shared->size ? "" : "s");
 
-    fprintf(stream, "%*s%-*s %u\n", indent, "", fwidth,
+    HDfprintf(stream, "%*s%-*s %u\n", indent, "", fwidth,
 		"Version:", dt->shared->version);
 
     if (H5T_COMPOUND == dt->shared->type) {
-	fprintf(stream, "%*s%-*s %u\n", indent, "", fwidth,
-		"Number of members:",
-		dt->shared->u.compnd.nmembs);
-	for(i = 0; i < dt->shared->u.compnd.nmembs; i++) {
-	    sprintf(buf, "Member %u:", i);
-	    fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
-		    buf,
-		    dt->shared->u.compnd.memb[i].name);
-	    fprintf(stream, "%*s%-*s %lu\n", indent+3, "", MAX(0, fwidth-3),
-		    "Byte offset:",
-		    (unsigned long)(dt->shared->u.compnd.memb[i].offset));
-	    H5O_dtype_debug(f, dxpl_id, dt->shared->u.compnd.memb[i].type, stream,
-			    indent + 3, MAX(0, fwidth - 3));
-	}
-    } else if(H5T_ENUM == dt->shared->type) {
-	fprintf(stream, "%*s%s\n", indent, "", "Base type:");
-	H5O_dtype_debug(f, dxpl_id, dt->shared->parent, stream, indent+3, MAX(0, fwidth-3));
-	fprintf(stream, "%*s%-*s %u\n", indent, "", fwidth,
-		"Number of members:",
-		dt->shared->u.enumer.nmembs);
-	for(i = 0; i < dt->shared->u.enumer.nmembs; i++) {
-	    sprintf(buf, "Member %u:", i);
-	    fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
-		    buf,
-		    dt->shared->u.enumer.name[i]);
-	    fprintf(stream, "%*s%-*s 0x", indent, "", fwidth,
-		    "Raw bytes of value:");
-	    for(k = 0; k < dt->shared->parent->shared->size; k++)
-		fprintf(stream, "%02x",
-			dt->shared->u.enumer.value[i*dt->shared->parent->shared->size + k]);
-	    fprintf(stream, "\n");
-	} /* end for */
-
-    } else if(H5T_OPAQUE == dt->shared->type) {
-	fprintf(stream, "%*s%-*s \"%s\"\n", indent, "", fwidth,
-		"Tag:", dt->shared->u.opaque.tag);
-    } else if(H5T_REFERENCE == dt->shared->type) {
-	fprintf(stream, "%*s%-*s\n", indent, "", fwidth,
-		"Fix dumping reference types!");
-    } else if(H5T_STRING == dt->shared->type) {
+        HDfprintf(stream, "%*s%-*s %u\n", indent, "", fwidth,
+		    "Number of members:",
+            dt->shared->u.compnd.nmembs);
+        for(i = 0; i < dt->shared->u.compnd.nmembs; i++) {
+            HDsprintf(buf, "Member %u:", i);
+            HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+                buf,
+                dt->shared->u.compnd.memb[i].name);
+            HDfprintf(stream, "%*s%-*s %lu\n", indent+3, "", MAX(0, fwidth-3),
+                "Byte offset:",
+                (unsigned long)(dt->shared->u.compnd.memb[i].offset));
+            H5O_dtype_debug(f, dxpl_id, dt->shared->u.compnd.memb[i].type, stream,
+                    indent + 3, MAX(0, fwidth - 3));
+        } /* end for */
+    } /* end if */
+    else if(H5T_ENUM == dt->shared->type) {
+        HDfprintf(stream, "%*s%s\n", indent, "", "Base type:");
+        H5O_dtype_debug(f, dxpl_id, dt->shared->parent, stream, indent+3, MAX(0, fwidth-3));
+        HDfprintf(stream, "%*s%-*s %u\n", indent, "", fwidth,
+            "Number of members:",
+            dt->shared->u.enumer.nmembs);
+        for(i = 0; i < dt->shared->u.enumer.nmembs; i++) {
+            HDsprintf(buf, "Member %u:", i);
+            HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+                buf,
+                dt->shared->u.enumer.name[i]);
+            HDfprintf(stream, "%*s%-*s 0x", indent, "", fwidth,
+                "Raw bytes of value:");
+            for(k = 0; k < dt->shared->parent->shared->size; k++)
+                HDfprintf(stream, "%02x", dt->shared->u.enumer.value[i*dt->shared->parent->shared->size + k]);
+            HDfprintf(stream, "\n");
+        } /* end for */
+    } /* end else if */
+    else if(H5T_OPAQUE == dt->shared->type) {
+        HDfprintf(stream, "%*s%-*s \"%s\"\n", indent, "", fwidth,
+            "Tag:", dt->shared->u.opaque.tag);
+    } /* end else if */
+    else if(H5T_REFERENCE == dt->shared->type) {
+        HDfprintf(stream, "%*s%-*s\n", indent, "", fwidth,
+            "Fix dumping reference types!");
+    } /* end else if */
+    else if(H5T_STRING == dt->shared->type) {
         switch(dt->shared->u.atomic.u.s.cset) {
             case H5T_CSET_ASCII:
                 s = "ASCII";
@@ -1785,17 +1803,17 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             case H5T_CSET_RESERVED_13:
             case H5T_CSET_RESERVED_14:
             case H5T_CSET_RESERVED_15:
-                sprintf(buf, "H5T_CSET_RESERVED_%d", (int)(dt->shared->u.atomic.u.s.cset));
+                HDsprintf(buf, "H5T_CSET_RESERVED_%d", (int)(dt->shared->u.atomic.u.s.cset));
                 s = buf;
                 break;
 
             case H5T_CSET_ERROR:
             default:
-                sprintf(buf, "Unknown character set: %d", (int)(dt->shared->u.atomic.u.s.cset));
+                HDsprintf(buf, "Unknown character set: %d", (int)(dt->shared->u.atomic.u.s.cset));
                 s = buf;
                 break;
         } /* end switch */
-        fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+        HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
                 "Character Set:",
                 s);
 
@@ -1825,20 +1843,21 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             case H5T_STR_RESERVED_13:
             case H5T_STR_RESERVED_14:
             case H5T_STR_RESERVED_15:
-                sprintf(buf, "H5T_STR_RESERVED_%d", (int)(dt->shared->u.atomic.u.s.pad));
+                HDsprintf(buf, "H5T_STR_RESERVED_%d", (int)(dt->shared->u.atomic.u.s.pad));
                 s = buf;
                 break;
 
             case H5T_STR_ERROR:
             default:
-                sprintf(buf, "Unknown string padding: %d", (int)(dt->shared->u.atomic.u.s.pad));
+                HDsprintf(buf, "Unknown string padding: %d", (int)(dt->shared->u.atomic.u.s.pad));
                 s = buf;
                 break;
         } /* end switch */
-        fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+        HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
                 "String Padding:",
                 s);
-    } else if(H5T_VLEN == dt->shared->type) {
+    } /* end else if */
+    else if(H5T_VLEN == dt->shared->type) {
         switch(dt->shared->u.vlen.type) {
             case H5T_VLEN_SEQUENCE:
                 s = "sequence";
@@ -1851,11 +1870,11 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             case H5T_VLEN_BADTYPE:
             case H5T_VLEN_MAXTYPE:
             default:
-                sprintf(buf, "H5T_VLEN_%d", dt->shared->u.vlen.type);
+                HDsprintf(buf, "H5T_VLEN_%d", dt->shared->u.vlen.type);
                 s = buf;
                 break;
         } /* end switch */
-        fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+        HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
                 "Vlen type:", s);
 
         switch(dt->shared->u.vlen.loc) {
@@ -1870,11 +1889,11 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             case H5T_LOC_BADLOC:
             case H5T_LOC_MAXLOC:
             default:
-                sprintf(buf, "H5T_LOC_%d", (int)dt->shared->u.vlen.loc);
+                HDsprintf(buf, "H5T_LOC_%d", (int)dt->shared->u.vlen.loc);
                 s = buf;
                 break;
         } /* end switch */
-        fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+        HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
                 "Location:", s);
 
         /* Extra information for VL-strings */
@@ -1902,17 +1921,17 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
                 case H5T_CSET_RESERVED_13:
                 case H5T_CSET_RESERVED_14:
                 case H5T_CSET_RESERVED_15:
-                    sprintf(buf, "H5T_CSET_RESERVED_%d", (int)(dt->shared->u.vlen.cset));
+                    HDsprintf(buf, "H5T_CSET_RESERVED_%d", (int)(dt->shared->u.vlen.cset));
                     s = buf;
                     break;
 
                 case H5T_CSET_ERROR:
                 default:
-                    sprintf(buf, "Unknown character set: %d", (int)(dt->shared->u.vlen.cset));
+                    HDsprintf(buf, "Unknown character set: %d", (int)(dt->shared->u.vlen.cset));
                     s = buf;
                     break;
             } /* end switch */
-            fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+            HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
                     "Character Set:",
                     s);
 
@@ -1942,32 +1961,34 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
                 case H5T_STR_RESERVED_13:
                 case H5T_STR_RESERVED_14:
                 case H5T_STR_RESERVED_15:
-                    sprintf(buf, "H5T_STR_RESERVED_%d", (int)(dt->shared->u.vlen.pad));
+                    HDsprintf(buf, "H5T_STR_RESERVED_%d", (int)(dt->shared->u.vlen.pad));
                     s = buf;
                     break;
 
                 case H5T_STR_ERROR:
                 default:
-                    sprintf(buf, "Unknown string padding: %d", (int)(dt->shared->u.vlen.pad));
+                    HDsprintf(buf, "Unknown string padding: %d", (int)(dt->shared->u.vlen.pad));
                     s = buf;
                     break;
             } /* end switch */
-            fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+            HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
                     "String Padding:",
                     s);
         } /* end if */
-    } else if(H5T_ARRAY == dt->shared->type) {
-	fprintf(stream, "%*s%-*s %u\n", indent, "", fwidth,
-		"Rank:",
-		dt->shared->u.array.ndims);
-        fprintf(stream, "%*s%-*s {", indent, "", fwidth, "Dim Size:");
+    } /* end else if */
+    else if(H5T_ARRAY == dt->shared->type) {
+        HDfprintf(stream, "%*s%-*s %u\n", indent, "", fwidth,
+            "Rank:",
+            dt->shared->u.array.ndims);
+        HDfprintf(stream, "%*s%-*s {", indent, "", fwidth, "Dim Size:");
         for(i = 0; i < dt->shared->u.array.ndims; i++)
-            fprintf(stream, "%s%u", (i ? ", " : ""), (unsigned)dt->shared->u.array.dim[i]);
-        fprintf(stream, "}\n");
-	fprintf(stream, "%*s%s\n", indent, "", "Base type:");
-	H5O_dtype_debug(f, dxpl_id, dt->shared->parent, stream, indent + 3, MAX(0, fwidth - 3));
-    } else {
-	switch (dt->shared->u.atomic.order) {
+            HDfprintf(stream, "%s%u", (i ? ", " : ""), (unsigned)dt->shared->u.array.dim[i]);
+        HDfprintf(stream, "}\n");
+        HDfprintf(stream, "%*s%s\n", indent, "", "Base type:");
+        H5O_dtype_debug(f, dxpl_id, dt->shared->parent, stream, indent + 3, MAX(0, fwidth - 3));
+    } /* end else if */
+    else {
+        switch (dt->shared->u.atomic.order) {
             case H5T_ORDER_LE:
                 s = "little endian";
                 break;
@@ -1990,25 +2011,25 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
 
             case H5T_ORDER_ERROR:
             default:
-                sprintf(buf, "H5T_ORDER_%d", dt->shared->u.atomic.order);
+                HDsprintf(buf, "H5T_ORDER_%d", dt->shared->u.atomic.order);
                 s = buf;
                 break;
-	} /* end switch */
-	fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
-		"Byte order:",
-		s);
+        } /* end switch */
+        HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+            "Byte order:",
+            s);
 
-	fprintf(stream, "%*s%-*s %lu bit%s\n", indent, "", fwidth,
-		"Precision:",
-		(unsigned long)(dt->shared->u.atomic.prec),
-		1==dt->shared->u.atomic.prec?"":"s");
+        HDfprintf(stream, "%*s%-*s %lu bit%s\n", indent, "", fwidth,
+            "Precision:",
+            (unsigned long)(dt->shared->u.atomic.prec),
+            1==dt->shared->u.atomic.prec?"":"s");
 
-	fprintf(stream, "%*s%-*s %lu bit%s\n", indent, "", fwidth,
-		"Offset:",
-		(unsigned long)(dt->shared->u.atomic.offset),
-		1==dt->shared->u.atomic.offset?"":"s");
+        HDfprintf(stream, "%*s%-*s %lu bit%s\n", indent, "", fwidth,
+            "Offset:",
+            (unsigned long)(dt->shared->u.atomic.offset),
+            1==dt->shared->u.atomic.offset?"":"s");
 
-	switch (dt->shared->u.atomic.lsb_pad) {
+        switch (dt->shared->u.atomic.lsb_pad) {
             case H5T_PAD_ZERO:
                 s = "zero";
                 break;
@@ -2026,11 +2047,11 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             default:
                 s = "pad?";
                 break;
-	} /* end switch */
-	fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
-		"Low pad type:", s);
+        } /* end switch */
+        HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+            "Low pad type:", s);
 
-	switch (dt->shared->u.atomic.msb_pad) {
+        switch (dt->shared->u.atomic.msb_pad) {
             case H5T_PAD_ZERO:
                 s = "zero";
                 break;
@@ -2048,12 +2069,12 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             default:
                 s = "pad?";
                 break;
-	} /* end switch */
-	fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
-		"High pad type:", s);
+        } /* end switch */
+        HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+            "High pad type:", s);
 
-	if (H5T_FLOAT == dt->shared->type) {
-	    switch (dt->shared->u.atomic.u.f.pad) {
+        if (H5T_FLOAT == dt->shared->type) {
+            switch (dt->shared->u.atomic.u.f.pad) {
                 case H5T_PAD_ZERO:
                     s = "zero";
                     break;
@@ -2070,16 +2091,16 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
                 case H5T_NPAD:
                 default:
                     if (dt->shared->u.atomic.u.f.pad < 0)
-                        sprintf(buf, "H5T_PAD_%d", -(dt->shared->u.atomic.u.f.pad));
+                        HDsprintf(buf, "H5T_PAD_%d", -(dt->shared->u.atomic.u.f.pad));
                     else
-                        sprintf(buf, "bit-%d", dt->shared->u.atomic.u.f.pad);
+                        HDsprintf(buf, "bit-%d", dt->shared->u.atomic.u.f.pad);
                     s = buf;
                     break;
-	    } /* end switch */
-	    fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
-		    "Internal pad type:", s);
+            } /* end switch */
+            HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+                "Internal pad type:", s);
 
-	    switch (dt->shared->u.atomic.u.f.norm) {
+            switch (dt->shared->u.atomic.u.f.norm) {
                 case H5T_NORM_IMPLIED:
                     s = "implied";
                     break;
@@ -2094,38 +2115,39 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
 
                 case H5T_NORM_ERROR:
                 default:
-                    sprintf(buf, "H5T_NORM_%d", (int) (dt->shared->u.atomic.u.f.norm));
+                    HDsprintf(buf, "H5T_NORM_%d", (int) (dt->shared->u.atomic.u.f.norm));
                     s = buf;
-	    } /* end switch */
-	    fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
-		    "Normalization:", s);
+            } /* end switch */
+            HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+                "Normalization:", s);
 
-	    fprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
-		    "Sign bit location:",
-		    (unsigned long) (dt->shared->u.atomic.u.f.sign));
+            HDfprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
+                "Sign bit location:",
+                (unsigned long) (dt->shared->u.atomic.u.f.sign));
 
-	    fprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
-		    "Exponent location:",
-		    (unsigned long) (dt->shared->u.atomic.u.f.epos));
+            HDfprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
+                "Exponent location:",
+                (unsigned long) (dt->shared->u.atomic.u.f.epos));
 
-	    fprintf(stream, "%*s%-*s 0x%08lx\n", indent, "", fwidth,
-		    "Exponent bias:",
-		    (unsigned long) (dt->shared->u.atomic.u.f.ebias));
+            HDfprintf(stream, "%*s%-*s 0x%08lx\n", indent, "", fwidth,
+                "Exponent bias:",
+                (unsigned long) (dt->shared->u.atomic.u.f.ebias));
 
-	    fprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
-		    "Exponent size:",
-		    (unsigned long) (dt->shared->u.atomic.u.f.esize));
+            HDfprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
+                "Exponent size:",
+                (unsigned long) (dt->shared->u.atomic.u.f.esize));
 
-	    fprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
-		    "Mantissa location:",
-		    (unsigned long) (dt->shared->u.atomic.u.f.mpos));
+            HDfprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
+                "Mantissa location:",
+                (unsigned long) (dt->shared->u.atomic.u.f.mpos));
 
-	    fprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
-		    "Mantissa size:",
-		    (unsigned long) (dt->shared->u.atomic.u.f.msize));
+            HDfprintf(stream, "%*s%-*s %lu\n", indent, "", fwidth,
+                "Mantissa size:",
+                (unsigned long) (dt->shared->u.atomic.u.f.msize));
 
-	} else if (H5T_INTEGER == dt->shared->type) {
-	    switch (dt->shared->u.atomic.u.i.sign) {
+        } /* end if */
+        else if (H5T_INTEGER == dt->shared->type) {
+            switch (dt->shared->u.atomic.u.i.sign) {
                 case H5T_SGN_NONE:
                     s = "none";
                     break;
@@ -2137,14 +2159,14 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
                 case H5T_SGN_ERROR:
                 case H5T_NSGN:
                 default:
-                    sprintf(buf, "H5T_SGN_%d", (int) (dt->shared->u.atomic.u.i.sign));
+                    HDsprintf(buf, "H5T_SGN_%d", (int) (dt->shared->u.atomic.u.i.sign));
                     s = buf;
                     break;
-	    } /* end switch */
-	    fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
-		    "Sign scheme:", s);
-	}
-    }
+            } /* end switch */
+            HDfprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
+                "Sign scheme:", s);
+        } /* end else if */
+    } /* end else */
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5O_dtype_debug() */
