@@ -29,10 +29,7 @@
 /* Module Setup */
 /****************/
 
-#define H5P_PACKAGE		/*suppress error about including H5Ppkg   */
-
-/* Interface initialization */
-#define H5_INTERFACE_INIT_FUNC	H5P__init_deprec_interface
+#include "H5Pmodule.h"          /* This source code file is part of the H5P module */
 
 
 /***********/
@@ -80,51 +77,6 @@
 /*******************/
 
 
-
-/*--------------------------------------------------------------------------
-NAME
-   H5P__init_deprec_interface -- Initialize interface-specific information
-USAGE
-    herr_t H5P__init_deprec_interface()
-RETURNS
-    Non-negative on success/Negative on failure
-DESCRIPTION
-    Initializes any interface-specific data or routines.  (Just calls
-    H5P_init() currently).
-
---------------------------------------------------------------------------*/
-static herr_t
-H5P__init_deprec_interface(void)
-{
-    FUNC_ENTER_STATIC_NOERR
-
-    FUNC_LEAVE_NOAPI(H5P_init())
-} /* H5P__init_deprec_interface() */
-
-
-/*--------------------------------------------------------------------------
-NAME
-   H5P__term_deprec_interface -- Terminate interface
-USAGE
-    herr_t H5P__term_deprec_interface()
-RETURNS
-    Non-negative on success/Negative on failure
-DESCRIPTION
-    Terminates interface.  (Just resets H5_interface_initialize_g
-    currently).
-
---------------------------------------------------------------------------*/
-herr_t
-H5P__term_deprec_interface(void)
-{
-    FUNC_ENTER_PACKAGE_NOERR
-
-    /* Mark closed */
-    H5_interface_initialize_g = 0;
-
-    FUNC_LEAVE_NOAPI(0)
-} /* H5P__term_deprec_interface() */
-
 #ifndef H5_NO_DEPRECATED_SYMBOLS
 
 /*--------------------------------------------------------------------------
@@ -290,7 +242,7 @@ H5Pregister1(hid_t cls_id, const char *name, size_t size, void *def_value,
 
     /* Create the new property list class */
     orig_pclass = pclass;
-    if((ret_value = H5P_register(&pclass, name, size, def_value, prp_create, prp_set, prp_get, prp_delete, prp_copy, NULL, prp_close)) < 0)
+    if((ret_value = H5P_register(&pclass, name, size, def_value, prp_create, prp_set, prp_get, NULL, NULL, prp_delete, prp_copy, NULL, prp_close)) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to register property in class");
 
     /* Check if the property class changed and needs to be substituted in the ID */
@@ -473,11 +425,196 @@ H5Pinsert1(hid_t plist_id, const char *name, size_t size, void *value,
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "properties >0 size must have default")
 
     /* Create the new property list class */
-    if((ret_value = H5P_insert(plist, name, size, value, prp_set, prp_get, prp_delete, prp_copy, NULL, prp_close)) < 0)
+    if((ret_value = H5P_insert(plist, name, size, value, prp_set, prp_get,
+            NULL, NULL, prp_delete, prp_copy, NULL, prp_close)) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to register property in plist")
 
 done:
     FUNC_LEAVE_API(ret_value)
 }   /* H5Pinsert1() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5Pget_version
+ *
+ * Purpose:	Retrieves version information for various parts of a file.
+ *
+ *		SUPER:		The file super block.
+ *		FREELIST:	The global free list.
+ *		STAB:		The root symbol table entry.
+ *		SHHDR:		Shared object headers.
+ *
+ *		Any (or even all) of the output arguments can be null
+ *		pointers.
+ *
+ * Return:	Success:	Non-negative, version information is returned
+ *				through the arguments.
+ *
+ *		Failure:	Negative
+ *
+ * Programmer:	Robb Matzke
+ *		Wednesday, January  7, 1998
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5Pget_version(hid_t plist_id, unsigned *super/*out*/, unsigned *freelist/*out*/,
+    unsigned *stab/*out*/, unsigned *shhdr/*out*/)
+{
+    H5P_genplist_t *plist;     	/* Property list pointer */
+    herr_t ret_value = SUCCEED;	/* Return value */
+
+    FUNC_ENTER_API(FAIL)
+    H5TRACE5("e", "ixxxx", plist_id, super, freelist, stab, shhdr);
+
+    /* Get the plist structure */
+    if(NULL == (plist = H5P_object_verify(plist_id,H5P_FILE_CREATE)))
+        HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "can't find object for ID")
+
+    /* Get values */
+    if(super)
+        if(H5P_get(plist, H5F_CRT_SUPER_VERS_NAME, super) < 0)
+            HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get superblock version")
+    if(freelist)
+        *freelist = HDF5_FREESPACE_VERSION;     /* (hard-wired) */
+    if(stab)
+        *stab = HDF5_OBJECTDIR_VERSION;         /* (hard-wired) */
+    if(shhdr)
+        *shhdr = HDF5_SHAREDHEADER_VERSION;     /* (hard-wired) */
+
+done:
+    FUNC_LEAVE_API(ret_value)
+} /* end H5Pget_version() */
+
+/*-------------------------------------------------------------------------
+ * Function:	H5Pset_file_space
+ *
+ * Purpose:	    It is mapped to H5Pset_file_space_strategy().
+ *
+ * Return:	    Non-negative on success/Negative on failure
+ *
+ * Programmer:	Vailin Choi; Jan 2017
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5Pset_file_space(hid_t plist_id, H5F_file_space_type_t strategy, hsize_t threshold)
+{
+
+    H5F_fspace_strategy_t new_strategy;                     /* File space strategy type */
+    hbool_t new_persist = H5F_FREE_SPACE_PERSIST_DEF;       /* Persisting free-space or not */
+    hsize_t new_threshold = H5F_FREE_SPACE_THRESHOLD_DEF;   /* Free-space section threshold */
+    H5F_file_space_type_t in_strategy = strategy;           /* Input strategy */
+    hsize_t in_threshold = threshold;                       /* Input threshold */
+    herr_t ret_value = SUCCEED;                             /* Return value */
+
+    FUNC_ENTER_API(FAIL)
+    H5TRACE3("e", "iFfh", plist_id, strategy, threshold);
+
+    if((unsigned)in_strategy >= H5F_FILE_SPACE_NTYPES)
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid strategy")
+    /*
+     *  For 1.10.0 H5Pset_file_space:
+     *      If strategy is zero, the property is not changed; 
+     *      the existing strategy is retained.
+     *      If threshold is zero, the property is not changed; 
+     *      the existing threshold is retained.
+     */
+    if(!in_strategy)
+        H5Pget_file_space(plist_id, &in_strategy, NULL);
+    if(!in_threshold)
+        H5Pget_file_space(plist_id, NULL, &in_threshold);
+
+    switch(in_strategy) {
+        case H5F_FILE_SPACE_ALL_PERSIST:
+            new_strategy = H5F_FSPACE_STRATEGY_FSM_AGGR;
+            new_persist = TRUE;
+            new_threshold = in_threshold;
+            break;
+    
+        case H5F_FILE_SPACE_ALL:
+            new_strategy = H5F_FSPACE_STRATEGY_FSM_AGGR;
+            new_threshold = in_threshold;
+            break;
+
+        case H5F_FILE_SPACE_AGGR_VFD:
+            new_strategy = H5F_FSPACE_STRATEGY_AGGR;
+            break;
+
+        case H5F_FILE_SPACE_VFD:
+            new_strategy = H5F_FSPACE_STRATEGY_NONE;
+            break;
+        
+        case H5F_FILE_SPACE_NTYPES:
+        case H5F_FILE_SPACE_DEFAULT:
+        default:
+            HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid file space strategy")
+    }
+
+    if(H5Pset_file_space_strategy(plist_id, new_strategy, new_persist, new_threshold) < 0)
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTSET, FAIL, "can't set file space strategy")
+
+done:
+    FUNC_LEAVE_API(ret_value)
+} /* H5Pset_file_space() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5Pget_file_space
+ *
+ * Purpose:	    It is mapped to H5Pget_file_space_strategy().
+ *
+ * Return:	    Non-negative on success/Negative on failure
+ *
+ * Programmer:	Vailin Choi; Jan 2017
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5Pget_file_space(hid_t plist_id, H5F_file_space_type_t *strategy, hsize_t *threshold)
+{
+    H5F_fspace_strategy_t new_strategy;     /* File space strategy type */
+    hbool_t new_persist;                    /* Persisting free-space or not */
+    hsize_t new_threshold;                  /* Free-space section threshold */
+    herr_t ret_value = SUCCEED;             /* Return value */
+
+    FUNC_ENTER_API(FAIL)
+    H5TRACE3("e", "i*Ff*h", plist_id, strategy, threshold);
+
+    /* Get current file space info */
+    if(H5Pget_file_space_strategy(plist_id, &new_strategy, &new_persist, &new_threshold) < 0)
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get file space strategy")
+
+    /* Get value(s) */
+    if(strategy) {
+        switch(new_strategy) {
+
+            case H5F_FSPACE_STRATEGY_FSM_AGGR:
+                if(new_persist)
+                    *strategy = H5F_FILE_SPACE_ALL_PERSIST;
+                else
+                    *strategy = H5F_FILE_SPACE_ALL;
+            break;
+
+            case H5F_FSPACE_STRATEGY_AGGR:
+                *strategy = H5F_FILE_SPACE_AGGR_VFD;
+                break;
+
+            case H5F_FSPACE_STRATEGY_NONE:
+                *strategy = H5F_FILE_SPACE_VFD;
+                break;
+
+            case H5F_FSPACE_STRATEGY_PAGE:
+            case H5F_FSPACE_STRATEGY_NTYPES:
+            default:
+                HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid file space strategy")
+        }
+    }
+
+    if(threshold)
+        *threshold = new_threshold;
+
+done:
+    FUNC_LEAVE_API(ret_value)
+} /* H5Pget_file_space() */
 #endif /* H5_NO_DEPRECATED_SYMBOLS */
 
