@@ -20,10 +20,7 @@
 /* Module Setup */
 /****************/
 
-#define H5T_PACKAGE		/*suppress error about including H5Tpkg	  */
-
-/* Interface initialization */
-#define H5_INTERFACE_INIT_FUNC	H5T_init_array_interface
+#include "H5Tmodule.h"          /* This source code file is part of the H5T module */
 
 
 /***********/
@@ -75,28 +72,6 @@
 /*******************/
 
 
-
-/*--------------------------------------------------------------------------
-NAME
-   H5T_init_array_interface -- Initialize interface-specific information
-USAGE
-    herr_t H5T_init_array_interface()
-
-RETURNS
-    Non-negative on success/Negative on failure
-DESCRIPTION
-    Initializes any interface-specific data or routines.  (Just calls
-    H5T_init_iterface currently).
-
---------------------------------------------------------------------------*/
-static herr_t
-H5T_init_array_interface(void)
-{
-    FUNC_ENTER_NOAPI_NOINIT_NOERR
-
-    FUNC_LEAVE_NOAPI(H5T_init())
-} /* H5T_init_array_interface() */
-
 
 /*-------------------------------------------------------------------------
  * Function:	H5Tarray_create2
@@ -177,8 +152,8 @@ done:
 H5T_t *
 H5T__array_create(H5T_t *base, unsigned ndims, const hsize_t dim[/* ndims */])
 {
-    H5T_t	*ret_value;	/* new array data type	*/
-    unsigned    u;              /* local index variable */
+    unsigned    u;                      /* Local index variable */
+    H5T_t	*ret_value = NULL;	/* New array data type	*/
 
     FUNC_ENTER_PACKAGE
 
