@@ -465,7 +465,7 @@ test_poly(const hid_t dxpl_id_polynomial)
     for(row = 0; row < ROWS; row++)
         for(col = 0; col < COLS; col++) {
             windchillC = (int) ((5.0f / 9.0f) * (windchillFfloat[row][col] - 32));
-            polyflres[row][col] = (float) ((2.0f + windchillC) * ((windchillC - 8.0f) / 2.0f));
+            polyflres[row][col] = ((2.0f + (float)windchillC) * (((float)windchillC - 8.0f) / 2.0f));
         }
 
     TESTING("data transform, polynomial transform (int->float)")
