@@ -18,12 +18,11 @@
  * Purpose:	Test unlinking operations.
  */
 
-#define H5G_PACKAGE		/*suppress error about including H5Gpkg	  */
+#define H5G_FRIEND		/*suppress error about including H5Gpkg	  */
 
 /* Define this macro to indicate that the testing APIs should be available */
 #define H5G_TESTING
 
-#include <time.h>
 #include "h5test.h"
 #include "H5Gpkg.h"		/* Groups				*/
 
@@ -2439,9 +2438,9 @@ int
 main(void)
 {
     hid_t	fapl, fapl2, file;
-    int	nerrors = 0;
+    int		nerrors = 0;
     char	filename[1024];
-    hbool_t new_format;
+    unsigned	new_format;
 
     /* Metadata cache parameters */
     int mdc_nelmts;
@@ -2552,7 +2551,7 @@ main(void)
 
     if (nerrors) {
         printf("***** %d FAILURE%s! *****\n", nerrors, 1==nerrors?"":"S");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     puts("All unlink tests passed.");
