@@ -234,7 +234,9 @@ extern int facc_type;				/*Test file access type */
 extern int dxfer_coll_type;
 
 /* Test program prototypes */
+void test_plist_ed(void);
 void zero_dim_dset(void);
+void test_file_properties(void);
 void multiple_dset_write(void);
 void multiple_group_write(void);
 void multiple_group_read(void);
@@ -242,6 +244,7 @@ void collective_group_write(void);
 void independent_group_read(void);
 void test_fapl_mpio_dup(void);
 void test_split_comm_access(void);
+void test_page_buffer_access(void);
 void dataset_atomicity(void);
 void dataset_writeInd(void);
 void dataset_writeAll(void);
