@@ -26,7 +26,7 @@
  *             2) when diff was found (normal mode)
  *-------------------------------------------------------------------------
  */
- int
+H5_ATTR_PURE int
 print_objname (diff_opt_t * opts, hsize_t nfound)
 {
     return ((opts->m_verbose || nfound) && !opts->m_quiet) ? 1 : 0;
