@@ -43,7 +43,7 @@ unsigned    packed_bits_num;    /* number of packed bits to display */
 unsigned    packed_data_offset; /* offset of packed bits to display */
 unsigned    packed_data_length; /* length of packed bits to display */
 unsigned long long packed_data_mask;  /* mask in which packed bits to display */
-int          enable_error_stack= FALSE; /* re-enable error stack */
+int         enable_error_stack = 0;   /* re-enable error stack; disable=0 enable=1 */
 
 /* sort parameters */
 H5_index_t   sort_by           = H5_INDEX_NAME; /*sort_by [creation_order | name]  */
@@ -548,9 +548,15 @@ h5tools_fopen(const char *fname, unsigned flags, hid_t fapl, const char *driver,
         if ((my_fapl = h5tools_get_fapl(fapl, driver, &drivernum)) < 0)
             goto done;
 
-        H5E_BEGIN_TRY {
+        /* allow error stack display if enable-error-stack has optional arg number */
+        if (enable_error_stack > 1) {
             fid = H5Fopen(fname, flags, my_fapl);
-        } H5E_END_TRY;
+        }
+        else {
+            H5E_BEGIN_TRY {
+                fid = H5Fopen(fname, flags, my_fapl);
+            } H5E_END_TRY;
+        }
 
         if (fid == FAIL)
             goto done;
@@ -563,9 +569,15 @@ h5tools_fopen(const char *fname, unsigned flags, hid_t fapl, const char *driver,
             if((my_fapl = h5tools_get_fapl(fapl, drivernames[drivernum], NULL)) < 0)
                 goto done;
 
-            H5E_BEGIN_TRY {
+            /* allow error stack display if enable-error-stack has optional arg number */
+            if (enable_error_stack > 1) {
                 fid = H5Fopen(fname, flags, my_fapl);
-            } H5E_END_TRY;
+            }
+            else {
+                H5E_BEGIN_TRY {
+                    fid = H5Fopen(fname, flags, my_fapl);
+                } H5E_END_TRY;
+            }
 
             if (fid != FAIL)
                 break;
@@ -606,7 +618,7 @@ done:
  *           failure - 0.
  *-------------------------------------------------------------------------
  */
-static size_t
+H5_ATTR_PURE static size_t
 h5tools_count_ncols(const char *s)
 {
     register size_t i;
@@ -1386,7 +1398,7 @@ render_bin_output(FILE *stream, hid_t container, hid_t tid, void *_mem,  hsize_t
 
                         for (block_index = 0; block_index < block_nelmts; block_index++) {
                             mem = ((unsigned char*)_mem) + block_index * size;
-                            region_id = H5Rdereference(container, H5R_DATASET_REGION, mem);
+                            region_id = H5Rdereference2(container, H5P_DEFAULT, H5R_DATASET_REGION, mem);
                             if (region_id >= 0) {
                                 region_space = H5Rget_region(container, H5R_DATASET_REGION, mem);
                                 if (region_space >= 0) {
@@ -1709,7 +1721,7 @@ CATCH
  *           FALSE otherwise
  *-------------------------------------------------------------------------
  */
-hbool_t
+H5_ATTR_PURE hbool_t
 h5tools_is_zero(const void *_mem, size_t size)
 {
     const unsigned char *mem = (const unsigned char *) _mem;
