@@ -458,7 +458,7 @@ dump_tables(find_objs_t *info)
  *              Failure:    FAIL   if object is not found
  *-------------------------------------------------------------------------
  */
- obj_t *
+H5_ATTR_PURE obj_t *
 search_obj(table_t *table, haddr_t objno)
 {
     unsigned u;
@@ -482,8 +482,7 @@ search_obj(table_t *table, haddr_t objno)
  *-------------------------------------------------------------------------
  */
 static herr_t
-find_objs_cb(const char *name, const H5O_info_t *oinfo, const char *already_seen,
-    void *op_data)
+find_objs_cb(const char *name, const H5O_info_t *oinfo, const char *already_seen, void *op_data)
 {
     find_objs_t *info = (find_objs_t*)op_data;
     herr_t ret_value = 0;
@@ -784,13 +783,13 @@ h5tools_setstatus(int D_status)
     h5tools_d_status = D_status;
 }
 
-const char *
+H5_ATTR_PURE const char *
 h5tools_getprogname(void)
 {
    return h5tools_progname;
 }
 
-int
+H5_ATTR_PURE int
 h5tools_getstatus(void)
 {
    return h5tools_d_status;
