@@ -148,7 +148,7 @@ trav_addr_add(trav_addr_t *visited, haddr_t addr, const char *path)
  * Return:   TRUE/FALSE
  *-------------------------------------------------------------------------
  */
-static const char *
+H5_ATTR_PURE static const char *
 trav_addr_visited(trav_addr_t *visited, haddr_t addr)
 {
     size_t u;           /* Local index variable */
@@ -452,7 +452,7 @@ done:
  *           -1 if not found
  *-------------------------------------------------------------------------
  */
-ssize_t
+H5_ATTR_PURE ssize_t
 h5trav_getindex(const trav_info_t *info, const char *obj)
 {
     size_t u;           /* Local index variable */
@@ -618,7 +618,7 @@ done:
  *           -1 if not found
  *-------------------------------------------------------------------------
  */
-int
+H5_ATTR_PURE int
 h5trav_getindext(const char *name, const trav_table_t *table)
 {
     unsigned int i;
@@ -1124,7 +1124,7 @@ done:
  * Return:   TRUE/FALSE
  *-------------------------------------------------------------------------
  */
-hbool_t
+H5_ATTR_PURE hbool_t
 symlink_is_visited(symlink_trav_t *visited, H5L_type_t type, const char *file, const char *path)
 {
     size_t u;  /* Local index variable */
